fignum = 1;
close all
addpath('Functions')
%% results
transformation = ''; % 'asinh' or ''
dynamic_choice = 0; % 1 or 0
mc = 0;
distribution = 'g';
model = '7';
hour = 10; % hour for analysis
GARCH_type = 'GJRGARCH';
market = 'DA';
% loading data
load(['../../../Data_MATLAB_Drive/Data/', GARCH_type, '_simulation_results_PL_', market, '_model_', model,...
        '_transformation_', transformation,...
        '_dynamic_choice_', num2str(dynamic_choice),...
        '_mc_', num2str(mc),'_dist_', distribution, '.mat'],...
        'real_value', 'prediction', 'residua', 'mae_hourly', 'rmse_hourly',...
            'garch_coefficients', 'con_var', 'quantiles', 'quantiles_no_garch',...
            'pinball', 'pinball_no_garch', 'archtest_results', 'aic_matrice',...
            'bic_matrice', 'kstest_results', 'kstesttstud_results',...
            'price_mu_vector', 'price_sigma_vector');
%% GARCH coefficients boxplots for chosen hour - nie mam pomysłu jak to napisać ogólnie (dla dynamic), jak będzie potrzeba to napiszę
% hour = 1; % hour for analysis
% % box plot of GARCH coefficients
% figure(fignum)
% titles = ["Constant", "ARCH", "GARCH"];
% for i = 1:3
%     subplot(1,3,i)
%     boxplot(garch_coefficients(hour, :, i))
%     title(titles(i))
% end
% fignum = fignum + 1;

%% box plot of conditional variance predictions
figure(fignum)
boxplot(con_var(hour, :))
title("Conditional variance prediction")
fignum = fignum + 1;

%% pinball and quantiles comparison
disp('mean of pinball:')
disp(squeeze(mean(pinball(hour, :, :), 2)))
disp('% of observations that are below given quantiles:')
disp(squeeze(sum(quantiles(hour, :, :) < real_value(hour, :), 2) / length(prediction)))

%% Data view
figure(fignum)
subplot(2,1,1)
plot(real_value(hour,:))
hold on
plot(prediction(hour,:))
plot(quantiles(hour, :, 1), '--')
plot(quantiles(hour, :, end), '--')
plot(quantiles_no_garch(hour, :, 1), '--')
plot(quantiles_no_garch(hour, :, end), '--')
hold off
title("Data, prediction and 5 and 95 percentiles")
legend("Data", "Prediction", "0.05 GARCH", "0.95 GARCH", "0.05 residuals variance", "0.95 residuals variance")
subplot(2,1,2)
plot(con_var(hour,:))
title("Conditional variance")
fignum = fignum + 1;

%% ACF PACF plots of first calibration window residuals
for i = 1:24
    figure(fignum)
    subplot(2,1,1)
    autocorr(residua(i, :, 1) .^ 2, 'NumLags',10)
    title(['ACF for hour ', num2str(i)])
    subplot(2,1,2)
    parcorr(residua(i, :, 1) .^2, 'NumLags',10)
    title(['PACF for hour ', num2str(i)])
    fignum = fignum + 1;
end

%% Searching for zero parameters
% for hour = 1:24
%     figure(fignum)
%     titles = ["Constant", "ARCH", "GARCH"];
%     for i = 1:3
%         subplot(1,3,i)
%         boxplot(garch_coefficients(hour, :, i))
%     end
%     sgtitle(['Boxplot for hour ', num2str(hour)])
%     fignum = fignum + 1;
% end

%% AIC BIC criteria outcome
ind = 1:9;
figure(fignum)
for hour = 1:24
    subplot(2,1,1)
    hold on
    plot(hour, ind(mean(bic_matrice(hour,:,:), 2) == min(mean(bic_matrice(hour,:,:), 2))), 'o', 'Color', 'b')
    hold off
    subplot(2,1,2)
    hold on
    plot(hour, ind(mean(aic_matrice(hour,:,:), 2) == min(mean(aic_matrice(hour,:,:), 2))), 'o', 'Color', 'b')
    hold off
end
subplot(2,1,1)
grid on
xticks(1:24)
xlabel("hours")
permutations = permn([1,2,7], 2);
set(gca,'ytick',ind,'yticklabel',num2str(permutations(1:end,1:2)))
ylabel("[p,q] pairs")
title("BIC")
subplot(2,1,2)
grid on
xticks(1:24)
xlabel("hours")
permutations = permn([1,2,7], 2);
set(gca,'ytick',ind,'yticklabel',num2str(permutations(1:end,1:2)))
ylabel("[p,q] pairs")
title("AIC")
fignum = fignum + 1;

%% kstest results - 0 means that the test fails to reject hypothesis of normality
disp("kstest - fraction of h = 1 results:")
disp([[1:24]', sum(kstest_results(:, :, 1), 2) / length(kstest_results(:, :, 1))])
startDate = datenum('01-01-2017');
endDate = datenum('12-31-2020');
xData = linspace(startDate, endDate, 1461);
for hour = 1:24
    figure(fignum)
    plot(xData(374:end), kstest_results(hour, :, 2))
    datetick('x','mm-yyyy','keeplimits', 'keepticks')
    ylim([0, 1])
    title(['P-value for hour ', num2str(hour)])
    fignum = fignum + 1;
end
%% kstest for t-Student results - 0 means that the test fails to reject hypothesis that sample has t-Student distribution with location and scale
disp("kstest t-Student, fitted - fraction of h = 1 results:")
disp([[1:24]', sum(kstesttstud_results(:, :, 1), 2) / length(kstesttstud_results(:, :, 1))])
startDate = datenum('01-01-2017');
endDate = datenum('12-31-2020');
xData = linspace(startDate, endDate, 1461);
for hour = 1:24
    figure(fignum)
    plot(xData(374:end), kstesttstud_results(hour, :, 2))
    datetick('x','mm-yyyy','keeplimits', 'keepticks')
    ylim([0, 1])
    title(['P-value for hour ', num2str(hour)])
    fignum = fignum + 1;
end
%% archtest results - doc: "h = 1 indicates that you should reject null hypothesis of no conditional heteroscedasticity and conclude that there are significant ARCH effects"
disp("archtest - fraction of h = 1 results:")
no_asinh = sum(archtest_results(:, :, 1), 2);
disp([[1:24]', sum(archtest_results(:, :, 1), 2) / length(archtest_results(:, :, 1))])
startDate = datenum('01-01-2017');
endDate = datenum('12-31-2020');
xData = linspace(startDate, endDate, 1461);
for hour = 1:24
    figure(fignum)
    plot(xData(374:end), archtest_results(hour, :, 2))
    datetick('x','mm-yyyy','keeplimits', 'keepticks')
    ylim([0, 1])
    title(['P-value for hour ', num2str(hour)])
    fignum = fignum + 1;
end
