% File contains more specific results. GJR and models with no transformation were
% rejected based on analysis from Pinball_comparison_GE file.
%% DA
% constants
alpha = 0.1; % alpha for D--M tests
fignum = 1;
quantiles = 0.01:0.01:0.99;
% loading simulation data
DA_0_asinh1_0_g = load('../../../Data_MATLAB_Drive/Data/GARCH_simulation_results_GE_DA_model_7_transformation_asinh_dynamic_choice_0_mc_0_dist_g.mat',...
    'pinball', 'pinball_no_garch', 'mae_hourly', 'rmse_hourly', 'real_value',  'prediction');
DA_0_asinh1_1_g = load('../../../Data_MATLAB_Drive/Data/GARCH_simulation_results_GE_DA_model_7_transformation_asinh_dynamic_choice_0_mc_1_dist_g.mat',...
    'pinball', 'pinball_no_garch', 'mae_hourly', 'rmse_hourly');
DA_0_asinh1_0_t = load('../../../Data_MATLAB_Drive/Data/GARCH_simulation_results_GE_DA_model_7_transformation_asinh_dynamic_choice_0_mc_0_dist_t.mat',...
    'pinball', 'pinball_no_garch', 'mae_hourly', 'rmse_hourly');
DA_0_asinh1_1_t = load('../../../Data_MATLAB_Drive/Data/GARCH_simulation_results_GE_DA_model_7_transformation_asinh_dynamic_choice_0_mc_1_dist_t.mat',...
    'pinball', 'pinball_no_garch', 'mae_hourly', 'rmse_hourly');
% collecting models PS and their labels in a list
models_list = {DA_0_asinh1_0_g.pinball, DA_0_asinh1_1_g.pinball,...
    DA_0_asinh1_0_t.pinball, DA_0_asinh1_1_t.pinball};
labels = {'gaussian GARCH(1,1) hist. asinh1', 'gaussian GARCH(1,1) MC asinh1',...
    't-Student GARCH(1,1) hist. asinh1', 't-Student GARCH(1,1) MC asinh1'};
% PS D--M tests aggregated
close all
addpath('Functions')
% summing over 24 hours
count_matrice = zeros([length(models_list), length(models_list)]);
for i = 1:length(models_list)
    for j = 1:length(models_list)
        if i == j
            count_matrice(i,j) = NaN;
        end
    end
end
for hour = 1:24
    pvalmatyrice = ones([length(models_list), length(models_list)]);
    for m = 1:length(models_list)
        for n = 1:length(models_list)
            [DM, p] = dmtest(mean(models_list{m}(hour, :, :), 3)', mean(models_list{n}(hour, :, :), 3)');
            pvalmatyrice(n, m) = p; % model on y is worse than model on x if p-value < alpha (alpha = 0.1)
        end
    end
    count = pvalmatyrice < alpha;
    count_matrice = count_matrice + count;
end
xvalues = labels;
yvalues = labels;
figure(fignum)
h = heatmap(xvalues, yvalues, count_matrice);
h.FontSize = 6;
caxis([0 24])
title('PS D--M hourly aggregated test')
fignum = fignum + 1;
% PS D--M test for chosen quantiles
for q = [5, 10, 25, 50, 75, 90, 95]
    % summing over 24 hours
    count_matrice = zeros([length(models_list), length(models_list)]);
    for i = 1:length(models_list)
        for j = 1:length(models_list)
            if i == j
                count_matrice(i,j) = NaN;
            end
        end
    end
    for hour = 1:24
        pvalmatyrice = ones([length(models_list), length(models_list)]);
        for m = 1:length(models_list)
            for n = 1:length(models_list)
                [DM, p] = dmtest(models_list{m}(hour, :, q)', models_list{n}(hour, :, q)');
                pvalmatyrice(n, m) = p; % model on y is worse than model on x if p-value < alpha (alpha = 0.1)
            end
        end
        count = pvalmatyrice < alpha;
        count_matrice = count_matrice + count;
    end
    xvalues = labels;
    yvalues = labels;
    figure(fignum)
    h = heatmap(xvalues, yvalues, count_matrice);
    h.FontSize = 6;
    caxis([0 24])
    fignum = fignum + 1;
    title(['PS D--M hourly aggregated test for quantile ', num2str(quantiles(q))])
    fignum = fignum + 1;
end
% quantile wise aggregated D--M test
figure(fignum)
count_matrice = zeros([length(models_list), length(models_list)]);
for i = 1:length(models_list)
    for j = 1:length(models_list)
        if i == j
            count_matrice(i,j) = NaN;
        end
    end
end
for quantile = 1:99
    pvalmatyrice = ones([length(models_list), length(models_list)]);
    for m = 1:length(models_list)
        for n = 1:length(models_list)
            [DM, p] = dmtest(mean(models_list{m}(:, :, quantile), 1)', mean(models_list{n}(:, :, quantile), 1)');
            pvalmatyrice(n, m) = p; % model on y is worse than model on x if p-value < alpha 
        end
    end
    count = pvalmatyrice < alpha;
    count_matrice = count_matrice + count;
end
xvalues = labels;
yvalues = labels;
h = heatmap(xvalues, yvalues, count_matrice);
h.FontSize = 6;
caxis([0 99])
title('D--M test quantile wise aggregated test')
fignum = fignum + 1;
% quantile wise aggregated D--M test represented hourly
for hour = 1:24
    figure(fignum)
    count_matrice = zeros([length(models_list), length(models_list)]);
    for i = 1:length(models_list)
        for j = 1:length(models_list)
            if i == j
                count_matrice(i,j) = NaN;
            end
        end
    end
    for quantile = 1:99
        pvalmatyrice = ones([length(models_list), length(models_list)]);
        for m = 1:length(models_list)
            for n = 1:length(models_list)
                [DM, p] = dmtest(models_list{m}(hour, :, quantile)', models_list{n}(hour, :, quantile)');
                pvalmatyrice(n, m) = p; % model on y is worse than model on x if p-value < alpha 
            end
        end
        count = pvalmatyrice < alpha;
        count_matrice = count_matrice + count;
    end
    xvalues = labels;
    yvalues = labels;
    h = heatmap(xvalues, yvalues, count_matrice);
    h.FontSize = 6;
    caxis([0 99])
    title(['Hour ', num2str(hour)])
    fignum = fignum + 1;
end
%% ID
% constants
alpha = 0.1; % alpha for D--M tests
fignum = 1;
quantiles = 0.01:0.01:0.99;
% loading simulation data
ID_0_asinh1_0_g = load('../../../Data_MATLAB_Drive/Data/GARCH_simulation_results_GE_ID_model_7_transformation_asinh_dynamic_choice_0_mc_0_dist_g.mat',...
    'pinball', 'pinball_no_garch', 'mae_hourly', 'rmse_hourly', 'real_value',  'prediction');
ID_0_asinh1_1_g = load('../../../Data_MATLAB_Drive/Data/GARCH_simulation_results_GE_ID_model_7_transformation_asinh_dynamic_choice_0_mc_1_dist_g.mat',...
    'pinball', 'pinball_no_garch', 'mae_hourly', 'rmse_hourly', 'prediction', 'real_value');
ID_0_asinh1_0_t = load('../../../Data_MATLAB_Drive/Data/GARCH_simulation_results_GE_ID_model_7_transformation_asinh_dynamic_choice_0_mc_0_dist_t.mat',...
    'pinball', 'pinball_no_garch', 'mae_hourly', 'rmse_hourly', 'prediction', 'real_value');
ID_0_asinh1_1_t = load('../../../Data_MATLAB_Drive/Data/GARCH_simulation_results_GE_ID_model_7_transformation_asinh_dynamic_choice_0_mc_1_dist_t.mat',...
    'pinball', 'pinball_no_garch', 'mae_hourly', 'rmse_hourly', 'prediction', 'real_value');
% collecting models PS and their labels in a list
models_list = {ID_0_asinh1_0_g.pinball, ID_0_asinh1_1_g.pinball,...
    ID_0_asinh1_0_t.pinball, ID_0_asinh1_1_t.pinball};
labels = {'gaussian GARCH(1,1) hist. asinh1', 'gaussian GARCH(1,1) MC asinh1',...
    't-Student GARCH(1,1) hist. asinh1', 't-Student GARCH(1,1) MC asinh1'};
% PS D--M tests aggregated
close all
addpath('Functions')
% summing D--M results over 24 hours
count_matrice = zeros([length(models_list), length(models_list)]);
for i = 1:length(models_list)
    for j = 1:length(models_list)
        if i == j
            count_matrice(i,j) = NaN;
        end
    end
end
for hour = 1:24
    pvalmatyrice = ones([length(models_list), length(models_list)]);
    for m = 1:length(models_list)
        for n = 1:length(models_list)
            [DM, p] = dmtest(mean(models_list{m}(hour, :, :), 3)', mean(models_list{n}(hour, :, :), 3)');
            pvalmatyrice(n, m) = p; % model on y is worse than model on x if p-value < alpha (alpha = 0.1)
        end
    end
    count = pvalmatyrice < alpha;
    count_matrice = count_matrice + count;
end
xvalues = labels;
yvalues = labels;
figure(fignum)
h = heatmap(xvalues, yvalues, count_matrice);
h.FontSize = 6;
caxis([0 24])
title('PS D--M hourly aggregated test')
fignum = fignum + 1;
% PS D--M test for chosen quantiles
for q = [5, 10, 25, 50, 75, 90, 95]
    % summing over 24 hours
    count_matrice = zeros([length(models_list), length(models_list)]);
    for i = 1:length(models_list)
        for j = 1:length(models_list)
            if i == j
                count_matrice(i,j) = NaN;
            end
        end
    end
    for hour = 1:24
        pvalmatyrice = ones([length(models_list), length(models_list)]);
        for m = 1:length(models_list)
            for n = 1:length(models_list)
                [DM, p] = dmtest(models_list{m}(hour, :, q)', models_list{n}(hour, :, q)');
                pvalmatyrice(n, m) = p; % model on y is worse than model on x if p-value < alpha (alpha = 0.1)
            end
        end
        count = pvalmatyrice < alpha;
        count_matrice = count_matrice + count;
    end
    xvalues = labels;
    yvalues = labels;
    figure(fignum)
    h = heatmap(xvalues, yvalues, count_matrice);
    h.FontSize = 6;
    caxis([0 24])
    fignum = fignum + 1;
    title(['PS D--M hourly aggregated test for quantile ', num2str(quantiles(q))])
    fignum = fignum + 1;
end
% quantile wise aggregated D--M test
figure(fignum)
count_matrice = zeros([length(models_list), length(models_list)]);
for i = 1:length(models_list)
    for j = 1:length(models_list)
        if i == j
            count_matrice(i,j) = NaN;
        end
    end
end
for quantile = 1:99
    pvalmatyrice = ones([length(models_list), length(models_list)]);
    for m = 1:length(models_list)
        for n = 1:length(models_list)
            [DM, p] = dmtest(mean(models_list{m}(:, :, quantile), 1)', mean(models_list{n}(:, :, quantile), 1)');
            pvalmatyrice(n, m) = p; % model on y is worse than model on x if p-value < alpha 
        end
    end
    count = pvalmatyrice < alpha;
    count_matrice = count_matrice + count;
end
xvalues = labels;
yvalues = labels;
h = heatmap(xvalues, yvalues, count_matrice);
h.FontSize = 6;
caxis([0 99])
title('D--M quantile wise aggregated test')
fignum = fignum + 1;
% quantile wise aggregated D--M test represented hourly
for hour = 1:24
    figure(fignum)
    count_matrice = zeros([length(models_list), length(models_list)]);
    for i = 1:length(models_list)
        for j = 1:length(models_list)
            if i == j
                count_matrice(i,j) = NaN;
            end
        end
    end
    for quantile = 1:99
        pvalmatyrice = ones([length(models_list), length(models_list)]);
        for m = 1:length(models_list)
            for n = 1:length(models_list)
                [DM, p] = dmtest(models_list{m}(hour, :, quantile)', models_list{n}(hour, :, quantile)');
                pvalmatyrice(n, m) = p; % model on y is worse than model on x if p-value < alpha 
            end
        end
        count = pvalmatyrice < alpha;
        count_matrice = count_matrice + count;
    end
    xvalues = labels;
    yvalues = labels;
    h = heatmap(xvalues, yvalues, count_matrice);
    h.FontSize = 6;
    caxis([0 99])
    title(['D--M quantile wise aggregated test for hour ', num2str(hour)])
    fignum = fignum + 1;
end
