% Simulation took approx. 16h on a single machine (4 cores, 4 workers)
% GARCH
close all
clear all
addpath('Functions')
warning('off', 'all') % to suppress warnings
data = importdata('../Data/data_Germany_20170101_20201231.txt');
%% parameters declaration
start = 373 + 365; % start of eval window (in days)
stop = size(data, 1) / 24; % end of eval window (in days)
training_window = 365; % training window length
model = '7'; % ARX models from 1 to 9
hours = 1:24; % gives control over hours that we want to simulate on
p = 1; % GARCH parameters p and q
q = 1;
GARCH_type = 'GARCH';
N_mc = 365; % monte carlo steps for probability forecast
quantiles_choice = 0.01:0.01:0.99;
dynamic_choice = 0;
inf_crit_models = [1, 2, 7]; % GARCH orders to test (all 2 element [p, q] sets from inf_crit_models)
transformations = 2;
monte_carlo = 1;
mc_and_garch_distribution = 1;
% GERMAN DA - all simulations (for all combinations of dynamic choice or GARCH(1,1), MC or hist, transformation or no transformation
data_german = importdata('../Data/data_Germany_20170101_20201231.txt');
for dyn_choice = 0:dynamic_choice
    for trans = 0:transformations
        for mc = 1:monte_carlo
            for dist = 1:mc_and_garch_distribution
                if trans == 0
                    transformation = '';
                elseif trans == 1
                    transformation = 'asinh1';
                elseif trans == 2
                    transformation = 'asinh2';
                end
                if dist == 0
                    distribution = 'g';
                elseif dist == 1
                    distribution = 't';
                end
                disp(['DA simulation for ', num2str(dyn_choice), transformation, num2str(mc), distribution, 'GERMAN DA'])
                tic;
                [real_value, prediction, residua, samples, samples_no_garch, mae_hourly, rmse_hourly,...
                            garch_coefficients, con_var, quantiles, quantiles_no_garch,...
                            pinball, pinball_no_garch, archtest_results, aic_matrice,...
                            bic_matrice, kstest_results, kstesttstud_results,...
                            price_mu_vector, price_sigma_vector, prediction_transformed] = ...
                    OOS_predict_ARX_GARCH_DA_prices_parallel_GE(start, stop, training_window,...
                                                data_german, model, hours,...
                                                GARCH_type, p, q,...
                                                N_mc, quantiles_choice,...
                                                dyn_choice, inf_crit_models,...
                                                transformation, mc,...
                                                distribution);
                toc
                beep
                if length(hours) == 24
                    save(['../../../Data_MATLAB_Drive/Data/', GARCH_type, '_OOS_simulation_results_GE_DA_', 'model_', model,...
                        '_transformation_', transformation,...
                        '_dynamic_choice_', num2str(dyn_choice),...
                        '_mc_', num2str(mc),'_dist_', distribution, '.mat'],...
                        'real_value', 'prediction', 'residua', 'samples', 'samples_no_garch', 'mae_hourly', 'rmse_hourly',...
                            'garch_coefficients', 'con_var', 'quantiles', 'quantiles_no_garch',...
                            'pinball', 'pinball_no_garch', 'archtest_results', 'aic_matrice',...
                            'bic_matrice', 'kstest_results', 'kstesttstud_results',...
                            'price_mu_vector', 'price_sigma_vector', 'prediction_transformed');
                else
                    disp("Results for single hour were not saved.")
                end
            end
        end
    end
end
% GERMAN ID market - all simulations
data_german = importdata('../Data/data_Germany_20170101_20201231.txt');
for dyn_choice = 0:dynamic_choice
    for trans = 0:transformations
        for mc = 1:monte_carlo
            for dist = 1:mc_and_garch_distribution
                if trans == 0
                    transformation = '';
                elseif trans == 1
                    transformation = 'asinh1';
                elseif trans == 2
                    transformation = 'asinh2';
                end
                if dist == 0
                    distribution = 'g';
                elseif dist == 1
                    distribution = 't';
                end
                disp(['ID simulation for ', num2str(dyn_choice), transformation, num2str(mc), distribution, 'GERMAN ID'])
                tic;
                [real_value, prediction, residua, samples, samples_no_garch, mae_hourly, rmse_hourly,...
                            garch_coefficients, con_var, quantiles, quantiles_no_garch,...
                            pinball, pinball_no_garch, archtest_results, aic_matrice,...
                            bic_matrice, kstest_results, kstesttstud_results,...
                            price_mu_vector, price_sigma_vector, prediction_transformed] = ...
                    OOS_predict_ARX_GARCH_ID_prices_parallel_GE(start, stop, training_window,...
                                                data_german, model, hours,...
                                                GARCH_type, p, q,...
                                                N_mc, quantiles_choice,...
                                                dyn_choice, inf_crit_models,...
                                                transformation, mc,...
                                                distribution);
                toc
                beep
                if length(hours) == 24
                    save(['../../../Data_MATLAB_Drive/Data/', GARCH_type, '_OOS_simulation_results_GE_ID_', 'model_', model,...
                        '_transformation_', transformation,...
                        '_dynamic_choice_', num2str(dyn_choice),...
                        '_mc_', num2str(mc),'_dist_', distribution, '.mat'],...
                        'real_value', 'prediction', 'residua', 'samples', 'samples_no_garch', 'mae_hourly', 'rmse_hourly',...
                            'garch_coefficients', 'con_var', 'quantiles', 'quantiles_no_garch',...
                            'pinball', 'pinball_no_garch', 'archtest_results', 'aic_matrice',...
                            'bic_matrice', 'kstest_results', 'kstesttstud_results',...
                            'price_mu_vector', 'price_sigma_vector', 'prediction_transformed');
                else
                    disp("Results for single hour were not saved.")
                end
            end
        end
    end
end
% POLISH DA - all simulations (for all combinations of dynamic choice or GARCH(1,1), MC or hist, transformation or no transformation
data_polish = importdata('../Data/data_Poland_20170101_20201231.txt');
for dyn_choice = 0:dynamic_choice
    for trans = 0:transformations
        for mc = 1:monte_carlo
            for dist = 1:mc_and_garch_distribution
                if trans == 0
                    transformation = '';
                elseif trans == 1
                    transformation = 'asinh1';
                elseif trans == 2
                    transformation = 'asinh2';
                end
                if dist == 0
                    distribution = 'g';
                elseif dist == 1
                    distribution = 't';
                end
                disp(['DA simulation for ', num2str(dyn_choice), transformation, num2str(mc), distribution, 'POLISH DA'])
                tic;
                [real_value, prediction, residua, samples, samples_no_garch, mae_hourly, rmse_hourly,...
                            garch_coefficients, con_var, quantiles, quantiles_no_garch,...
                            pinball, pinball_no_garch, archtest_results, aic_matrice,...
                            bic_matrice, kstest_results, kstesttstud_results,...
                            price_mu_vector, price_sigma_vector, prediction_transformed] = ...
                    OOS_predict_ARX_GARCH_DA_prices_parallel_PL(start, stop, training_window,...
                                                data_polish, model, hours,...
                                                GARCH_type, p, q,...
                                                N_mc, quantiles_choice,...
                                                dyn_choice, inf_crit_models,...
                                                transformation, mc,...
                                                distribution);
                toc
                beep
                if length(hours) == 24
                    save(['../../../Data_MATLAB_Drive/Data/', GARCH_type, '_OOS_simulation_results_PL_DA_', 'model_', model,...
                        '_transformation_', transformation,...
                        '_dynamic_choice_', num2str(dyn_choice),...
                        '_mc_', num2str(mc),'_dist_', distribution, '.mat'],...
                        'real_value', 'prediction', 'residua', 'samples', 'samples_no_garch', 'mae_hourly', 'rmse_hourly',...
                            'garch_coefficients', 'con_var', 'quantiles', 'quantiles_no_garch',...
                            'pinball', 'pinball_no_garch', 'archtest_results', 'aic_matrice',...
                            'bic_matrice', 'kstest_results', 'kstesttstud_results',...
                            'price_mu_vector', 'price_sigma_vector', 'prediction_transformed');
                else
                    disp("Results for single hour were not saved.")
                end
            end
        end
    end
end
% POLISH ID market - all simulations
data_polish = importdata('../Data/data_Poland_20170101_20201231.txt');
for dyn_choice = 0:dynamic_choice
    for trans = 0:transformations
        for mc = 1:monte_carlo
            for dist = 1:mc_and_garch_distribution
                if trans == 0
                    transformation = '';
                elseif trans == 1
                    transformation = 'asinh1';
                elseif trans == 2
                    transformation = 'asinh2';
                end  
                if dist == 0
                    distribution = 'g';
                elseif dist == 1
                    distribution = 't';
                end
                disp(['ID simulation for ', num2str(dyn_choice), transformation, num2str(mc), distribution, 'POLISH ID'])
                tic;
                [real_value, prediction, residua, samples, samples_no_garch, mae_hourly, rmse_hourly,...
                            garch_coefficients, con_var, quantiles, quantiles_no_garch,...
                            pinball, pinball_no_garch, archtest_results, aic_matrice,...
                            bic_matrice, kstest_results, kstesttstud_results,...
                            price_mu_vector, price_sigma_vector, prediction_transformed] = ...
                    OOS_predict_ARX_GARCH_ID_prices_parallel_PL(start, stop, training_window,...
                                                data_polish, model, hours,...
                                                GARCH_type, p, q,...
                                                 N_mc, quantiles_choice,...
                                                dyn_choice, inf_crit_models,...
                                                transformation, mc,...
                                                distribution);
                toc
                beep
                if length(hours) == 24
                    save(['../../../Data_MATLAB_Drive/Data/', GARCH_type, '_OOS_simulation_results_PL_ID_', 'model_', model,...
                        '_transformation_', transformation,...
                        '_dynamic_choice_', num2str(dyn_choice),...
                        '_mc_', num2str(mc),'_dist_', distribution, '.mat'],...
                        'real_value', 'prediction', 'residua', 'samples', 'samples_no_garch', 'mae_hourly', 'rmse_hourly',...
                            'garch_coefficients', 'con_var', 'quantiles', 'quantiles_no_garch',...
                            'pinball', 'pinball_no_garch', 'archtest_results', 'aic_matrice',...
                            'bic_matrice', 'kstest_results', 'kstesttstud_results',...
                            'price_mu_vector', 'price_sigma_vector', 'prediction_transformed');
                else
                    disp("Results for single hour were not saved.")
                end
            end
        end
    end
end
%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% GJRGARCH
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
warning('off', 'all') % to suppress warnings
%% parameters declaration
start = 373 + 365; % start of eval window (in days)
stop = size(data, 1) / 24; % end of eval window (in days)
training_window = 365; % training window length
model = '7'; % ARX models from 1 to 9
hours = 1:24; % gives control over hours that we want to simulate on
p = 1; % GARCH parameters p and q
q = 1;
GARCH_type = 'GJRGARCH';
N_mc = 365; % monte carlo steps for probability forecast
quantiles_choice = 0.01:0.01:0.99;
dynamic_choice = 0;
inf_crit_models = [1, 2, 7]; % GARCH orders to test (all 2 element [p, q] sets from inf_crit_models)
transformations = 2;
monte_carlo = 1;
mc_and_garch_distribution = 1;
% GERMAN DA - all simulations (for all combinations of dynamic choice or GARCH(1,1), MC or hist, transformation or no transformation
% data_german = importdata('../Data/data_Germany_20170101_20201231.txt');
% for dyn_choice = 0:dynamic_choice
%     for trans = 0:transformations
%         for mc = 0:monte_carlo
%             for dist = 0:mc_and_garch_distribution
%                 if trans == 0
%                     transformation = '';
%                 elseif trans == 1
%                     transformation = 'asinh1';
%                 elseif trans == 2
%                     transformation = 'asinh2';
%                 end
%                 if dist == 0
%                     distribution = 'g';
%                 elseif dist == 1
%                     distribution = 't';
%                 end
%                 disp(['DA simulation for ', num2str(dyn_choice), transformation, num2str(mc), distribution, 'GERMAN DA'])
%                 tic;
%                 [real_value, prediction, residua, samples, samples_no_garch, mae_hourly, rmse_hourly,...
%                             garch_coefficients, con_var, quantiles, quantiles_no_garch,...
%                             pinball, pinball_no_garch, archtest_results, aic_matrice,...
%                             bic_matrice, kstest_results, kstesttstud_results,...
%                             price_mu_vector, price_sigma_vector, prediction_transformed] = ...
%                     OOS_predict_ARX_GARCH_DA_prices_parallel_GE(start, stop, training_window,...
%                                                 data_german, model, hours,...
%                                                 GARCH_type, p, q,...
%                                                 N_mc, quantiles_choice,...
%                                                 dyn_choice, inf_crit_models,...
%                                                 transformation, mc,...
%                                                 distribution);
%                 toc
%                 beep
%                 if length(hours) == 24
%                     save(['../../../Data_MATLAB_Drive/Data/', GARCH_type, '_OOS_simulation_results_GE_DA_', 'model_', model,...
%                         '_transformation_', transformation,...
%                         '_dynamic_choice_', num2str(dyn_choice),...
%                         '_mc_', num2str(mc),'_dist_', distribution, '.mat'],...
%                         'real_value', 'prediction', 'residua', 'samples', 'samples_no_garch', 'mae_hourly', 'rmse_hourly',...
%                             'garch_coefficients', 'con_var', 'quantiles', 'quantiles_no_garch',...
%                             'pinball', 'pinball_no_garch', 'archtest_results', 'aic_matrice',...
%                             'bic_matrice', 'kstest_results', 'kstesttstud_results',...
%                             'price_mu_vector', 'price_sigma_vector', 'prediction_transformed');
%                 else
%                     disp("Results for single hour were not saved.")
%                 end
%             end
%         end
%     end
% end
% GERMAN ID market - all simulations
data_german = importdata('../Data/data_Germany_20170101_20201231.txt');
for dyn_choice = 0:dynamic_choice
    for trans = 0:transformations
        for mc = 0:monte_carlo
            for dist = 0:mc_and_garch_distribution
                if trans == 0
                    transformation = '';
                elseif trans == 1
                    transformation = 'asinh1';
                elseif trans == 2
                    transformation = 'asinh2';
                end
                if dist == 0
                    distribution = 'g';
                elseif dist == 1
                    distribution = 't';
                end
                disp(['ID simulation for ', num2str(dyn_choice), transformation, num2str(mc), distribution, 'GERMAN ID'])
                tic;
                [real_value, prediction, residua, samples, samples_no_garch, mae_hourly, rmse_hourly,...
                            garch_coefficients, con_var, quantiles, quantiles_no_garch,...
                            pinball, pinball_no_garch, archtest_results, aic_matrice,...
                            bic_matrice, kstest_results, kstesttstud_results,...
                            price_mu_vector, price_sigma_vector, prediction_transformed] = ...
                    OOS_predict_ARX_GARCH_ID_prices_parallel_GE(start, stop, training_window,...
                                                data_german, model, hours,...
                                                GARCH_type, p, q,...
                                                N_mc, quantiles_choice,...
                                                dyn_choice, inf_crit_models,...
                                                transformation, mc,...
                                                distribution);
                toc
                beep
                if length(hours) == 24
                    save(['../../../Data_MATLAB_Drive/Data/', GARCH_type, '_OOS_simulation_results_GE_ID_', 'model_', model,...
                        '_transformation_', transformation,...
                        '_dynamic_choice_', num2str(dyn_choice),...
                        '_mc_', num2str(mc),'_dist_', distribution, '.mat'],...
                        'real_value', 'prediction', 'residua', 'samples', 'samples_no_garch', 'mae_hourly', 'rmse_hourly',...
                            'garch_coefficients', 'con_var', 'quantiles', 'quantiles_no_garch',...
                            'pinball', 'pinball_no_garch', 'archtest_results', 'aic_matrice',...
                            'bic_matrice', 'kstest_results', 'kstesttstud_results',...
                            'price_mu_vector', 'price_sigma_vector', 'prediction_transformed');
                else
                    disp("Results for single hour were not saved.")
                end
            end
        end
    end
end
% POLISH DA - all simulations (for all combinations of dynamic choice or GARCH(1,1), MC or hist, transformation or no transformation
data_polish = importdata('../Data/data_Poland_20170101_20201231.txt');
for dyn_choice = 0:dynamic_choice
    for trans = 0:transformations
        for mc = 0:monte_carlo
            for dist = 0:mc_and_garch_distribution
                if trans == 0
                    transformation = '';
                elseif trans == 1
                    transformation = 'asinh1';
                elseif trans == 2
                    transformation = 'asinh2';
                end
                if dist == 0
                    distribution = 'g';
                elseif dist == 1
                    distribution = 't';
                end
                disp(['DA simulation for ', num2str(dyn_choice), transformation, num2str(mc), distribution, 'POLISH DA'])
                tic;
                [real_value, prediction, residua, samples, samples_no_garch, mae_hourly, rmse_hourly,...
                            garch_coefficients, con_var, quantiles, quantiles_no_garch,...
                            pinball, pinball_no_garch, archtest_results, aic_matrice,...
                            bic_matrice, kstest_results, kstesttstud_results,...
                            price_mu_vector, price_sigma_vector, prediction_transformed] = ...
                    OOS_predict_ARX_GARCH_DA_prices_parallel_PL(start, stop, training_window,...
                                                data_polish, model, hours,...
                                                GARCH_type, p, q,...
                                                N_mc, quantiles_choice,...
                                                dyn_choice, inf_crit_models,...
                                                transformation, mc,...
                                                distribution);
                toc
                beep
                if length(hours) == 24
                    save(['../../../Data_MATLAB_Drive/Data/', GARCH_type, '_OOS_simulation_results_PL_DA_', 'model_', model,...
                        '_transformation_', transformation,...
                        '_dynamic_choice_', num2str(dyn_choice),...
                        '_mc_', num2str(mc),'_dist_', distribution, '.mat'],...
                        'real_value', 'prediction', 'residua', 'samples', 'samples_no_garch', 'mae_hourly', 'rmse_hourly',...
                            'garch_coefficients', 'con_var', 'quantiles', 'quantiles_no_garch',...
                            'pinball', 'pinball_no_garch', 'archtest_results', 'aic_matrice',...
                            'bic_matrice', 'kstest_results', 'kstesttstud_results',...
                            'price_mu_vector', 'price_sigma_vector', 'prediction_transformed');
                else
                    disp("Results for single hour were not saved.")
                end
            end
        end
    end
end
% POLISH ID market - all simulations
data_polish = importdata('../Data/data_Poland_20170101_20201231.txt');
for dyn_choice = 0:dynamic_choice
    for trans = 0:transformations
        for mc = 0:monte_carlo
            for dist = 0:mc_and_garch_distribution
                if trans == 0
                    transformation = '';
                elseif trans == 1
                    transformation = 'asinh1';
                elseif trans == 2
                    transformation = 'asinh2';
                end  
                if dist == 0
                    distribution = 'g';
                elseif dist == 1
                    distribution = 't';
                end
                disp(['ID simulation for ', num2str(dyn_choice), transformation, num2str(mc), distribution, 'POLISH ID'])
                tic;
                [real_value, prediction, residua, samples, samples_no_garch, mae_hourly, rmse_hourly,...
                            garch_coefficients, con_var, quantiles, quantiles_no_garch,...
                            pinball, pinball_no_garch, archtest_results, aic_matrice,...
                            bic_matrice, kstest_results, kstesttstud_results,...
                            price_mu_vector, price_sigma_vector, prediction_transformed] = ...
                    OOS_predict_ARX_GARCH_ID_prices_parallel_PL(start, stop, training_window,...
                                                data_polish, model, hours,...
                                                GARCH_type, p, q,...
                                                N_mc, quantiles_choice,...
                                                dyn_choice, inf_crit_models,...
                                                transformation, mc,...
                                                distribution);
                toc
                beep
                if length(hours) == 24
                    save(['../../../Data_MATLAB_Drive/Data/', GARCH_type, '_OOS_simulation_results_PL_ID_', 'model_', model,...
                        '_transformation_', transformation,...
                        '_dynamic_choice_', num2str(dyn_choice),...
                        '_mc_', num2str(mc),'_dist_', distribution, '.mat'],...
                        'real_value', 'prediction', 'residua', 'samples', 'samples_no_garch', 'mae_hourly', 'rmse_hourly',...
                            'garch_coefficients', 'con_var', 'quantiles', 'quantiles_no_garch',...
                            'pinball', 'pinball_no_garch', 'archtest_results', 'aic_matrice',...
                            'bic_matrice', 'kstest_results', 'kstesttstud_results',...
                            'price_mu_vector', 'price_sigma_vector', 'prediction_transformed');
                else
                    disp("Results for single hour were not saved.")
                end
            end
        end
    end
end
