function final_weights = optimal_scenarios_measure(price_scenario_RB, price_scenario_RDN, DA_prediction, option, measure, varargin)

    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%% Input data %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % prediction_QRA_RB - prediction prices using quantile regression method for the day ahead market
    % prediction_QRA_RDN - prediction prices using quantile regression method for the complementary market
    % option - method of finding value of w_ht, which maximalize s_ht 
    % measure - measure of risk
    minArgs = 5;
    maxArgs = 6;
    narginchk(minArgs, maxArgs)
    if nargin == 6
        transaction_side = varargin{1};
    end

    step_no = 11;

    nr_U = size(price_scenario_RB, 1);
    no_of_days = size(price_scenario_RB, 2);

    weights = linspace(0, 1, step_no);

    weights_end_05 = zeros(1, no_of_days);
    weights_end_95 = zeros(1, no_of_days);
    weights_end = zeros(1, no_of_days);

    for k = 1:no_of_days
        scenario_h_t = zeros(length(weights), nr_U);
        quantile_weights_05 = zeros(1,length(weights));
        quantile_weights_95 = zeros(1,length(weights)); 
        quantile_weights = zeros(1,length(weights));
        % Choosing the measure
        switch measure 
            case 'std'
                % grid search
                for i = 1:length(weights)
                    scenario_h_t(i,:) = scenario(weights, price_scenario_RB(:, k), price_scenario_RDN(:, k), i);
                    quantile_weights(i) = std(scenario_h_t(i, :));
                end
                weight_start = weights(quantile_weights == min(quantile_weights));  % chose the best weight
            
                if option == 1
                    fun = @(weights) quantile_weight(weights, price_scenario_RB(:, k), price_scenario_RDN(:, k), 0.05, measure);
                    lb = 0; % weights boundaries
                    ub = 1;
                    A = []; % linear constraints
                    b = [];
                    Aeq = [];
                    beq = [];
                    options = optimoptions('fmincon', 'Display', 'off');
                    fmincon_weight = fmincon(fun, weight_start, A, b, Aeq, beq, lb, ub, [], options);
                    weights_end(k) = fmincon_weight;
                elseif option == 2
                    disp(['No such option ', num2str(option), ' available for measure ', measure])
                end
            case 'std_and_profit'
                % grid search
                for i = 1:length(weights) % TO DO - dopisz warunek także w grid search!
                    scenario_h_t(i,:) = scenario(weights, price_scenario_RB(:, k), price_scenario_RDN(:, k), i);
                    quantile_weights(i) = std(scenario_h_t(i, :));
                end
                weight_start = weights(quantile_weights == min(quantile_weights));  % chose the best weight
            
                if option == 1
                    fun = @(weights) quantile_weight(weights, price_scenario_RB(:, k), price_scenario_RDN(:, k), 0.05, measure);
                    lb = 0; % weights boundaries
                    ub = 1;
                    A = -mean(price_scenario_RB(:, k)) + mean(price_scenario_RDN(:, k)); % linear constraints
                    b = mean(price_scenario_RDN(:, k)) - DA_prediction(k);
                    Aeq = [];
                    beq = [];
                    options = optimoptions('fmincon', 'Display', 'off');
                    fmincon_weight = fmincon(fun, weight_start, A, b, Aeq, beq, lb, ub, [], options);
                    weights_end(k) = fmincon_weight;
                elseif option == 2
                    disp(['No such option ', num2str(option), ' available for measure ', measure])
                end
            case 'semistd_down'
                % grid search
                for i = 1:length(weights)
                    scenario_h_t(i,:) = scenario(weights, price_scenario_RB(:, k), price_scenario_RDN(:, k), i);
                    quantile_weights(i) = semistd(scenario_h_t(i, :), -1);
                end
                weight_start = weights(quantile_weights == min(quantile_weights));  % chose the best weight
            
                if option == 1
                    fun = @(weights) quantile_weight(weights, price_scenario_RB(:, k), price_scenario_RDN(:, k), 0.05, measure);
                    lb = 0; % weights boundaries
                    ub = 1;
                    A = []; % linear constraints
                    b = [];
                    Aeq = [];
                    beq = [];
                    options = optimoptions('fmincon', 'Display', 'off');
                    fmincon_weight = fmincon(fun, weight_start, A, b, Aeq, beq, lb, ub, [], options);
                    weights_end(k) = fmincon_weight;
                elseif option == 2
                    disp(['No such option ', num2str(option), ' available for measure ', measure])
                end
            case 'semistd_up'
                % grid search
                for i = 1:length(weights)
                    scenario_h_t(i,:) = scenario(weights, price_scenario_RB(:, k), price_scenario_RDN(:, k), i);
                    quantile_weights(i) = semistd(scenario_h_t(i, :), 1);
                end
                weight_start = weights(quantile_weights == min(quantile_weights));  % chose the best weight
            
                if option == 1
                    fun = @(weights) quantile_weight(weights, price_scenario_RB(:, k), price_scenario_RDN(:, k), 0.05, measure);
                    lb = 0; % weights boundaries
                    ub = 1;
                    A = []; % linear constraints
                    b = [];
                    Aeq = [];
                    beq = [];
                    options = optimoptions('fmincon', 'Display', 'off');
                    fmincon_weight = fmincon(fun, weight_start, A, b, Aeq, beq, lb, ub, [], options);
                    weights_end(k) = fmincon_weight;
                elseif option == 2
                    disp(['No such option ', num2str(option), ' available for measure ', measure])
                end
            case 'quantile'
                for i = 1 :length(weights)
                    scenario_h_t(i,:) = scenario(weights, price_scenario_RB(:, k), price_scenario_RDN(:, k), i);
                    quantile_weights_05(i) = quantile(scenario_h_t(i,:), 0.05);  
                    quantile_weights_95(i) = quantile(scenario_h_t(i,:), 0.95);
                end
            case 'expectiles'
                for i = 1 :length(weights)
                    scenario_h_t(i,:) = scenario(weights, price_scenario_RB(:, k), price_scenario_RDN(:, k), i);
                    quantile_weights_05(i) = expectile(scenario_h_t(i,:),0.05);  
                    quantile_weights_95(i) = expectile(scenario_h_t(i,:),0.95);
                end
            case 'ES'
                for i = 1 :length(weights)
                    scenario_h_t(i,:) = scenario(weights, price_scenario_RB(:, k), price_scenario_RDN(:, k), i);
                    VaR_05 = quantile(scenario_h_t(i,:),0.05);
                    VaR_95 = quantile(scenario_h_t(i,:),0.95);
                    index_05 = find(scenario_h_t(i,:) < VaR_05);
                    index_95 = find(scenario_h_t(i,:) > VaR_95);
                    help_vec_05 = zeros(1,length(index_05));
                    help_vec_95 = zeros(1,length(index_95));
                    for z = 1:length(index_05)
                        help_vec_05(z) = scenario_h_t(i,index_05(z));
                    end
                    for z = 1:length(index_95)
                        help_vec_95(z) = scenario_h_t(i,index_95(z));
                    end
                    quantile_weights_05(i) =  mean(help_vec_05);
                    quantile_weights_95(i) =  mean(help_vec_95);    
                end
            % In case of sharpe ratio, median/IQR, q_95/q_5, we consider only the
            % case when we have an optimal strategy, so both weights: 
            % 'weights_end_05' and 'weights_end_95' are the same.
            case 'sharpe' %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
                for i = 1 :length(weights)
                    scenario_h_t(i,:) = scenario(weights, price_scenario_RB(:, k), price_scenario_RDN(:, k), i);
                    if -mean(scenario_h_t(i,:)) > 0 && strcmp(transaction_side, 'buyer')
                        quantile_weights(i) = mean(scenario_h_t(i,:)) * std(scenario_h_t(i,:)); % for buyer we want to minimize both std and mean
                    elseif mean(scenario_h_t(i,:)) > 0 && strcmp(transaction_side, 'seller')
                        quantile_weights(i) = mean(scenario_h_t(i,:)) / std(scenario_h_t(i,:));
                    elseif -mean(scenario_h_t(i,:)) < 0 && strcmp(transaction_side, 'buyer')
                        quantile_weights(i) = mean(scenario_h_t(i,:)) / std(scenario_h_t(i,:)); % for buyer we want to minimize both std and mean
                    elseif mean(scenario_h_t(i,:)) < 0 && strcmp(transaction_side, 'seller')
                        quantile_weights(i) = mean(scenario_h_t(i,:)) * std(scenario_h_t(i,:));
                    end
                end
                weight_start = weights(quantile_weights == max(quantile_weights));  % chose the best weight

                if option == 1
                    fun = @(weights) -quantile_weight(weights, price_scenario_RB(:, k), price_scenario_RDN(:, k), 0.05, measure, transaction_side);
                    lb = 0; % weights boundaries
                    ub = 1;
                    A = []; % linear constraints
                    b = [];
                    Aeq = [];
                    beq = [];
                    options = optimoptions('fmincon', 'Display', 'off');
                    fmincon_weight = fmincon(fun, weight_start, A, b, Aeq, beq, lb, ub, [], options);
                    weights_end(k) = fmincon_weight;
                elseif option == 2
                    disp(['No such option as ', num2str(option), ' available for measure ', measure])
                end
             
            case 'q_50_IQR'
                for i = 1 :length(weights)
                    scenario_h_t(i,:) = scenario(weights, price_scenario_RB(:, k), price_scenario_RDN(:, k), i);
                    quantile_weights_05(i) = quantile(scenario_h_t(i,:),0.5)/iqr(scenario_h_t(i,:));  
                    quantile_weights_95(i) = quantile(scenario_h_t(i,:),0.5)/iqr(scenario_h_t(i,:));  
                end
                
            case 'median'
                for i = 1 :length(weights)
                    scenario_h_t(i,:) = scenario(weights, price_scenario_RB(:, k), price_scenario_RDN(:, k), i);
                    quantile_weights_05(i) = quantile(scenario_h_t(i,:),0.5); 
                    quantile_weights_95(i) = median(scenario_h_t(i,:));
                end
                
            case 'mean'
                for i = 1 :length(weights)
                    scenario_h_t(i,:) = scenario(weights, price_scenario_RB(:, k), price_scenario_RDN(:, k), i);
                    quantile_weights(i) = mean(scenario_h_t(i,:));
                end
                % looking for a index, where the value of quantile is maximal
                weight_start = weights(quantile_weights == max(quantile_weights));  % chose the best weight

                %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
                %%%%%%%%%% Looking for value of w_ht, which maximalize s_ht %%%%%%%%%%

                if option == 1 % The first way
                    fun = @(weights) -quantile_weight(weights, price_scenario_RB(:, k), price_scenario_RDN(:, k), 0.05, measure);
                    lb = 0;
                    ub = 1;
                    A = [];
                    b = [];
                    Aeq = [];
                    beq = [];
                    options = optimoptions('fmincon','Display','off');
                    w = fmincon(fun, weight_start, A, b, Aeq, beq, lb, ub, [], options);
                    weights_end(k) = w;
                end
            case 'mean_and_std'
                for i = 1 :length(weights)
                    scenario_h_t(i,:) = scenario(weights, price_scenario_RB(:, k), price_scenario_RDN(:, k), i);
                    quantile_weights(i) = mean(scenario_h_t(i,:));
                end
                weight_start = weights(quantile_weights == max(quantile_weights));  % chose the best weight
                nonlinear_constraint = @(w) mean_and_std_nonlinear_constraint( ...
                        w * price_scenario_RB(:, k) + (1 - w) * price_scenario_RDN(:, k), ...
                        price_scenario_RDN(:, k), price_scenario_RB(:, k));
                if option == 1
                    fun = @(weights) -quantile_weight(weights, price_scenario_RB(:, k), price_scenario_RDN(:, k), 0.05, measure);
                    lb = 0; % weights boundaries
                    ub = 1;
                    A = []; % linear constraints
                    b = [];
                    Aeq = [];
                    beq = [];
                    options = optimoptions('fmincon', 'Display', 'off');
                    fmincon_weight = fmincon(fun, weight_start, A, b, Aeq, beq, lb, ub, nonlinear_constraint, options);
                    weights_end(k) = fmincon_weight;
                elseif option == 2
                    disp(['No such option ', num2str(option), ' available for measure ', measure])
                end
        end

        switch measure
            case {'quantile', 'expectiles', 'ES', 'q_50_IQR', 'median'}
     
                % looking for a index, where the value of quantile is maximal
                weight_start_05 = weights(quantile_weights_05 == max(quantile_weights_05));  % chose the best weight
                weight_start_95 = weights(quantile_weights_95 == max(quantile_weights_95));

                %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
                %%%%%%%%%% Looking for value of w_ht, which maximalize s_ht %%%%%%%%%%

                if option == 1 % The first way
                    fun_05 = @(weights) -quantile_weight(weights, price_scenario_RB(:, k), price_scenario_RDN(:, k), 0.05, measure);
                    fun_95 = @(weights) -quantile_weight(weights, price_scenario_RB(:, k), price_scenario_RDN(:, k), 0.95, measure);
                    lb = 0;
                    ub = 1;
                    A = [];
                    b = [];
                    Aeq = [];
                    beq = [];
                    options = optimoptions('fmincon','Display','off');
                    w_05 = fmincon(fun_05, weight_start_05, A, b, Aeq, beq, lb, ub, [], options);
                    w_95 = fmincon(fun_95, weight_start_95, A, b, Aeq, beq, lb, ub, [], options);
                    weights_end_05(k) = w_05;
                    weights_end_95(k) = w_95;

                elseif option == 2  % The second way
                    % fit the curve to data
                    fit_curve_05 = polyfit(weights, quantile_weights_05, 1);
                    fit_curve_95 = polyfit(weights, quantile_weights_95, 1);

                    % looking for value of w_ht, which maximalize s_ht
                    fun_05 = @(w_05)((fit_curve_05(1)*w_05+fit_curve_05(2)));
                    fun_95 = @(w_95)((fit_curve_95(1)*w_95+fit_curve_95(2)));
                    lb = 0;
                    ub = 1;
                    A = [];
                    b = [];
                    Aeq = [];
                    beq = [];
                    % w_05 = fminsearch(@(w_05) fun_05(w_05)*-1, weight_start_05);
                    options = optimoptions('fmincon','Display','off');
                    w_05 = fmincon(@(w_05) -1*fun_05(w_05),weight_start_05,A,b,Aeq,beq,lb,ub,[],options);
                    w_95 = fmincon(@(w_95) -1*fun_95(w_95),weight_start_95,A,b,Aeq,beq,lb,ub,[], options);
                    weights_end_05(k) = w_05;
                    weights_end_95(k) = w_95;
                end    
        end
    end
    switch measure
        case {'quantile', 'expectiles', 'ES', 'q_50_IQR', 'median'}
            final_weights = [weights_end_05; weights_end_95];
        case {'std', 'std_and_profit', 'semistd_down', 'semistd_up', 'sharpe', 'mean_and_std', 'mean'}
            final_weights = weights_end;
    end
end
