%%%%%%%%% Model ARX for intra day prices, german market %%%%%%%%

function [real_value, prediction, residua, samples_all,...
            samples_no_garch_all, mae_hourly, rmse_hourly,...
            garch_coefficients, con_var, quantiles, quantiles_no_garch,...
            pinball, pinball_no_garch, archtest_results, aic_matrice,...
            bic_matrice, kstest_results, kstesttstud_results,...
            price_mu_vector, price_sigma_vector, prediction_transformed] = ...
    predict_ARX_GARCH_ID_prices_parallel_GE(start, stop, training_window,...
                                data, model, hours, GARCH_type,...
                                p_GARCH, q_GARCH,...
                                N_mc, quantiles_choice,...
                                dynamic_choice, inf_crit_models,...
                                transformation, monte_carlo, mc_dist)
   
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %%%%%%%%%%%%%%%%%%%%%%%%%% Input data %%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % start (int) - starting point of evalueation window (in days)
    % stop (int) - end point of evaluation window (in days)
    % data (table) - matrix with explanatory and exogenous variables
    % model - use one of nine different models (every model contains also
    % min and max price of previous day and dummi:
        % '1' - X1,X2,X3,X4,X5,X6, L =[1,2,7]
        % '2'- X1,X2,X3,X4,X5,X6, L = [1,2]
        % '3'- X2,X3,X4,X6, L = [1,2,7]
        % '4'- X2,X3,X4,X6, L =[1,2]
        % '5' - X1,X2,X3, L =[1,2,7]
        % '6' - X1,X2,X3, L = [1,2]
        % '7' - X1,X2,X3,X4,X5,X6, L = [1,2,...,7]
        % '8' - L = [1,2,...,7]
        % '9' - X2,X3,X4,X6, L =[1,2,...,7]
    % X1 - Forecast of generation
    % X2 - Forecast of wind generation
    % X3 - Forecast of load
    % X4 - Solar
    % X5 - Co2
    % X6 - Gas
    if start - training_window - 7 <= 0
        error("The starting point of simulation can not be less than training window length!");
    elseif stop > size(data, 1)
        error("The end point of simulation can not be greater than length of Your sample!");
    elseif str2double(model) > 9 ||  str2double(model) < 1
        error("The model numbers spans from 1 to 9!")
    elseif ~strcmp(transformation, '') && ~strcmp(transformation, 'asinh')
        error("There is no such transformation")
    end
    options = optimoptions(@fmincon,'Algorithm','interior-point');
    prediction = ones(length(hours), stop - start) .* -999; % multiply by -999 to potentially see whether stg gone wrong 
    prediction_transformed = ones(length(hours), stop - start) .* -999;
    residua = ones(length(hours), training_window, stop - start) .* -999;
    samples_all = ones(length(hours), training_window, stop - start) .* -999;
    samples_no_garch_all = ones(length(hours), training_window, stop - start) .* -999;
    real_value = ones(length(hours), stop - start) .* -999;
    con_var = ones(length(hours), stop - start) .* -999;
    mae_hourly = ones(length(hours), 1) .* -999;
    rmse_hourly = ones(length(hours), 1) .* -999;
    garch_coefficients = cell([length(hours), 3]);
    quantiles = ones(length(hours), stop - start, length(quantiles_choice)) .* -999;
    pinball = ones(length(hours), stop - start, length(quantiles_choice)) .* -999;
    quantiles_no_garch = ones(length(hours), stop - start, length(quantiles_choice)) .* -999;
    pinball_no_garch = ones(length(hours), stop - start, length(quantiles_choice)) .* -999;
    archtest_results = ones(length(hours), stop - start, 2) .* -999;
    kstest_results = ones(length(hours), stop - start, 2) .* -999;
    kstesttstud_results = ones(length(hours), stop - start, 2) .* -999;
    perm = permn(inf_crit_models, 2); % all possible models to extract BIC/AIC
    aic_matrice = ones(length(hours), stop - start, size(perm, 1)) .* -999;
    bic_matrice = ones(length(hours), stop - start, size(perm, 1)) .* -999;
    price_mu_vector = ones(length(hours), stop - start) .* -999; % mean and std of prices windows (to perform asinh transformation/inverse transformation)
    price_sigma_vector = ones(length(hours), stop - start) .* -999;
    % Matrix of prediction prices
    parfor hour = hours
        X = [];
        X_reg = [];
        tmp = data; % to avoid broadcast variable
        pinball_row = ones(stop - start, length(quantiles_choice)) .* -999;
        quantiles_row = ones(stop - start, length(quantiles_choice)) .* -999;
        pinball_row_no_garch = ones(stop - start, length(quantiles_choice)) .* -999;
        quantiles_row_no_garch = ones(stop - start, length(quantiles_choice)) .* -999;
        con_var_row = ones(1, stop - start) .* -999;
        garch_coefficients_row_const = ones(stop - start, 1) .* -999; % to hold place for constant and GARCH and ARCH polynomials
        garch_coefficients_row_GARCH = ones(stop - start, max(inf_crit_models)) .* -999;
        garch_coefficients_row_ARCH = ones(stop - start, max(inf_crit_models)) .* -999;
        prediction_row = ones(1, stop - start) .* -999;
        prediction_transformed_row = ones(1, stop - start) .* -999;
        real_value_row = ones(1, stop - start) .* -999;
        residua_window = ones(training_window, stop - start) .* -999;
        samples_row = ones(training_window, stop - start) .* -999;
        samples_no_garch_row = ones(training_window, stop - start) .* -999;
        archtest_results_window = ones(stop - start, 2) .* -999;
        kstest_results_window = ones(stop - start, 2) .* -999;
        kstesttstud_results_window = ones(stop - start, 2) .* -999;
        aic_matrice_row = ones(stop - start, size(perm, 1)) .* -999;
        bic_matrice_row = ones(stop - start, size(perm, 1)) .* -999;
        price_mu_vector_row = ones(1, stop - start) .* -999;
        price_sigma_vector_row = ones(1, stop - start) .* -999;
        for day = start + 1:stop
            data_window = tmp(day * 24 - (training_window * 24) - (8 * 24) + 1:day * 24,:);
            if strcmp(transformation, 'asinh')
                % "Price" - vector of prices for each hour separately
                [standardized_price, mu_price, sigma_price] =...
                    zscore(data_window(hour:24:end-24, 4));
                price = asinh(standardized_price);
                price_mu_vector_row(day - training_window - 8) = mu_price;
                price_sigma_vector_row(day - training_window - 8) = sigma_price;
                % Prices of RDN
                standardized_RDN_price =...
                    zscore(data_window(hour:24:end-24,3));
                price_RDN = asinh(standardized_RDN_price);  
                % Minimum of the previous day of RDN market
                standardized_price_min = ...
                    zscore(data_window(hour:24:end-24,16));
                price_RDN_min = asinh(standardized_price_min);
                % Maximum of the previous day of RDN market
                standardized_price_max = ...
                    zscore(data_window(hour:24:end-24,17));
                price_RDN_max = asinh(standardized_price_max);
            elseif strcmp(transformation, '')
                % "Price" - vector of prices for each hour separately
                price = data_window(hour:24:end-24,4);
                % Prices of RDN
                price_RDN = data_window(hour:24:end-24,3);   
                % Minimum of the previous day of RDN market
                price_RDN_min = data_window(hour:24:end-24,16); 
                % Maximum of the previous day of RDN market
                price_RDN_max = data_window(hour:24:end-24,17); 
            end

            % Forecast of generation
            generation = data_window(hour:24:end,6);

            % Forecast of wind generation
            wind = data_window(hour:24:end,7);

            % Forecast of load
            load = data_window(hour:24:end,5);

            % Solar
            solar = data_window(hour:24:end,8);

            % Co2
            Co2 = data_window(hour:24:end-24,10);

            % Gas
            gas = data_window(hour:24:end-24,9);

            % Dummies
            D_Sat = data_window(hour:24:end,12);
            D_Sun = data_window(hour:24:end,13);
            D_Mon = data_window(hour:24:end,14);
            D_rest = data_window(hour:24:end,15);
            Y = price(8:end);
            % Model '1' - X1,X2,X3,X4,X5,X6, L =[2,7]
            if model == '1'  
                % Matrix X in ARX's model
                X = [price_RDN(7:end-1),price(6:end-2),price(1:end-7),...
                    generation(8:end-1), wind(8:end-1), load(8:end-1),...
                    solar(8:end-1), Co2(7:end-1), gas(7:end-1),...
                    price_RDN_min(7:end-1), price_RDN_max(7:end-1),...
                    D_Sat(8:end-1),D_Sun(8:end-1),D_Mon(8:end-1),D_rest(8:end-1)];
                % Prediction on d+1 day
                X_reg = [price_RDN(end), price(end-1),price(end-6),generation(end), wind(end),load(end),...
                    solar(end), Co2(end),gas(end),...
                    price_RDN_min(end),price_RDN_max(end),...
                    D_Sat(end),D_Sun(end),D_Mon(end), D_rest(end)];
            % Model '2'- X1,X2,X3,X4,X5,X6,X7, L = [2]
            elseif model == '2'
                % Matrix X in ARX's model
                X = [price_RDN(7:end-1),price(6:end-2),generation(8:end-1),wind(8:end-1),load(8:end-1),...
                    solar(8:end-1),Co2(7:end-1),gas(7:end-1),...
                    price_RDN_min(7:end-1),price_RDN_max(7:end-1),...
                    D_Sat(8:end-1),D_Sun(8:end-1), D_Mon(8:end-1),D_rest(8:end-1)];
                % Prediction on d+1 day
                X_reg = [price_RDN(end), price(end-1),generation(end), wind(end),load(end),...
                    solar(end), Co2(end),gas(end),...
                    price_RDN_min(end),price_RDN_max(end),...
                    D_Sat(end),D_Sun(end),D_Mon(end),D_rest(end)];
            % Model '3'- X2,X3,X4,X6, L = [2,7]
            elseif model == '3'
                % Matrix X in ARX's model
                X = [price_RDN(7:end-1),price(6:end-2),price(1:end-7),wind(8:end-1),load(8:end-1),...
                    solar(8:end-1), gas(7:end-1),...
                    price_RDN_min(7:end-1),price_RDN_max(7:end-1),...
                    D_Sat(8:end-1),D_Sun(8:end-1),D_Mon(8:end-1),D_rest(8:end-1)];
                % Prediction on d+1 day
                X_reg = [price_RDN(end), price(end-1),price(end-6),wind(end),load(end),...
                    solar(end), gas(end),...
                    price_RDN_min(end),price_RDN_max(end),...
                    D_Sat(end),D_Sun(end),D_Mon(end),D_rest(end)];
            % Model '4'- X2,X3,X4,X6, L =[2]
            elseif model == '4'
                 % Matrix X in ARX's model
                X = [price_RDN(7:end-1),price(6:end-2),wind(8:end-1),load(8:end-1),...
                    solar(8:end-1), gas(7:end-1),...
                    price_RDN_min(7:end-1),price_RDN_max(7:end-1),...
                    D_Sat(8:end-1),D_Sun(8:end-1),D_Mon(8:end-1),D_rest(8:end-1)];
                % Prediction on d+1 day
                X_reg = [price_RDN(end), price(end-1),wind(end),load(end),...
                    solar(end), gas(end),...
                    price_RDN_min(end),price_RDN_max(end),...
                    D_Sat(end),D_Sun(end),D_Mon(end),D_rest(end)];      
            % Model '5' - X1,X2,X3, L =[2,7]
            elseif model == '5'
                  % Matrix X in ARX's model
                X = [price_RDN(7:end-1),price(6:end-2),price(1:end-7),...
                    generation(8:end-1),wind(8:end-1),load(8:end-1),...
                    price_RDN_min(7:end-1),price_RDN_max(7:end-1),...
                    D_Sat(8:end-1),D_Sun(8:end-1),D_Mon(8:end-1),D_rest(8:end-1)];
                % Prediction on d+1 day
                X_reg = [price_RDN(end), price(end-1),price(end-6),...
                    generation(end), wind(end),load(end),...
                    price_RDN_min(end),price_RDN_max(end),...
                    D_Sat(end),D_Sun(end),D_Mon(end),D_rest(end)];
            % Model '6' - X1,X2,X3, L =[2]
            elseif model == '6'
                 % Matrix X in ARX's model
                X = [price_RDN(7:end-1),price(6:end-2),...
                    generation(8:end-1),wind(8:end-1),load(8:end-1),...
                    price_RDN_min(7:end-1),price_RDN_max(7:end-1),...
                    D_Sat(8:end-1),D_Sun(8:end-1),D_Mon(8:end-1),D_rest(8:end-1)];
                % Prediction on d+1 day
                X_reg = [price_RDN(end), price(end-1),...
                    generation(end), wind(end),load(end),...
                    price_RDN_min(end),price_RDN_max(end),...
                    D_Sat(end),D_Sun(end),D_Mon(end),D_rest(end)];
            % Model '7' - X1,X2,X3,X4,X5,X6,X7, L =[2,...,7]
            elseif model == '7'
                if any([1,2,3,4,23,24] == hour)
                    % Matrix X in ARX's model
                    X = [price_RDN(7:end-1),price(6:end-2),...
                        price(5:end-3),price(4:end-4),price(3:end-5),...
                        price(2:end-6),price(1:end-7),...
                        generation(8:end-1),wind(8:end-1),load(8:end-1),...
                        Co2(7:end-1),gas(7:end-1),...
                        price_RDN_min(7:end-1),price_RDN_max(7:end-1),...
                        D_Sat(8:end-1),D_Sun(8:end-1),...
                        D_Mon(8:end-1),D_rest(8:end-1)];
                    % Prediction on d+1 day
                    X_reg = [price_RDN(end), price(end-1),...
                        price(end-2),price(end-3),...
                        price(end-4),price(end-5),price(end-6),...
                        generation(end), wind(end),load(end),...
                        Co2(end),gas(end),...
                        price_RDN_min(end),price_RDN_max(end),...
                        D_Sat(end),D_Sun(end),D_Mon(end),D_rest(end)];
                else
                    % Matrix X in ARX's model
                    X = [price_RDN(7:end-1),price(6:end-2),...
                        price(5:end-3),price(4:end-4),price(3:end-5),...
                        price(2:end-6),price(1:end-7),...
                        generation(8:end-1),wind(8:end-1),load(8:end-1),...
                        solar(8:end-1),Co2(7:end-1),gas(7:end-1),...
                        price_RDN_min(7:end-1),price_RDN_max(7:end-1),...
                        D_Sat(8:end-1),D_Sun(8:end-1),...
                        D_Mon(8:end-1),D_rest(8:end-1)];
                    % Prediction on d+1 day
                    X_reg = [price_RDN(end), price(end-1),...
                        price(end-2),price(end-3),...
                        price(end-4),price(end-5),price(end-6),...
                        generation(end), wind(end),load(end),...
                        solar(end), Co2(end),gas(end),...
                        price_RDN_min(end),price_RDN_max(end),...
                        D_Sat(end),D_Sun(end),D_Mon(end),D_rest(end)];
                end
            % Model '8' - L =[2,...,7]
            elseif model == '8'
                 % Matrix X in ARX's model
                X = [price_RDN(7:end-1),price(6:end-2),...
                    price(5:end-3),price(4:end-4),...
                    price(3:end-5),price(2:end-6),price(1:end-7),...
                    price_RDN_min(7:end-1),price_RDN_max(7:end-1),...
                    D_Sat(8:end-1),D_Sun(8:end-1),...
                    D_Mon(8:end-1),D_rest(8:end-1)];
                % Prediction on d+1 day
                X_reg = [price_RDN(end), price(end-1),...
                    price(end-2), price(end-3), price(end-4), price(end-5), price(end-6),...
                    price_RDN_min(end),price_RDN_max(end),...
                    D_Sat(end),D_Sun(end),D_Mon(end),D_rest(end)];
            % Model '9' - X2,X3,X4,X6, L =[2,...,7]
            elseif model == '9'
                 % Matrix X in ARX's model
                X = [price_RDN(7:end-1), price(6:end-2), price(5:end-3), price(4:end-4), price(3:end-5), price(2:end-6), price(1:end-7),...
                    wind(8:end-1),load(8:end-1),...
                    solar(8:end-1), gas(7:end-1),...
                    price_RDN_min(7:end-1),price_RDN_max(7:end-1),...
                    D_Sat(8:end-1),D_Sun(8:end-1),D_Mon(8:end-1),D_rest(8:end-1)];
                % Prediction on d+1 day
                X_reg = [price_RDN(end), price(end-1), price(end-2), price(end-3), price(end-4), price(end-5), price(end-6),...
                    wind(end),load(end),...
                    solar(end), gas(end),...
                    price_RDN_min(end),price_RDN_max(end),...
                    D_Sat(end),D_Sun(end),D_Mon(end),D_rest(end)];
            end  
            %% ARX ESTIMATION AND PREDICTION
            [beta, ~, window_residuals]  = regress(Y, X);
            residua_window(:, day - training_window - 8) = window_residuals;
            price_point = X_reg * beta;
            real_value_row(day - training_window - 8) = data_window((training_window + 7) * 24 + hour:24:end, 4);
            %% GARCH ESTIMATION
            if dynamic_choice == 1 % test which order is the best based on AIC and then perform simulation for it
                Models_list = cell(1, size(perm, 1));
                for permutation = 1:size(perm, 1)
                    p_perm = perm(permutation, 1);
                    q_perm = perm(permutation, 2);
                    if strcmp(GARCH_type, 'GARCH')
                        Mdl = garch(p_perm, q_perm);
                    elseif strcmp(GARCH_type, 'EGARCH')
                        Mdl = egarch(p_perm, q_perm);
                    elseif strcmp(GARCH_type, 'GJRGARCH')
                        Mdl = gjr(p_perm, q_perm);
                    end
                    if mc_dist == 't'
                        Mdl.Distribution = 't';
                    end
                    [EstMdl, EstParamCov, logL] = estimate(Mdl, window_residuals, 'Display', 'off','options',options);
                    Models_list{permutation} = EstMdl;
                   

                    numParams = sum(any(...
                                EstParamCov)); % how many parameters does the GARCH(p,q) have?
                    [aic, bic] = aicbic(logL, numParams, training_window);
                    aic_matrice_row(day - start, permutation) = aic;
                    bic_matrice_row(day - start, permutation) = bic;
                end
                % find the minimum index
                [~, index_of_min] = min(...
                    aic_matrice_row(day - start, :));
                EstMdl = Models_list{index_of_min};
            elseif dynamic_choice == 0 % perform simulation for choosen p and q
                if strcmp(GARCH_type, 'GARCH')
                    Mdl = garch(p_GARCH, q_GARCH);
                elseif strcmp(GARCH_type, 'EGARCH')
                    Mdl = egarch(p_GARCH, q_GARCH);
                elseif strcmp(GARCH_type, 'GJRGARCH')
                    Mdl = gjr(p_GARCH, q_GARCH);
                end
                if mc_dist == 't'
                    Mdl.Distribution = 't';
                end
                EstMdl = estimate(Mdl, window_residuals, 'Display','off','options',options);
            end
            garch_coefficients_row_const(day - training_window - 8) = ...
                [EstMdl.Constant]; % to hold place for constant and GARCH and ARCH polynomials
            garch_coefficients_row_GARCH(day - training_window - 8,...
                1:length([EstMdl.GARCH{:}])) = ...
                    [EstMdl.GARCH{:}];
            garch_coefficients_row_ARCH(day - training_window - 8,...
                1:length([EstMdl.ARCH{:}])) = ...
                    [EstMdl.ARCH{:}];
            [decision, pval] = archtest(window_residuals);
            archtest_results_window(...
                day - training_window - 8, :) =...
                    [decision, pval]; % Engle test for residual heteroscedasticity
            standardized_residuals = (window_residuals) ./...
                sqrt(infer(EstMdl, window_residuals));
            %% TESTING DISTRIBUTION OF RESIDUALS
            [decision, pval] = kstest(standardized_residuals);
            kstest_results_window(...
                day - training_window - 8, :) =...
                    [decision, pval]; % test for the normality
            warning('off', 'all') % to suppress warnings
            Dist_DA = fitdist(standardized_residuals, 'tLocationScale');
            test_cdf = [standardized_residuals,...
                cdf('tlocationscale',...
                standardized_residuals,...
                Dist_DA.mu, Dist_DA.sigma, Dist_DA.nu)];
            [decision, pval] = kstest(standardized_residuals, 'CDF', test_cdf);
            kstesttstud_results_window(...
                day - training_window - 8, :) =...
                    [decision, pval]; % test for the t-Student
            %% FORECAST OF CONDITIONAL VARIANCE
            if day < start + 8 % to allow p up to 7
                con_var_row(day - training_window - 8) = forecast(...
                    EstMdl, 1, window_residuals);
            else
                con_var_row(day - training_window - 8) = forecast(...
                    EstMdl, 1, window_residuals, 'V0',...
                    con_var_row(...
                        day - training_window - 8 - 7:...
                            day - training_window - 9)');
            end
            %% SAMPLE SIMULATION
            if strcmp(transformation, 'asinh') % inverse asinh transformation
                if monte_carlo == 1
                    if mc_dist == 'g' % gaussian
                        samples = sigma_price .* sinh(price_point + ...
                                normrnd(0, sqrt(...
                                con_var_row(...
                                day - training_window - 8)),...
                                [1, N_mc])) + mu_price;
                        samples_no_garch = sigma_price .* sinh(price_point + ...
                                normrnd(0, std(window_residuals),...
                                [1, N_mc])) + mu_price;
                    elseif mc_dist == 't' % tStudent
                        samples = sigma_price .* sinh(price_point + ...
                                sqrt(...
                                con_var_row(...
                                day - training_window - 8)) .* random(...
                                    'tlocationscale', 0,...
                                    1,...
                                    EstMdl.Distribution.DoF,...
                                    [1, N_mc])) + mu_price;
                        Dist_DA_no_garch = fitdist(window_residuals, 'tLocationScale');
                        samples_no_garch = sigma_price .* sinh(price_point + ...
                                random(...
                                    'tlocationscale', 0,...
                                    Dist_DA_no_garch.sigma,...
                                    Dist_DA_no_garch.nu,...
                                    [1, N_mc])) + mu_price;
                    end
                elseif monte_carlo == 0 % historical simulation
                    samples = sigma_price .* sinh(price_point + ...
                            standardized_residuals .* sqrt(...
                                con_var_row(...
                                    day - training_window - 8))) + mu_price;
                    samples_no_garch = sigma_price .* sinh(price_point + ...
                                window_residuals) + mu_price;
                end
            elseif strcmp(transformation, '')
                if monte_carlo == 1
                    if mc_dist == 'g' % gaussian
                        samples = price_point + ...
                            normrnd(0, sqrt(con_var_row(...
                                day - training_window - 8)),...
                                [1, N_mc]);
                        samples_no_garch = X_reg * beta + ...
                                normrnd(0, std(window_residuals),...
                                [1, N_mc]);
                    elseif mc_dist == 't' % tStudent
                        samples = X_reg * beta + ...
                                sqrt(...
                                con_var_row(...
                                day - training_window - 8)) .* random(...
                                    'tlocationscale', 0,...
                                    1,...
                                    EstMdl.Distribution.DoF,...
                                    [1, N_mc]);
                        Dist_DA_no_garch = fitdist(window_residuals, 'tLocationScale');
                        samples_no_garch = X_reg * beta + ...
                                random(...
                                    'tlocationscale', 0,...
                                    Dist_DA_no_garch.sigma,...
                                    Dist_DA_no_garch.nu,...
                                    [1, N_mc]);
                    end
                elseif monte_carlo == 0 % historical simulation
                    samples = price_point + ...
                            standardized_residuals .* sqrt(...
                                con_var_row(...
                                day - training_window - 8));
                    samples_no_garch = X_reg * beta + ...
                                window_residuals;
                end
            end
            if strcmp(transformation, '')
                prediction_row(day - training_window - 8) = price_point;
            elseif strcmp(transformation, 'asinh')
                prediction_row(day - training_window - 8) = ...
                    mean(samples); % prediction in "real world"
                prediction_transformed_row(day - training_window - 8) = price_point; % prediction in "asinh world"
            end
            %% CALCULATION OF QUANTILES AND PINBALL
            quantiles_row(...
                day - training_window - 8, :) = ...
                    quantile(samples, quantiles_choice);
            quantiles_row_no_garch(...
                day - training_window - 8, :) = ...
                    quantile(samples_no_garch, quantiles_choice);
            pinball_row(day - training_window - 8, :) = pinball_loss(quantiles_choice,...
                    real_value_row(day - training_window - 8), quantiles_row(day - training_window - 8, :));
            pinball_row_no_garch(day - training_window - 8, :) = pinball_loss(quantiles_choice,...
            real_value_row(day - training_window - 8), quantiles_row_no_garch(day - training_window - 8, :));
                                  samples_row( :,day-start) = samples;
            samples_no_garch_row(:,day-start) = samples_no_garch;
        end
        pinball(hour, :, :) = pinball_row;
        quantiles(hour, :, :) = quantiles_row;
        pinball_no_garch(hour, :, :) = pinball_row_no_garch;
        quantiles_no_garch(hour, :, :) = quantiles_row_no_garch;
        con_var(hour, :) = con_var_row;
        garch_coefficients{hour} = ...
            {garch_coefficients_row_const,...
            garch_coefficients_row_GARCH,...
            garch_coefficients_row_ARCH};
        prediction(hour, :) = prediction_row;
        prediction_transformed(hour, :) = prediction_transformed_row;
        real_value(hour, :) = real_value_row;
        residua(hour, :, :) = residua_window;
        samples_all(hour, :, :) = samples_row;
        samples_no_garch_all(hour, :, :) = samples_no_garch_row;
        archtest_results(hour, :, :) = archtest_results_window;
        kstest_results(hour, :, :) = kstest_results_window;
        kstesttstud_results(hour, :, :) = kstesttstud_results_window;
        aic_matrice(hour, :, :) = aic_matrice_row;
        bic_matrice(hour, :, :) = bic_matrice_row;
        price_mu_vector(hour, :) = price_mu_vector_row;
        price_sigma_vector(hour, :) = price_sigma_vector_row;
        mae_hourly(hour) = MAE_func(prediction(hour, :), real_value(hour, :));
        rmse_hourly(hour) = RMSE_func(prediction(hour, :), real_value(hour, :));
        disp(['Simulation for hour ', num2str(hour), ' - finished'])
    end
end
