function [profit, total_profit, hourly_std_profit, wallet,...
        hourly_std_wallet, hourly_mean_wallet, daily_std_wallet,...
        daily_mean_wallet] = ...
    strategy_evaluation_metrics(real_price_RDN, real_price_RB, weights, transaction_side, market)

    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%% Input data %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    
    % real_price_RDN - real prices for the day ahead market in the third window
    % real_price_RB - real prices for the complementary market in the third window
    % amount_day_3 - amount of day in validation's window 
    % nr_U - number of scenarios
    % option - method of finding value of w_ht, which maximalize s_ht 
    % measure - measure of risk

    no_of_days = size(real_price_RDN, 2);

    DA_rv = real_price_RB;
    ID_rv = real_price_RDN;
    if strcmp(transaction_side, "seller") % we want our wallet to be worth less than DA
        ID_rv(ID_rv < DA_rv) = DA_rv(ID_rv < DA_rv);
    elseif strcmp(transaction_side, "buyer")
        ID_rv(ID_rv > DA_rv) = DA_rv(ID_rv > DA_rv);
    end

    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%% Hourly profit %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    profit = zeros(24, no_of_days);
    wallet = zeros(24, no_of_days);
    for hour = 1:24
        for i = 1:no_of_days
            if strcmp(market, "DA")
                profit(hour, i) = weights(i, hour) * real_price_RB(hour, i) + (1 - weights(i, hour)) * real_price_RDN(hour, i) - real_price_RDN(hour, i);
            elseif strcmp(market, "ID")
                profit(hour, i) = weights(i, hour) * real_price_RB(hour, i) + (1 - weights(i, hour)) * real_price_RDN(hour, i) - real_price_RB(hour, i);
            elseif strcmp(market, "max_ID_DA")
                profit(hour, i) = weights(i, hour) * real_price_RB(hour, i) + (1 - weights(i, hour)) * real_price_RDN(hour, i) - ID_rv(hour, i);
            end
            wallet(hour, i) = weights(i, hour) * real_price_RB(hour, i) + (1 - weights(i, hour))* real_price_RDN(hour, i);
        end
    end
    if strcmp(transaction_side, "buyer") % we want our wallet to be worth less than DA
        profit = -profit;
    end
    hourly_std_wallet = std(wallet, 0, 2);
    daily_std_wallet = std(wallet, 0, 1);
    hourly_mean_wallet = mean(wallet, 2);
    daily_mean_wallet = mean(wallet, 1);
    hourly_std_profit = std(profit, 0, 2);
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%% Total profit %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    total_profit = sum(sum(profit));
end
