%% zoomPlot     add inlaid plot to current figure
%       [p,z]   = zoomPlot(x,y,xbounds,pos,vertex) where: 
%       x,y     = vectors being plotted
%       xbounds = [x1 x2] specifies the zoom indices, where x1 is the 
%               first x value displayed in the zoom plot and x2 is the last.
%       pos     = [left, bottom, width, height] specifies the location and
%               size of the side of the zoom box, relative to the lower-left
%               corner of the Figure window, in normalized units where (0,0)
%               is the lower-left corner and (1.0,1.0) is the upper-right.
% {opt} vertex  = toggles connecting lines corresponding to vertices, where 1 
%               corresponds to the top left vertex and continuing clockwise, 
%               4 corresponds to the bottom right vertex. All 4 vertices can 
%               be included.
% {opt} p       = axes handle for larger plot
% {opt} z       = axes handle for zoom plot
% 
% Note: place title, labels, and legend BEFORE placing zoom plot,
%     otherwise zoomPlot returns the handle of the original axes (p).
%     Change title using p.Title.String = 'insert title here'
% 
% Kelsey Bower (kelsey.bower@case.edu) 10/20/15
 

function [p z] = zoomPlot(x,y,xbounds,pos,vertex, markers, marker_sizes, include_DA, line_x, line_y)
    % Please retain the following:
    % 
    % Original Author: 
    % Kelsey Bower, kelsey.bower@case.edu
%     if nargin > 7
%         disp('Too many arguments. zoomPlot(x,y,xbounds,pos,vertex)\n')
%     elseif nargin < 5
%         vertex = [1 4];
%         include_DA = 0;
%     elseif nargin >= 5 && nargin < 6
%         vertex = varargin{1};
%         include_DA = 0;
%     elseif nargin == 6
%         vertex = varargin{1};
%         include_DA = varargin{2};
%     end
    % Get current axis position and limits
    p = gca;
    if include_DA == 0
        x_DA_ID = x(end-1:end);
        x = x(1:end-2);
        y_DA_ID = y(end-1:end);
        y = y(1:end-2);
    end
    index = find(x>=xbounds(1) & x<=xbounds(2)); % Find indexes of points in zoomPlot
    dif_y = 0.1*(max(y(index)) - min(y(index)));
    % Calculate x,y points of zoomPlot
    x1 = (pos(1)-p.Position(1))/p.Position(3)*diff(p.XLim)+p.XLim(1);
    x2 = (pos(1)+pos(3)-p.Position(1))/p.Position(3)*diff(p.XLim)+(p.XLim(1));
    y1 = (pos(2)-p.Position(2))/p.Position(4)*diff(p.YLim)+p.YLim(1);
    y2 = ((pos(2)+pos(4)-p.Position(2))/p.Position(4))*diff(p.YLim)+p.YLim(1);
    
    % Plot lines connecting zoomPlot to original plot points
    rectangle('Position',[xbounds(1) min(y(index)) - dif_y diff(xbounds) max(y(index))-min(y(index)) + 2*dif_y]);
    hold on
    if any(vertex==1)
        plot([xbounds(1) x1], [max(y(index)) + dif_y y2], 'k', 'LineWidth', 0.05, 'Color', [193, 193, 215]/255); % Line to vertex 1
    end
    if any(vertex==2)
        plot([xbounds(2) x2], [max(y(index)) + dif_y y2], 'k', 'LineWidth', 0.05, 'Color', [193, 193, 215]/255); % Line to vertex 2
    end
    if any(vertex==3)
        plot([xbounds(2) x2], [min(y(index)) - dif_y y1], 'k', 'LineWidth', 0.05, 'Color', [193, 193, 215]/255); % Line to vertex 4
    end
    if any(vertex==4)
        plot([xbounds(1) x1], [min(y(index)) - dif_y y1], 'k', 'LineWidth', 0.05, 'Color', [193, 193, 215]/255); % Line to vertex 3
    end
    
    % Plot zoomPlot and change axis
    z = axes('position', pos);
    box on 
    hold on
    if include_DA == 0
        for x_ind = 1:length(x)
            plot(x(x_ind), y(x_ind), markers{x_ind}, 'MarkerSize', marker_sizes{x_ind})
        end
        Colors = {[91, 91, 91]/255, [191, 99, 46]/255};
        for x_ind = 1:length(x_DA_ID)
            plot(x_DA_ID(x_ind), y_DA_ID(x_ind), 'x', 'Color', Colors{x_ind}, 'MarkerSize', 7)
        end
    elseif include_DA == 1
        for x_ind = 1:length(x) - 2
            plot(x(x_ind), y(x_ind), markers{x_ind}, 'MarkerSize', marker_sizes{x_ind})
        end
        Colors = {[91, 91, 91]/255, [191, 99, 46]/255};
        for x_ind = length(x) - 1:length(x)
            plot(x(x_ind), y(x_ind), 'x', 'Color', Colors{x_ind - length(x) + 2}, 'MarkerSize', 7)
        end
    end
    xline(line_x, ':')
    yline(line_y, ':')
    axis([xbounds(1) xbounds(2) min(y(index)) - dif_y max(y(index)) + dif_y]);
    change_ticks_no(3, 'y')
    change_ticks_no(3, 'x')
    ytickformat('%.2f')
    xtickformat('%.1f')
    ax = gca;
    ax.FontSize = 4; 
end
