addpath('Functions')
close all

country = 'PL';
show_difference = 0; % show heat maps of difference between quantiles of two samples, without testing
show_pval = 0; % show p-values of test in heat maps
quantile_buyer = 0.95; % quantiles to be used in procedure
quantile_seller = 0.05;
alpha = 0.05; % significance level

compare_option = 1; % 0 - VaR, 1 - std (variance equality test)

load(['Wallet_values/', country, '_wallet_values_buyer_GARCH_GARCH__.mat'])
Buyer_GARCH_GARCH__ = Wallet_values;
load(['Wallet_values/', country, '_wallet_values_buyer_GARCH_GARCH_asinh_asinh.mat'])
Buyer_GARCH_GARCH_asinh_asinh = Wallet_values;
load(['Wallet_values/', country, '_wallet_values_seller_GARCH_GARCH__.mat'])
Seller_GARCH_GARCH__ = Wallet_values;
load(['Wallet_values/', country, '_wallet_values_seller_GARCH_GARCH_asinh_asinh.mat'])
Seller_GARCH_GARCH_asinh_asinh = Wallet_values;

load(['Wallet_values/', country, '_wallet_values_buyer_ARX__.mat'])
Buyer_ARX__ = Wallet_values;
load(['Wallet_values/', country, '_wallet_values_buyer_ARX_asinh_asinh.mat'])
Buyer_ARX_asinh_asinh = Wallet_values;
load(['Wallet_values/', country, '_wallet_values_seller_ARX__.mat'])
Seller_ARX__ = Wallet_values;
load(['Wallet_values/', country, '_wallet_values_seller_ARX_asinh_asinh.mat'])
Seller_ARX_asinh_asinh = Wallet_values;

if strcmp(country, 'PL')
    load(['Wallet_values/', country, '_wallet_values_buyer_GARCH_GARCH__asinh.mat'])
    Buyer_GARCH_GARCH__asinh = Wallet_values;
    load(['Wallet_values/', country, '_wallet_values_seller_GARCH_GARCH__asinh.mat'])
    Seller_GARCH_GARCH__asinh = Wallet_values;

    load(['Wallet_values/', country, '_wallet_values_buyer_ARX__asinh.mat'])
    Buyer_ARX__asinh = Wallet_values;
    load(['Wallet_values/', country, '_wallet_values_seller_ARX__asinh.mat'])
    Seller_ARX__asinh = Wallet_values;
end

IS_country_DA = load(['../../../Data_MATLAB_Drive/Data/', 'GARCH', '_simulation_results_', country, '_DA_model_7_transformation__dynamic_choice_0_mc_0_dist_g.mat'],...
    'samples', 'samples_no_garch', 'real_value', 'prediction');
IS_country_ID = load(['../../../Data_MATLAB_Drive/Data/', 'GARCH', '_simulation_results_', country, '_ID_model_7_transformation__dynamic_choice_0_mc_0_dist_g.mat'],...
    'samples', 'samples_no_garch', 'real_value', 'prediction');

% extracting ID and DA means over intervals
Thresholds = [7, 13, 18, 22];
Mean_DA = ones(size(Thresholds, 2), size(Buyer_GARCH_GARCH_asinh_asinh, 3)) * -999;
Mean_ID = ones(size(Thresholds, 2), size(Buyer_GARCH_GARCH_asinh_asinh, 3)) * -999;
for threshold = 2:length(Thresholds) + 1
    h_0 = Thresholds(threshold - 1);
    if threshold == length(Thresholds) + 1
        h_1 = Thresholds(1) - 1;
        Mean_DA(threshold - 1, :) = mean([IS_country_DA.real_value(h_0:24, :); IS_country_DA.real_value(1:h_1, :)]);
        Mean_ID(threshold - 1, :) = mean([IS_country_ID.real_value(h_0:24, :); IS_country_ID.real_value(1:h_1, :)]);
    else
        h_1 = Thresholds(threshold) - 1;
        Mean_DA(threshold - 1, :) = mean(IS_country_DA.real_value(h_0:h_1, :));
        Mean_ID(threshold - 1, :) = mean(IS_country_ID.real_value(h_0:h_1, :));
    end
end

%% Buyer
Measures = {'std', 'sharpe', 'semistd_down', 'semistd_up', 'std_and_profit', 'mean_and_std', 'mean', 'DA', 'ID'};
Hours_intervals = {'7-12', '13-17', '18-21', '22-6'};
pvalmatyrice = ones([length(Measures), length(Measures)]) * -999;
if strcmp(country, 'PL')
    Models = {Buyer_GARCH_GARCH__, Buyer_GARCH_GARCH__asinh Buyer_GARCH_GARCH_asinh_asinh, Buyer_ARX__, Buyer_ARX__asinh, Buyer_ARX_asinh_asinh};
    Models_names = {'GARCH', 'GARCH 1xasinh', 'GARCH 2xasinh', 'ARX', 'ARX 1xasinh', 'ARX 2xasinh'}; % 1xasinh means that we use asinh only for ID market, 2x - both markets
elseif strcmp(country, 'GE')
    Models = {Buyer_GARCH_GARCH__, Buyer_GARCH_GARCH_asinh_asinh, Buyer_ARX__, Buyer_ARX_asinh_asinh};
    Models_names = {'GARCH', 'GARCH asinh', 'ARX', 'ARX asinh'};
end
% adding DA and ID to the measures
for model = 1:length(Models)
    Models{model}(:, length(Measures) - 1, :) = Mean_DA;
    Models{model}(:, length(Measures), :) = Mean_ID;
end

for hour_interval = 1:size(Buyer_GARCH_GARCH_asinh_asinh, 1)
    figure(hour_interval + size(Buyer_GARCH_GARCH_asinh_asinh, 1))
    for model = 1:length(Models_names)
        P_val_matrice = zeros([length(Measures), length(Measures)]);
        P_val_matrice(tril(ones(size(P_val_matrice))) == 1) = nan;
        if strcmp(country, 'PL')
            subplot(2, 3, model)
        elseif strcmp(country, 'GE')
            subplot(2, 2, model)
        end
        for m = 1:length(Measures)
            for n = 1:length(Measures)
                if compare_option ~= 1
                    if show_pval
                        for alpha = 0.99:-0.01:0
                            h = quantile_inf(squeeze(Models{model}(hour_interval, m, :)), ...
                                    squeeze(Models{model}(hour_interval, n, :)), quantile_buyer, 0, 0, alpha);
                            if h == 0
                                alpha_stop = alpha;
                                break
                            end
                        end
                        pvalmatyrice(n, m) = alpha_stop;
                    elseif show_difference
                        pvalmatyrice(n, m) = quantile(squeeze(Models{model}(hour_interval, m, :)), quantile_buyer) - quantile(squeeze(Models{model}(hour_interval, n, :)), quantile_buyer);
                    else
                        pvalmatyrice(n, m) = quantile_inf(squeeze(Models{model}(hour_interval, m, :)), ...
                                                    squeeze(Models{model}(hour_interval, n, :)), quantile_buyer, 0, 0, alpha);
                    end
                elseif compare_option
                    if show_pval
                        pvalmatyrice(n, m) = vartestn([squeeze(Models{model}(hour_interval, m, :)), ...
                                squeeze(Models{model}(hour_interval, n, :))], ...
                                'TestType', 'BrownForsythe', 'Display', 'off');
                    elseif show_difference
                        pvalmatyrice(n, m) = std(squeeze(Models{model}(hour_interval, m, :))) - std(squeeze(Models{model}(hour_interval, n, :)));
                    else
                        pvalmatyrice(n, m) = vartestn([squeeze(Models{model}(hour_interval, m, :)), ...
                                squeeze(Models{model}(hour_interval, n, :))], ...
                                'TestType', 'BrownForsythe', 'Display', 'off') < alpha;
                    end
                end
            end
        end
        P_val_matrice = P_val_matrice + pvalmatyrice;
        h = heatmap(Measures, Measures, round(P_val_matrice, 3));
        h.FontSize = 12;
        caxis([0 1])
        title(['Model: ', Models_names{model}])
        sgtitle(['Buyer, hour interval: ', Hours_intervals{hour_interval}])
    end
end

%% Seller
Measures = {'std', 'sharpe', 'semistd_down', 'semistd_up', 'std_and_profit', 'mean_and_std', 'mean', 'DA', 'ID'};
Hours_intervals = {'7-12', '13-17', '18-21', '22-6'};
pvalmatyrice = ones([length(Measures), length(Measures)]) * -999;
if strcmp(country, 'PL')
    Models = {Seller_GARCH_GARCH__, Seller_GARCH_GARCH__asinh Seller_GARCH_GARCH_asinh_asinh, Seller_ARX__, Seller_ARX__asinh, Seller_ARX_asinh_asinh};
    Models_names = {'GARCH', 'GARCH 1xasinh', 'GARCH 2xasinh', 'ARX', 'ARX 1xasinh', 'ARX 2xasinh'}; % 1xasinh means that we use asinh only for ID market, 2x - both markets
elseif strcmp(country, 'GE')
    Models = {Seller_GARCH_GARCH__, Seller_GARCH_GARCH_asinh_asinh, Seller_ARX__, Seller_ARX_asinh_asinh};
    Models_names = {'GARCH', 'GARCH asinh', 'ARX', 'ARX asinh'};
end
% adding DA and ID to the measures
for model = 1:length(Models)
    Models{model}(:, length(Measures) - 1, :) = Mean_DA;
    Models{model}(:, length(Measures), :) = Mean_ID;
end

for hour_interval = 1:size(Seller_GARCH_GARCH_asinh_asinh, 1)
    figure(hour_interval)
    for model = 1:length(Models_names)
        P_val_matrice = zeros([length(Measures), length(Measures)]);
        P_val_matrice(tril(ones(size(P_val_matrice))) == 1) = nan;
        if strcmp(country, 'PL')
            subplot(2, 3, model)
        elseif strcmp(country, 'GE')
            subplot(2, 2, model)
        end
        for m = 1:length(Measures)
            for n = 1:length(Measures)
                if compare_option ~= 1
                    if show_pval
                        for alpha = 0.99:-0.01:0
                            h = quantile_inf(squeeze(Models{model}(hour_interval, m, :)), ...
                                    squeeze(Models{model}(hour_interval, n, :)), quantile_seller, 0, 0, alpha);
                            if h == 0
                                alpha_stop = alpha;
                                break
                            end
                        end
                        pvalmatyrice(n, m) = alpha_stop;
                    elseif show_difference
                        pvalmatyrice(n, m) = quantile(squeeze(Models{model}(hour_interval, m, :)), quantile_seller) - quantile(squeeze(Models{model}(hour_interval, n, :)), quantile_seller);
                    else
                        pvalmatyrice(n, m) = quantile_inf(squeeze(Models{model}(hour_interval, m, :)), ...
                                                    squeeze(Models{model}(hour_interval, n, :)), quantile_seller, 0, 0, alpha);
                    end
                elseif compare_option
                    if show_pval
                        pvalmatyrice(n, m) = vartestn([squeeze(Models{model}(hour_interval, m, :)), ...
                                squeeze(Models{model}(hour_interval, n, :))], ...
                                'TestType', 'BrownForsythe', 'Display', 'off');
                    elseif show_difference
                        pvalmatyrice(n, m) = std(squeeze(Models{model}(hour_interval, m, :))) - std(squeeze(Models{model}(hour_interval, n, :)));
                    else
                        pvalmatyrice(n, m) = vartestn([squeeze(Models{model}(hour_interval, m, :)), ...
                                squeeze(Models{model}(hour_interval, n, :))], ...
                                'TestType', 'BrownForsythe', 'Display', 'off') < alpha;
                    end
                end
            end
        end
        P_val_matrice = P_val_matrice + pvalmatyrice;
        h = heatmap(Measures, Measures, round(P_val_matrice, 3));
        h.FontSize = 12;
        caxis([0 1])
        title(['Model: ', Models_names{model}])
        sgtitle(['Seller, hour interval: ', Hours_intervals{hour_interval}])
    end
end
