clear all
close all
addpath('Functions')
alpha = 0.05; % significance level for every DM test in notebook
start_eval=1;
mc='0';
options = optimoptions(@fmincon,'Algorithm','interior-point');
lags_arch = 1;
lags_acorr = 1;
fignum = 1;
Paper_width = 6.47699; % width of plot for TeX document (not universal - dependent on margins used!)

%% GE DA
close all
disp("GE DA")
Asinh = load(['../../../Data_MATLAB_Drive/Data/GARCH_simulation_results_GE_DA_model_7_transformation_',...
        'asinh', '_dynamic_choice_0_mc_',mc ,'_dist_g.mat'],...
            'mae_hourly', 'rmse_hourly', 'archtest_results',...
            'kstest_results', 'kstesttstud_results', 'real_value', 'prediction','samples_no_garch', 'residua');

Without_asinh = load(['../../../Data_MATLAB_Drive/Data/GARCH_simulation_results_GE_DA_model_7_transformation_',...
        '', '_dynamic_choice_0_mc_',mc ,'_dist_g.mat'],...
            'mae_hourly', 'rmse_hourly', 'archtest_results',...
            'kstest_results', 'kstesttstud_results', 'real_value', 'prediction', 'residua');

Asinh_results = ones(24, 1088);
No_asinh_results = ones(24, 1088);
Asinh_autocorr_results = ones(24, 1088);
No_asinh_autocorr_results = ones(24, 1088);

figure(fignum)
figure(fignum + 1)
for h = 1:24
    for t = 1:1088
        window_residuals = Asinh.residua(h, :, t);
        if ~mod(4, t)
            figure(fignum)
            subplot(2,1,1)
            hold on
            autocorr(window_residuals)
            subplot(2,1,2)
            hold on
            autocorr(window_residuals.^2)
        end
        Asinh_results(h, t) = archtest(window_residuals, Lags=lags_arch);
        Asinh_autocorr_results(h, t) = lbqtest(window_residuals, Lags=lags_acorr);
        
        window_residuals = Without_asinh.residua(h, :, t);
        if ~mod(4, t)
            figure(fignum + 1)
            subplot(2,1,1)
            hold on
            autocorr(window_residuals)
            subplot(2,1,2)
            hold on
            autocorr(window_residuals.^2)
        end
        No_asinh_results(h, t) = archtest(window_residuals, Lags=lags_arch);
        No_asinh_autocorr_results(h, t) = lbqtest(window_residuals, Lags=lags_acorr);
    end
    disp(h)
end
figure(fignum)
subplot(2,1,1)
title("Asinh res. GE DA")
subplot(2,1,2)
title("Asinh res.^2 GE DA")

figure(fignum + 1)
subplot(2,1,1)
title("res. GE DA")
subplot(2,1,2)
title("res.^2 GE DA")

disp("(Asinh) Fraction of rejected no ARCH hypotheses:")
mean(sum(Asinh_results, 2)) / 1088
disp("(Asinh) Fraction of rejected no autocorr. hypotheses:")
mean(sum(Asinh_autocorr_results, 2)) / 1088
noautocorr_arch = Asinh_results + Asinh_autocorr_results == 2;
disp("(Asinh) Fraction of rejected no autocorr. hypotheses & rejected no ARCH hypotheses:")
mean(sum(noautocorr_arch, 2)) / 1088

disp("(No asinh) Fraction of rejected no ARCH hypotheses:")
mean(sum(No_asinh_results, 2)) / 1088
disp("(No asinh) Fraction of rejected no autocorr. hypotheses:")
mean(sum(No_asinh_autocorr_results, 2)) / 1088
noautocorr_arch = No_asinh_results + No_asinh_autocorr_results == 2;
disp("(No asinh) Fraction of rejected no autocorr. hypotheses & rejected no ARCH hypotheses:")
mean(sum(noautocorr_arch, 2)) / 1088

% set width and save figure
set(fignum, 'units', 'inches', 'pos', [0 0 Paper_width 1.5 * Paper_width])
pos = get(fignum,'Position');
set(fignum, 'PaperPositionMode', 'Auto', 'PaperUnits', 'Inches', 'PaperSize', [pos(3), pos(4)])
print(fignum, 'Figures/autocorr_res_GE_DA.pdf', '-dpdf')

set(fignum + 1, 'units', 'inches', 'pos', [0 0 Paper_width 1.5 * Paper_width])
pos = get(fignum + 1,'Position');
set(fignum + 1, 'PaperPositionMode', 'Auto', 'PaperUnits', 'Inches', 'PaperSize', [pos(3), pos(4)])
print(fignum + 1, 'Figures/autocorr_res2_GE_DA.pdf', '-dpdf')

fignum = fignum + 2;

%% GE ID
close all

disp("GE ID")
Asinh = load(['../../../Data_MATLAB_Drive/Data/GARCH_simulation_results_GE_ID_model_7_transformation_',...
        'asinh', '_dynamic_choice_0_mc_',mc ,'_dist_g.mat'],...
            'mae_hourly', 'rmse_hourly', 'archtest_results',...
            'kstest_results', 'kstesttstud_results', 'real_value', 'prediction','samples_no_garch', 'residua');

Without_asinh = load(['../../../Data_MATLAB_Drive/Data/GARCH_simulation_results_GE_ID_model_7_transformation_',...
        '', '_dynamic_choice_0_mc_',mc ,'_dist_g.mat'],...
            'mae_hourly', 'rmse_hourly', 'archtest_results',...
            'kstest_results', 'kstesttstud_results', 'real_value', 'prediction', 'residua');

Asinh_results = ones(24, 1088);
Asinh_gjr_results = ones(24, 1088);
No_asinh_results = ones(24, 1088);
Asinh_autocorr_results = ones(24, 1088);
Asinh_gjr_autocorr_results = ones(24, 1088);
No_asinh_autocorr_results = ones(24, 1088);

figure(fignum)
figure(fignum + 1)
for h = 1:24
    for t = 1:1088
        window_residuals = Asinh.residua(h, :, t);
        if ~mod(4, t)
            figure(fignum)
            subplot(2,1,1)
            hold on
            autocorr(window_residuals)
            subplot(2,1,2)
            hold on
            autocorr(window_residuals.^2)
        end
        Asinh_results(h, t) = archtest(window_residuals, Lags=lags_arch);
        Asinh_autocorr_results(h, t) = lbqtest(window_residuals, Lags=lags_acorr);
        
        window_residuals = Without_asinh.residua(h, :, t);
        if ~mod(4, t)
            figure(fignum + 1)
            subplot(2,1,1)
            hold on
            autocorr(window_residuals)
            subplot(2,1,2)
            hold on
            autocorr(window_residuals.^2)
        end
        No_asinh_results(h, t) = archtest(window_residuals, Lags=lags_arch);
        No_asinh_autocorr_results(h, t) = lbqtest(window_residuals, Lags=lags_acorr);
    end
    disp(h)
end
figure(fignum)
subplot(2,1,1)
title("Asinh res. GE ID")
subplot(2,1,2)
title("Asinh res.^2 GE ID")

figure(fignum + 1)
subplot(2,1,1)
title("res. GE ID")
subplot(2,1,2)
title("res.^2 GE ID")

disp("(Asinh) Fraction of rejected no ARCH hypotheses:")
mean(sum(Asinh_results, 2)) / 1088
disp("(Asinh) Fraction of rejected no autocorr. hypotheses:")
mean(sum(Asinh_autocorr_results, 2)) / 1088
noautocorr_arch = Asinh_results + Asinh_autocorr_results == 2;
disp("(Asinh) Fraction of rejected no autocorr. hypotheses & rejected no ARCH hypotheses:")
mean(sum(noautocorr_arch, 2)) / 1088

disp("(No asinh) Fraction of rejected no ARCH hypotheses:")
mean(sum(No_asinh_results, 2)) / 1088
disp("(No asinh) Fraction of rejected no autocorr. hypotheses:")
mean(sum(No_asinh_autocorr_results, 2)) / 1088
noautocorr_arch = No_asinh_results + No_asinh_autocorr_results == 2;
disp("(No asinh) Fraction of rejected no autocorr. hypotheses & rejected no ARCH hypotheses:")
mean(sum(noautocorr_arch, 2)) / 1088

% set width and save figure
set(fignum, 'units', 'inches', 'pos', [0 0 Paper_width 1.5 * Paper_width])
pos = get(fignum,'Position');
set(fignum, 'PaperPositionMode', 'Auto', 'PaperUnits', 'Inches', 'PaperSize', [pos(3), pos(4)])
print(fignum, 'Figures/autocorr_res_GE_ID.pdf', '-dpdf')

set(fignum + 1, 'units', 'inches', 'pos', [0 0 Paper_width 1.5 * Paper_width])
pos = get(fignum + 1,'Position');
set(fignum + 1, 'PaperPositionMode', 'Auto', 'PaperUnits', 'Inches', 'PaperSize', [pos(3), pos(4)])
print(fignum + 1, 'Figures/autocorr_res2_GE_ID.pdf', '-dpdf')

fignum = fignum + 2;

%% PL DA
disp("PL DA")
Asinh = load(['../../../Data_MATLAB_Drive/Data/GARCH_simulation_results_PL_DA_model_7_transformation_',...
        'asinh', '_dynamic_choice_0_mc_', mc,'_dist_t.mat'],...
            'mae_hourly', 'rmse_hourly', 'archtest_results',...
            'kstest_results', 'kstesttstud_results', 'real_value', 'prediction','samples_no_garch', 'residua');

Without_asinh = load(['../../../Data_MATLAB_Drive/Data/GARCH_simulation_results_PL_DA_model_7_transformation_',...
        '', '_dynamic_choice_0_mc_', mc,'_dist_t.mat'],...
            'mae_hourly', 'rmse_hourly', 'archtest_results',...
            'kstest_results', 'kstesttstud_results', 'real_value', 'prediction', 'residua');

Asinh_results = ones(24, 1088);
No_asinh_results = ones(24, 1088);
Asinh_autocorr_results = ones(24, 1088);
No_asinh_autocorr_results = ones(24, 1088);

figure(fignum)
figure(fignum + 1)
for h = 1:24
    for t = 1:1088
        window_residuals = Asinh.residua(h, :, t);
        if ~mod(4, t)
            figure(fignum)
            subplot(2,1,1)
            hold on
            autocorr(window_residuals)
            subplot(2,1,2)
            hold on
            autocorr(window_residuals.^2)
        end
        Asinh_results(h, t) = archtest(window_residuals, Lags=lags_arch);
        Asinh_autocorr_results(h, t) = lbqtest(window_residuals, Lags=lags_acorr);
        
        window_residuals = Without_asinh.residua(h, :, t);
        if ~mod(4, t)
            figure(fignum + 1)
            subplot(2,1,1)
            hold on
            autocorr(window_residuals)
            subplot(2,1,2)
            hold on
            autocorr(window_residuals.^2)
        end
        No_asinh_results(h, t) = archtest(window_residuals, Lags=lags_arch);
        No_asinh_autocorr_results(h, t) = lbqtest(window_residuals, Lags=lags_acorr);
    end
    disp(h)
end
figure(fignum)
subplot(2,1,1)
title("Asinh res. PL DA")
subplot(2,1,2)
title("Asinh res.^2 PL DA")

figure(fignum + 1)
subplot(2,1,1)
title("res. PL DA")
subplot(2,1,2)
title("res.^2 PL DA")

disp("(Asinh) Fraction of rejected no ARCH hypotheses:")
mean(sum(Asinh_results, 2)) / 1088
disp("(Asinh) Fraction of rejected no autocorr. hypotheses:")
mean(sum(Asinh_autocorr_results, 2)) / 1088
noautocorr_arch = Asinh_results + Asinh_autocorr_results == 2;
disp("(Asinh) Fraction of rejected no autocorr. hypotheses & rejected no ARCH hypotheses:")
mean(sum(noautocorr_arch, 2)) / 1088

disp("(No asinh) Fraction of rejected no ARCH hypotheses:")
mean(sum(No_asinh_results, 2)) / 1088
disp("(No asinh) Fraction of rejected no autocorr. hypotheses:")
mean(sum(No_asinh_autocorr_results, 2)) / 1088
noautocorr_arch = No_asinh_results + No_asinh_autocorr_results == 2;
disp("(No asinh) Fraction of rejected no autocorr. hypotheses & rejected no ARCH hypotheses:")
mean(sum(noautocorr_arch, 2)) / 1088

% set width and save figure
set(fignum, 'units', 'inches', 'pos', [0 0 Paper_width 1.5 * Paper_width])
pos = get(fignum,'Position');
set(fignum, 'PaperPositionMode', 'Auto', 'PaperUnits', 'Inches', 'PaperSize', [pos(3), pos(4)])
print(fignum, 'Figures/autocorr_res_PL_DA.pdf', '-dpdf')

set(fignum + 1, 'units', 'inches', 'pos', [0 0 Paper_width 1.5 * Paper_width])
pos = get(fignum + 1,'Position');
set(fignum + 1, 'PaperPositionMode', 'Auto', 'PaperUnits', 'Inches', 'PaperSize', [pos(3), pos(4)])
print(fignum + 1, 'Figures/autocorr_res2_PL_DA.pdf', '-dpdf')

fignum = fignum + 2;

%% PL ID
disp("PL ID")
Asinh = load(['../../../Data_MATLAB_Drive/Data/GARCH_simulation_results_PL_ID_model_7_transformation_',...
        'asinh', '_dynamic_choice_0_mc_', mc,'_dist_t.mat'],...
            'mae_hourly', 'rmse_hourly', 'archtest_results',...
            'kstest_results', 'kstesttstud_results', 'real_value', 'prediction','samples_no_garch', 'residua');

Without_asinh = load(['../../../Data_MATLAB_Drive/Data/GARCH_simulation_results_PL_ID_model_7_transformation_',...
        '', '_dynamic_choice_0_mc_', mc,'_dist_t.mat'],...
            'mae_hourly', 'rmse_hourly', 'archtest_results',...
            'kstest_results', 'kstesttstud_results', 'real_value', 'prediction', 'residua');

Asinh_results = ones(24, 1088);
No_asinh_results = ones(24, 1088);
Asinh_autocorr_results = ones(24, 1088);
No_asinh_autocorr_results = ones(24, 1088);

figure(fignum)
figure(fignum + 1)
for h = 1:24
    for t = 1:1088
        window_residuals = Asinh.residua(h, :, t);
        if ~mod(4, t)
            figure(fignum)
            subplot(2,1,1)
            hold on
            autocorr(window_residuals)
            subplot(2,1,2)
            hold on
            autocorr(window_residuals.^2)
        end
        Asinh_results(h, t) = archtest(window_residuals, Lags=lags_arch);
        Asinh_autocorr_results(h, t) = lbqtest(window_residuals, Lags=lags_acorr);
        
        window_residuals = Without_asinh.residua(h, :, t);
        if ~mod(4, t)
            figure(fignum + 1)
            subplot(2,1,1)
            hold on
            autocorr(window_residuals)
            subplot(2,1,2)
            hold on
            autocorr(window_residuals.^2)
        end
        No_asinh_results(h, t) = archtest(window_residuals, Lags=lags_arch);
        No_asinh_autocorr_results(h, t) = lbqtest(window_residuals, Lags=lags_acorr);
    end
    disp(h)
end
figure(fignum)
subplot(2,1,1)
title("Asinh res. PL ID")
subplot(2,1,2)
title("Asinh res.^2 PL ID")

figure(fignum + 1)
subplot(2,1,1)
title("res. PL ID")
subplot(2,1,2)
title("res.^2 PL ID")

disp("(Asinh) Fraction of rejected no ARCH hypotheses:")
mean(sum(Asinh_results, 2)) / 1088
disp("(Asinh) Fraction of rejected no autocorr. hypotheses:")
mean(sum(Asinh_autocorr_results, 2)) / 1088
noautocorr_arch = Asinh_results + Asinh_autocorr_results == 2;
disp("(Asinh) Fraction of rejected no autocorr. hypotheses & rejected no ARCH hypotheses:")
mean(sum(noautocorr_arch, 2)) / 1088

disp("(No asinh) Fraction of rejected no ARCH hypotheses:")
mean(sum(No_asinh_results, 2)) / 1088
disp("(No asinh) Fraction of rejected no autocorr. hypotheses:")
mean(sum(No_asinh_autocorr_results, 2)) / 1088
noautocorr_arch = No_asinh_results + No_asinh_autocorr_results == 2;
disp("(No asinh) Fraction of rejected no autocorr. hypotheses & rejected no ARCH hypotheses:")
mean(sum(noautocorr_arch, 2)) / 1088

% set width and save figure
set(fignum, 'units', 'inches', 'pos', [0 0 Paper_width 1.5 * Paper_width])
pos = get(fignum,'Position');
set(fignum, 'PaperPositionMode', 'Auto', 'PaperUnits', 'Inches', 'PaperSize', [pos(3), pos(4)])
print(fignum, 'Figures/autocorr_res_PL_ID.pdf', '-dpdf')

set(fignum + 1, 'units', 'inches', 'pos', [0 0 Paper_width 1.5 * Paper_width])
pos = get(fignum + 1,'Position');
set(fignum + 1, 'PaperPositionMode', 'Auto', 'PaperUnits', 'Inches', 'PaperSize', [pos(3), pos(4)])
print(fignum + 1, 'Figures/autocorr_res2_PL_ID.pdf', '-dpdf')

fignum = fignum + 2;
