% script checks the specification of GARCH performing ARCH(p) test on
% ARX residues with removed autocorrelation. This removal is done according
% to method of adding recursive residuals to the model (Lumsdaine & Ng) or 
% using method of Taylor series suggested (Maki & Ota).
clear all

data = importdata('../Data/data_Poland_20170101_20201231.txt');

start = 373; % start of eval window (in days), one year for training + 7 days for AR
stop = size(data, 1) / 24; % end of eval window (in days)
transformation = 'asinh';
training_window = 365; % training window length
hours = 1:24; % gives control over hours that we want to simulate on

Lumsdaine_Ng = 0;
k = 200; % length of first training window for preparing recursive residuals
repeat_simu = 1; % if we want to repeat the simulations (1) or use saved data (0)
m = 3; % no. of components in Taylor series if option == 'Taylor'
rolling = 0; % if rolling == 0 we use expanding window as in the oryginal paper
option = 'Taylor'; % choose 'Taylor' or 'cumsum'

Maki_Ota = 1;

if repeat_simu
    if Lumsdaine_Ng
        residues = ones(length(hours), stop - start, training_window-k)*-999;
    elseif Maki_Ota
        residues = ones(length(hours), stop - start, training_window)*-999;
    end
    test_residues = ones(length(hours), stop - start, training_window-k-1)*-999;
    for hour = hours
        tmp = data;
        test_residues_row = ones(stop - start, training_window-k-1)*-999;
        if Lumsdaine_Ng
            residues_row = ones(stop - start, training_window-k)*-999;
        elseif Maki_Ota
            residues_row = ones(stop - start, training_window)*-999;
        end
        X = [];
        for  day = start + 1:stop
            data_window = tmp(day * 24 - (training_window * 24) - (8 * 24) + 1:day * 24,:);
            if strcmp(transformation, 'asinh')
                % "Price" - vector of prices for each hour separately
                [standardized_price, mu_price, sigma_price] =...
                    zscore(data_window(hour:24:end-24, 3));
                price = asinh(standardized_price);
                % Minimum of the previous day
                standardized_price_min = ...
                    zscore(data_window(hour:24:end-24,17));
                price_min = asinh(standardized_price_min);
                % Maximum of the previous day
                standardized_price_max = ...
                    zscore(data_window(hour:24:end-24,18));
                price_max = asinh(standardized_price_max);
            elseif strcmp(transformation, '')
                % "Price" - vector of prices for each hour separately
                price = data_window(hour:24:end-24,3);
                % Minimum of the previous day
                price_min = data_window(hour:24:end-24,17);

                % Maximum of the previous day
                price_max = data_window(hour:24:end-24,18);
            end

            % Forecast of energy demand
            demand = data_window(hour:24:end,5);

            % Forecast of wind generation
            wind = data_window(hour:24:end,6);

            % Forecast of reserves
            reserves = data_window(hour:24:end,7);

            % Imbalance
            imbalance = data_window(hour:24:end-24,8);

            % Co2
            Co2 = data_window(hour:24:end-24,9);

            % Gas
            gas = data_window(hour:24:end-24,10);

            % Temperature
            temp = data_window(hour:24:end,11);

            % Dummies
            D_Sat = data_window(hour:24:end,13);
            D_Sun = data_window(hour:24:end,14);
            D_Mon = data_window(hour:24:end,15);
            D_rest = data_window(hour:24:end,16);
            Y = price(8:end);
            % Model '7' - X1,X2,X3,X4,X5,X6 L =[1,2,...,7]
            X = [price(7:end-1), price(6:end-2),...
                price(5:end-3), price(4:end-4),...
                price(3:end-5), price(2:end-6), price(1:end-7),...
                demand(8:end-1), wind(8:end-1), reserves(8:end-1),...
                imbalance(7:end-1), Co2(7:end-1), gas(7:end-1),...
                temp(8:end-1), price_min(7:end-1),price_max(7:end-1),...
                D_Sat(8:end-1),D_Sun(8:end-1),D_Mon(8:end-1),D_rest(8:end-1)];
            % AR prices
            price_mat = ones(training_window, 7)*-999;
            for i = 1:7
                for t = 8:length(price)
                    price_mat(t - 7, i) = price(t - i);
                end
            end

            % AR prices from the double sum
            price_mat_multip = ones(training_window, 7^2)*-999;
            counter = 1;
            for i = 1:7
                for j = 1:7
                    for t = 8:length(price)
                        price_mat_multip(t - 7, counter) = price(t - i) * price(t - j);
                    end
                end
                counter = counter + 1;
            end

            % eXogenous variables
            lag_eXogenous = 2;
            eX_variables_1 = [demand(9 - lag_eXogenous:end),...
                wind(9 - lag_eXogenous:end),...
                reserves(9 - lag_eXogenous:end),...
                imbalance(8 - lag_eXogenous:end),...
                Co2(8 - lag_eXogenous:end),...
                gas(8 - lag_eXogenous:end),...
                temp(9 - lag_eXogenous:end),...
                price_min(8 - lag_eXogenous:end),...
                price_max(8 - lag_eXogenous:end)];
            eXogenous_mat = ones(training_window, lag_eXogenous, size(eX_variables_1, 2));
            for col = 1:size(eX_variables_1, 2)
                collumn = eX_variables_1(:, col);
                eXogenous_vec = ones(length(collumn) - lag_eXogenous, lag_eXogenous)*-999;
                for i = 1:lag_eXogenous
                    for t = lag_eXogenous + 1:length(collumn)
                        eXogenous_vec(t - lag_eXogenous, i) = collumn(t - i);
                    end
                end
                eXogenous_mat(:, :, col) = eXogenous_vec;
            end

            % eXogenous variables from double sum
            eXogenous_mat_multip = ones(training_window, lag_eXogenous^2, size(eX_variables_1, 2));
            for col = 1:size(eX_variables_1, 2)
                collumn = eX_variables_1(:, col);
                eXogenous_vec = ones(length(collumn) - lag_eXogenous, lag_eXogenous^2)*-999;
                counter = 1;
                for i = 1:lag_eXogenous
                    for j = 1:lag_eXogenous
                        for t = lag_eXogenous + 1:length(collumn)
                            eXogenous_vec(t - lag_eXogenous, counter) = collumn(t - i) * collumn(t - j);
                        end
                        counter = counter + 1;
                    end
                end
                eXogenous_mat_multip(:, :, col) = eXogenous_vec;
            end
            
            % putting it all together
            new_X = [price_mat, price_mat_multip,...
                D_Sat(8:end-1),D_Sun(8:end-1),...
                D_Mon(8:end-1),D_rest(8:end-1)];
            for col = 1:size(eX_variables_1, 2)
                new_X = [new_X, squeeze(eXogenous_mat_multip(:, :, col)),...
                    squeeze(eXogenous_mat(:, :, col))];
            end
            if Lumsdaine_Ng
                % recursive residuals computation
                for recursive_day = k:size(X, 1)-1
                    if rolling % rolling window
                        [beta, ~, ~]  = regress(Y(recursive_day - k + 1:recursive_day), X(recursive_day - k + 1:recursive_day, :));
                    else % expanding window
                        [beta, ~, ~]  = regress(Y(1:recursive_day), X(1:recursive_day, :));
                    end
                    price_point = X(recursive_day + 1, :) * beta;
                    err = Y(recursive_day + 1) - price_point;
                    residues_row(day - start, recursive_day - k + 1) = err;
                end
                % test residuals computation
                if strcmp(option, 'Taylor')
                    for taylor_exp = 1:m
                        X_rec = [X(k + 2:end, :), residues_row(day - start, 1:end-1)'.^taylor_exp];
                    end
                elseif strcmp(option, 'cumsum')
                    X_rec = [X(k + 2:end, :), cumsum(residues_row(day - start, 1:end-1)')];
                end
                [~, ~, test_errors]  = regress(Y(k + 2:end), X_rec);
                test_residues_row(day - start, :) = test_errors';
            elseif Maki_Ota
                % fitting ARX model
                [beta, ~, err]  = regress(Y, new_X);
                residues_row(day - start, :) = err;
%                 figure(day)
%                 plot(Y, 'DisplayName', 'data')
%                 hold on
%                 plot(new_X*beta, 'DisplayName', 'forecasts Taylor')
%                 [beta_old, ~, ~]  = regress(Y, X);
%                 plot(X*beta_old, 'DisplayName', 'forecasts')
%                 legend()
            end
%             if day > 400
%                 break
%             end
        end
%         break
        residues(hour, :, :) = residues_row;
        if Lumsdaine_Ng
            test_residues(hour, :, :) = test_residues_row;
        end
        disp(hour)
    end
    save(['specification_tests\rec_res_PL_DA', transformation, '.mat'], 'residues')
    save(['specification_tests\test_res_PL_DA', transformation, '.mat'], 'test_residues')
else
    load(['specification_tests\rec_res_PL_DA', transformation, '.mat'], 'residues');
    load(['specification_tests\test_res_PL_DA', transformation, '.mat'], 'test_residues');
end
%%
if Lumsdaine_Ng
    lags_arch = 2;
    lags_acorr = 2;
    arch_results = ones(24, 1088)*-999;
    autocorr_results = ones(24, 1088)*-999;
    for h = 1:24
        for t = 1:1088
            window_residuals = squeeze(test_residues(h, t, :));
            arch_results(h, t) = archtest(window_residuals, Lags=lags_arch);
            autocorr_results(h, t) = lbqtest(window_residuals, Lags=lags_acorr);
        end
    end
    disp("Fraction of rejected no ARCH hypotheses:")
    mean(sum(arch_results, 2)) / 1088
    disp("Fraction of rejected no autocorr. hypotheses:")
    mean(sum(autocorr_results, 2)) / 1088
    noautocorr_arch = arch_results + autocorr_results == 2;
    disp("Fraction of rejected no autocorr. hypotheses & rejected no ARCH hypotheses:")
    mean(sum(noautocorr_arch, 2)) / 1088
elseif Maki_Ota
    lags_arch = 2;
    lags_acorr = 2;
    arch_results = ones(24, 1088)*-999;
    autocorr_results = ones(24, 1088)*-999;
    for h = 1:24
        for t = 1:1088
            window_residuals = squeeze(residues(h, t, :));
            arch_results(h, t) = archtest(window_residuals, Lags=lags_arch);
            autocorr_results(h, t) = lbqtest(window_residuals, Lags=lags_acorr);
        end
    end
    disp("Fraction of rejected no ARCH hypotheses:")
    mean(sum(arch_results, 2)) / 1088
    disp("Fraction of rejected no autocorr. hypotheses:")
    mean(sum(autocorr_results, 2)) / 1088
    noautocorr_arch = arch_results + autocorr_results == 2;
    disp("Fraction of rejected no autocorr. hypotheses & rejected no ARCH hypotheses:")
    mean(sum(noautocorr_arch, 2)) / 1088
end
