addpath('Functions')
close all

% in this file we attempt to test the wallet probability forecasts

country = 'PL';
quantiles_buyer = 0.01:0.01:0.99; % you can also check one chosen quantile like 0.95 or 0.05
quantiles_seller = 0.01:0.01:0.99; % you can also check one chosen quantile like 0.95 or 0.05
alpha = 0.05;

load(['Joint_probability_forecasting_PS/', country, '_PS_buyer_GARCH_GARCH__.mat'])
Buyer_GARCH_GARCH__ = Joint_probability_forecasting_PS;
load(['Joint_probability_forecasting_PS/', country, '_PS_buyer_GARCH_GARCH_asinh_asinh.mat'])
Buyer_GARCH_GARCH_asinh_asinh = Joint_probability_forecasting_PS;
load(['Joint_probability_forecasting_PS/', country, '_PS_seller_GARCH_GARCH__.mat'])
Seller_GARCH_GARCH__ = Joint_probability_forecasting_PS;
load(['Joint_probability_forecasting_PS/', country, '_PS_seller_GARCH_GARCH_asinh_asinh.mat'])
Seller_GARCH_GARCH_asinh_asinh = Joint_probability_forecasting_PS;

load(['Joint_probability_forecasting_PS/', country, '_PS_buyer_ARX__.mat'])
Buyer_ARX__ = Joint_probability_forecasting_PS;
load(['Joint_probability_forecasting_PS/', country, '_PS_buyer_ARX_asinh_asinh.mat'])
Buyer_ARX_asinh_asinh = Joint_probability_forecasting_PS;
load(['Joint_probability_forecasting_PS/', country, '_PS_seller_ARX__.mat'])
Seller_ARX__ = Joint_probability_forecasting_PS;
load(['Joint_probability_forecasting_PS/', country, '_PS_seller_ARX_asinh_asinh.mat'])
Seller_ARX_asinh_asinh = Joint_probability_forecasting_PS;

if strcmp(country, 'PL')
    load(['Joint_probability_forecasting_PS/', country, '_PS_buyer_GARCH_GARCH__asinh.mat'])
    Buyer_GARCH_GARCH__asinh = Joint_probability_forecasting_PS;
    load(['Joint_probability_forecasting_PS/', country, '_PS_seller_GARCH_GARCH__asinh.mat'])
    Seller_GARCH_GARCH__asinh = Joint_probability_forecasting_PS;

    load(['Joint_probability_forecasting_PS/', country, '_PS_buyer_ARX__asinh.mat'])
    Buyer_ARX__asinh = Joint_probability_forecasting_PS;
    load(['Joint_probability_forecasting_PS/', country, '_PS_seller_ARX__asinh.mat'])
    Seller_ARX__asinh = Joint_probability_forecasting_PS;
end

%% Buyer
if strcmp(country, 'PL')
    Models = {Buyer_GARCH_GARCH__, Buyer_GARCH_GARCH__asinh Buyer_GARCH_GARCH_asinh_asinh, Buyer_ARX__, Buyer_ARX__asinh, Buyer_ARX_asinh_asinh};
    labels = {'GARCH', 'GARCH 1xasinh', 'GARCH 2xasinh', 'ARX', 'ARX 1xasinh', 'ARX 2xasinh'}; % 1xasinh means that we use asinh only for ID market, 2x - both markets
elseif strcmp(country, 'GE')
    Models = {Buyer_GARCH_GARCH__, Buyer_GARCH_GARCH_asinh_asinh, Buyer_ARX__, Buyer_ARX_asinh_asinh};
    labels = {'GARCH', 'GARCH asinh', 'ARX', 'ARX asinh'};
end
Measures = {'std', 'sharpe', 'semistd_down', 'semistd_up', 'std_and_profit', 'mean_and_std', 'mean'};
Hours_intervals = {'7-12', '13-17', '18-21', '22-6'};
pvalmatyrice = ones([length(Models), length(Models)]) * -999;
for measure = 1:size(Buyer_GARCH_GARCH_asinh_asinh, 2)
    figure(measure)
    for hour_interval = 1:size(Buyer_GARCH_GARCH_asinh_asinh, 1)
        P_val_matrice = zeros([length(Models), length(Models)]);
        P_val_matrice(eye(size(P_val_matrice)) == 1) = nan;
        subplot(2, 2, hour_interval)
        P_vals = ones(length(Models), length(Models)) * -999;
        for q = quantiles_buyer
            ind = find(0.01:0.01:0.99 == q);
            for m = 1:length(Models)
                for n = 1:length(Models)
                    [DM, p] = dmtest(squeeze(Models{m}(hour_interval, measure, :, ind)), squeeze(Models{n}(hour_interval, measure, :, ind)));
                    pvalmatyrice(n, m) = p;
                end
            end
            if length(quantiles_buyer) == 1
                P_val_matrice = P_val_matrice + pvalmatyrice;
            else
                count = pvalmatyrice < alpha;
                P_val_matrice = P_val_matrice + count;
            end
        end
        h = heatmap(labels, labels, round(P_val_matrice, 3));
        h.FontSize = 6;
        title(['Hour interval: ', Hours_intervals{hour_interval}])
        sgtitle(['Buyer, measure: ', Measures(measure)])
    end
end

%% Seller
if strcmp(country, 'PL')
    Models = {Seller_GARCH_GARCH__, Seller_GARCH_GARCH__asinh Seller_GARCH_GARCH_asinh_asinh, Seller_ARX__, Seller_ARX__asinh, Seller_ARX_asinh_asinh};
    labels = {'GARCH', 'GARCH 1xasinh', 'GARCH 2xasinh', 'ARX', 'ARX 1xasinh', 'ARX 2xasinh'}; % 1xasinh means that we use asinh only for ID market, 2x - both markets
elseif strcmp(country, 'GE')
    Models = {Seller_GARCH_GARCH__, Seller_GARCH_GARCH_asinh_asinh, Seller_ARX__, Seller_ARX_asinh_asinh};
    labels = {'GARCH', 'GARCH asinh', 'ARX', 'ARX asinh'};
end
Measures = {'std', 'sharpe', 'semistd_down', 'semistd_up', 'std_and_profit', 'mean_and_std', 'mean'};
Hours_intervals = {'7-12', '13-17', '18-21', '22-6'};
pvalmatyrice = ones([length(Models), length(Models)]) * -999;
for measure = 1:size(Seller_GARCH_GARCH_asinh_asinh, 2)
    figure(measure + size(Seller_GARCH_GARCH_asinh_asinh, 2))
    for hour_interval = 1:size(Seller_GARCH_GARCH_asinh_asinh, 1)
        P_val_matrice = zeros([length(Models), length(Models)]);
        P_val_matrice(eye(size(P_val_matrice)) == 1) = nan;
        subplot(2, 2, hour_interval)
        P_vals = ones(length(Models), length(Models)) * -999;
        for q = quantiles_seller
            ind = find(0.01:0.01:0.99 == q);
            for m = 1:length(Models)
                for n = 1:length(Models)
                    [DM, p] = dmtest(squeeze(Models{m}(hour_interval, measure, :, ind)), squeeze(Models{n}(hour_interval, measure, :, ind)));
                    pvalmatyrice(n, m) = p;
                end
            end
            if length(quantiles_seller) == 1
                P_val_matrice = P_val_matrice + pvalmatyrice;
            else
                count = pvalmatyrice < alpha;
                P_val_matrice = P_val_matrice + count;
            end
        end
        h = heatmap(labels, labels, round(P_val_matrice, 3));
        h.FontSize = 6;
        title(['Hour interval: ', Hours_intervals{hour_interval}])
        sgtitle(['Seller, measure: ', Measures(measure)])
    end
end
