clear all
close all
addpath('Functions')
alpha = 0.05; % significance level for every DM test in notebook
start_eval=1;
mc='0';
dist='t';
% figures global parameters
Paper_width = 6.47699; % width of plot for TeX document (not universal - dependent on margins used!)
set(groot,'defaultAxesTickLabelInterpreter','latex');  
set(groot,'defaulttextinterpreter','latex');
set(groot,'defaultLegendInterpreter','latex');
set(groot,'defaultAxesFontSize',8)
fig_count = 1;
save_figures = 1;
% DA and ID plots
Asinh = load(['../../../Data_MATLAB_Drive/Data/GARCH_simulation_results_GE_DA_model_7_transformation_',...
        'asinh', '_dynamic_choice_0_mc_', mc ,'_dist_',dist,'.mat'],...
            'mae_hourly', 'rmse_hourly', 'archtest_results',...
            'kstest_results', 'kstesttstud_results', 'real_value', 'prediction','samples_no_garch');
        
Asinh_gjr = load(['../../../Data_MATLAB_Drive/Data/GJRGARCH_simulation_results_GE_DA_model_7_transformation_',...
        'asinh', '_dynamic_choice_0_mc_', mc ,'_dist_',dist,'.mat'],...
            'mae_hourly', 'rmse_hourly');
            
Without_asinh = load(['../../../Data_MATLAB_Drive/Data/GARCH_simulation_results_GE_DA_model_7_transformation_',...
        '', '_dynamic_choice_0_mc_',mc ,'_dist_',dist,'.mat'],...
            'mae_hourly', 'rmse_hourly', 'archtest_results',...
            'kstest_results', 'kstesttstud_results', 'real_value', 'prediction');

t_1 = tiledlayout(12, 9, 'TileSpacing', 'tight', 'Padding', 'none');
figure(fig_count)
ax(1) = nexttile(1, [9,2]);
plot(Without_asinh.mae_hourly)
all_data = [Asinh.mae_hourly; Without_asinh.mae_hourly];
diff_data = 0.05*(max(all_data) - min(all_data));
ylim([min(all_data) - diff_data, max(all_data) + diff_data])
ylabel("MAE")

ax(2) = nexttile(3, [9,2]);
plot(mean(abs(Asinh.real_value-squeeze(mean(Asinh.samples_no_garch,2))),2));
hold on
plot(Asinh.mae_hourly)
plot(Asinh_gjr.mae_hourly)
ylim([min(all_data) - diff_data, max(all_data) + diff_data])
title('Asinh')
set(gca,'Yticklabel',[])

ax(3) = nexttile(82, [3,4]);
lin_models_list = {Asinh, Without_asinh};
labels = {'ARX asinh', 'ARX'};
count_matrice = zeros([length(lin_models_list), length(lin_models_list)]);
for i = 1:length(lin_models_list)
    for j = 1:length(lin_models_list)
        if i == j
            count_matrice(i,j) = NaN;
        end
    end
end
for hour = 1:24
    pvalmatyrice = ones([length(lin_models_list), length(lin_models_list)]);
    for m = 1:length(lin_models_list)
        for n = 1:length(lin_models_list)
            e_1 = abs(lin_models_list{m}.real_value(hour, :) - lin_models_list{m}.prediction(hour, :)) ;
            e_2 = abs(lin_models_list{n}.real_value(hour, :) - lin_models_list{n}.prediction(hour, :)) ;
            [DM, p] = dmtest(e_1', e_2');
            pvalmatyrice(n, m) = p; % model on y is worse than model on x if p-value < alpha 
        end
    end
    count = pvalmatyrice < alpha;
    count_matrice = count_matrice + count;
end
xvalues = labels;
yvalues = labels;
h = heatmap(xvalues, yvalues, count_matrice);
h.NodeChildren(3).XAxis.TickLabelInterpreter = 'latex';
h.NodeChildren(3).YAxis.TickLabelInterpreter = 'latex';
h.NodeChildren(1).TickLabelInterpreter = 'latex';
h.NodeChildren(2).TickLabelInterpreter = 'latex';
h.FontName = 'CMU Serif';
h.FontSize = 8;
caxis([0 24])
h.ColorbarVisible = 'off';

Asinh = load(['../../../Data_MATLAB_Drive/Data/GARCH_simulation_results_GE_ID_model_7_transformation_',...
        'asinh', '_dynamic_choice_0_mc_',mc ,'_dist_',dist,'.mat'],...
            'mae_hourly', 'rmse_hourly', 'archtest_results',...
            'kstest_results', 'kstesttstud_results', 'real_value', 'prediction','samples_no_garch');
Asinh_gjr = load(['../../../Data_MATLAB_Drive/Data/GJRGARCH_simulation_results_GE_ID_model_7_transformation_',...
        'asinh', '_dynamic_choice_0_mc_',mc ,'_dist_',dist,'.mat'],...
            'mae_hourly', 'rmse_hourly');

Without_asinh = load(['../../../Data_MATLAB_Drive/Data/GARCH_simulation_results_GE_ID_model_7_transformation_',...
        '', '_dynamic_choice_0_mc_',mc ,'_dist_',dist,'.mat'],...
            'mae_hourly', 'rmse_hourly', 'archtest_results',...
            'kstest_results', 'kstesttstud_results', 'real_value', 'prediction');

all_data = [Asinh.mae_hourly; Asinh_gjr.mae_hourly; Without_asinh.mae_hourly];
diff_data = 0.05*(max(all_data) - min(all_data));

ax(4) = nexttile(6, [9,2]);
plot(Without_asinh.mae_hourly)
ylim([min(all_data) - diff_data, max(all_data) + diff_data])

ax(5) = nexttile(8, [9,2]);
plot(mean(abs(Asinh.real_value-squeeze(mean(Asinh.samples_no_garch,2))),2), 'DisplayName', 'ARX');
hold on
plot(Asinh.mae_hourly, 'DisplayName', 'GARCH')
plot(Asinh_gjr.mae_hourly, 'DisplayName', 'GJRGARCH')
ylim([min(all_data) - diff_data, max(all_data) + diff_data])
title('Asinh')
set(gca,'Yticklabel',[])

lgd = legend('Orientation','horizontal','Box','off');
lgd.Layout.Tile = 'south';

ax(6) = nexttile(87, [3,4]);
lin_models_list = {Asinh, Without_asinh};
labels = {'ARX asinh', 'ARX'};
count_matrice = zeros([length(lin_models_list), length(lin_models_list)]);
for i = 1:length(lin_models_list)
    for j = 1:length(lin_models_list)
        if i == j
            count_matrice(i,j) = NaN;
        end
    end
end
for hour = 1:24
    pvalmatyrice = ones([length(lin_models_list), length(lin_models_list)]);
    for m = 1:length(lin_models_list)
        for n = 1:length(lin_models_list)
            e_1 = abs(lin_models_list{m}.real_value(hour, :) - lin_models_list{m}.prediction(hour, :)) ;
            e_2 = abs(lin_models_list{n}.real_value(hour, :) - lin_models_list{n}.prediction(hour, :)) ;
            [DM, p] = dmtest(e_1', e_2');
            pvalmatyrice(n, m) = p; % model on y is worse than model on x if p-value < alpha 
        end
    end
    count = pvalmatyrice < alpha;
    count_matrice = count_matrice + count;
end
xvalues = labels;
yvalues = labels;
h = heatmap(xvalues, yvalues, count_matrice);
h.NodeChildren(3).XAxis.TickLabelInterpreter = 'latex';
h.NodeChildren(3).YAxis.TickLabelInterpreter = 'latex';
h.NodeChildren(1).TickLabelInterpreter = 'latex';
h.NodeChildren(2).TickLabelInterpreter = 'latex';
h.FontName = 'CMU Serif';
h.FontSize = 8;
Ax = gca;
Ax.YDisplayLabels = nan(size(Ax.YDisplayData));
caxis([0 24])
set(fig_count, 'units', 'inches', 'pos', [0 0 Paper_width 0.45 * Paper_width])
pos = get(fig_count,'Position');
set(fig_count, 'PaperPositionMode', 'Auto', 'PaperUnits', 'Inches', 'PaperSize', [pos(3), pos(4)])
if save_figures == 1
    f = gcf;
    exportgraphics(f,'Figures/MAE_GE_DA_ID.pdf','ContentType','vector')
end
fig_count = fig_count + 1;
%% DA
close all

%DA_0__0_g = load('Data_MATLAB_Drive/Data/GARCH_simulation_results_GE_DA_model_7_transformation__dynamic_choice_0_mc_0_dist_g.mat',...
%    'pinball', 'pinball_no_garch', 'mae_hourly', 'rmse_hourly', 'real_value', 'prediction', 'con_var', 'residua', 'samples','samples_no_garch', 'price_mu_vector', 'price_sigma_vector');
%DA_0_asinh_0_g = load('Data_MATLAB_Drive/Data/GARCH_simulation_results_GE_DA_model_7_transformation_asinh_dynamic_choice_0_mc_0_dist_g.mat',...
%    'pinball', 'pinball_no_garch', 'mae_hourly', 'rmse_hourly', 'real_value',  'prediction', 'con_var', 'residua', 'samples','samples_no_garch', 'price_mu_vector', 'price_sigma_vector');
DA_0__1_t = load(['../../../Data_MATLAB_Drive/Data/GARCH_simulation_results_GE_DA_model_7_transformation__dynamic_choice_0_mc_',mc ,'_dist_',dist,'.mat'],...
    'pinball', 'pinball_no_garch', 'mae_hourly', 'rmse_hourly', 'real_value', 'prediction', 'con_var', 'residua', 'samples','samples_no_garch', 'price_mu_vector', 'price_sigma_vector');
DA_0_asinh_1_t = load(['../../../Data_MATLAB_Drive/Data/GARCH_simulation_results_GE_DA_model_7_transformation_asinh_dynamic_choice_0_mc_',mc ,'_dist_',dist,'.mat'],...
    'pinball', 'pinball_no_garch', 'mae_hourly', 'rmse_hourly', 'real_value', 'prediction', 'con_var', 'residua', 'samples','samples_no_garch', 'price_mu_vector', 'price_sigma_vector');
%DA_0__0_g_GJR = load('Data_MATLAB_Drive/Data/GJRGARCH_simulation_results_GE_DA_model_7_transformation__dynamic_choice_0_mc_0_dist_g.mat',...
 %   'pinball', 'pinball_no_garch', 'mae_hourly', 'rmse_hourly', 'real_value', 'prediction', 'con_var', 'residua', 'samples','samples_no_garch', 'price_mu_vector', 'price_sigma_vector');
%DA_0_asinh_0_g_GJR = load('Data_MATLAB_Drive/Data/GJRGARCH_simulation_results_GE_DA_model_7_transformation_asinh_dynamic_choice_0_mc_0_dist_g.mat',...
%    'pinball', 'pinball_no_garch', 'mae_hourly', 'rmse_hourly', 'real_value',  'prediction', 'con_var', 'residua', 'samples','samples_no_garch', 'price_mu_vector', 'price_sigma_vector');
DA_0__1_t_GJR = load(['../../../Data_MATLAB_Drive/Data/GJRGARCH_simulation_results_GE_DA_model_7_transformation__dynamic_choice_0_mc_',mc ,'_dist_',dist,'.mat'],...
    'pinball', 'pinball_no_garch', 'mae_hourly', 'rmse_hourly', 'real_value', 'prediction', 'con_var', 'residua', 'samples','samples_no_garch', 'price_mu_vector', 'price_sigma_vector');
DA_0_asinh_1_t_GJR = load(['../../../Data_MATLAB_Drive/Data/GJRGARCH_simulation_results_GE_DA_model_7_transformation_asinh_dynamic_choice_0_mc_',mc ,'_dist_',dist,'.mat'],...
    'pinball', 'pinball_no_garch', 'mae_hourly', 'rmse_hourly', 'real_value', 'prediction', 'con_var', 'residua', 'samples','samples_no_garch', 'price_mu_vector', 'price_sigma_vector');
models_list = {DA_0__1_t,... % without GARCH hist.
    DA_0__1_t,...
    DA_0__1_t_GJR,...
    DA_0_asinh_1_t,... % without GARCH hist. asinh
    DA_0_asinh_1_t,...
    DA_0_asinh_1_t_GJR};
line_styles = {'-', '--', ':', '-', '--', ':'};
line_widths = {0.7, 1, 1.3, 0.7, 1, 1.3};

for m=1:length(models_list)
    models_list{m}.pinball=models_list{m}.pinball(:,start_eval:end,:);
    models_list{m}.pinball_no_garch=models_list{m}.pinball_no_garch(:,start_eval:end,:);
    models_list{m}.real_value=models_list{m}.real_value(:,start_eval:end);
    models_list{m}.prediction=models_list{m}.prediction(:,start_eval:end);
    models_list{m}.con_var=models_list{m}.con_var(:,start_eval:end);
    models_list{m}.residua=models_list{m}.residua(:,:,start_eval:end);
    models_list{m}.samples=models_list{m}.samples(:,:,start_eval:end);
    models_list{m}.samples_no_garch=models_list{m}.samples_no_garch(:,:,start_eval:end);
    models_list{m}.price_mu_vector=models_list{m}.price_mu_vector(:,start_eval:end);
    models_list{m}.price_sigma_vector=models_list{m}.price_sigma_vector(:,start_eval:end);
end

DM_models_list = {DA_0__1_t.pinball_no_garch,...
    DA_0__1_t.pinball,... 
    DA_0__1_t_GJR.pinball,... 
    DA_0_asinh_1_t.pinball_no_garch,...
    DA_0_asinh_1_t.pinball,...
    DA_0_asinh_1_t_GJR.pinball }; 


for m=1:length(DM_models_list)
    DM_models_list{m}=DM_models_list{m}(:,start_eval:end,:);
end

labels = {'ARX',...
    'GARCH',...
    'GJRGARCH',...
    'ARX asinh',...
    'GARCH asinh',...
    'GJRGARCH asinh'};

t_2 = tiledlayout('flow', 'TileSpacing', 'tight', 'Padding', 'none');
figure(fig_count)

nexttile;

hold on
for m = 1:length(models_list)
    if m <= 8
        plot(mean(mean(models_list{m}.pinball(:, :, :), 2), 3), 'LineStyle', line_styles{m}, 'LineWidth', line_widths{m})
    else
        plot(mean(mean(models_list{m}.pinball_no_garch(:, :, :), 2), 3), 'LineStyle', line_styles{m}, 'LineWidth', line_widths{m})
    end
end

xlabel('hour')
ylabel('pinball score')
hold off
lgd = legend(labels, 'Orientation','horizontal','Box','off');
lgd.Layout.Tile = 'north';

nexttile;
hold on
quantiles = 0.01:0.01:0.99;
for m = 1:length(models_list)
    if m <= 8
        plot(quantiles, squeeze(mean(mean(models_list{m}.pinball(:, :, :), 2))), 'LineStyle', line_styles{m}, 'LineWidth', line_widths{m})
    else
        plot(quantiles, squeeze(mean(mean(models_list{m}.pinball_no_garch(:, :, :), 2))), 'LineStyle', line_styles{m}, 'LineWidth', line_widths{m})
    end
end

xlabel('quantile')
ylabel('pinball score')
hold off

nexttile;
count_matrice = zeros([length(DM_models_list), length(DM_models_list)]);
for i = 1:length(DM_models_list)
    for j = 1:length(DM_models_list)
        if i == j
            count_matrice(i,j) = NaN;
        end
    end
end
for hour = 1:24
    pvalmatyrice = ones([length(DM_models_list), length(DM_models_list)]);
    for m = 1:length(DM_models_list)
        for n = 1:length(DM_models_list)
            [DM, p] = dmtest(mean(DM_models_list{m}(hour, :, :), 3)', mean(DM_models_list{n}(hour, :, :), 3)');
            pvalmatyrice(n, m) = p; % model on y is worse than model on x if p-value < alpha 
        end
    end
    count = pvalmatyrice < alpha;
    count_matrice = count_matrice + count;
end
xvalues = labels;
yvalues = labels;
h = heatmap(xvalues, yvalues, count_matrice);
h.NodeChildren(3).XAxis.TickLabelInterpreter = 'latex';
h.NodeChildren(3).YAxis.TickLabelInterpreter = 'latex';
h.NodeChildren(1).TickLabelInterpreter = 'latex';
h.NodeChildren(2).TickLabelInterpreter = 'latex';
h.FontName = 'CMU Serif';
h.FontSize = 6;
caxis([0 24])

nexttile;
count_matrice = zeros([length(DM_models_list), length(DM_models_list)]);
for i = 1:length(DM_models_list)
    for j = 1:length(DM_models_list)
        if i == j
            count_matrice(i,j) = NaN;
        end
    end
end
for quantile = 1:99
    pvalmatyrice = ones([length(DM_models_list), length(DM_models_list)]);
    for m = 1:length(DM_models_list)
        for n = 1:length(DM_models_list)
            [DM, p] = dmtest(mean(DM_models_list{m}(:, :, quantile), 1)', mean(DM_models_list{n}(:, :, quantile), 1)');
            pvalmatyrice(n, m) = p; % model on y is worse than model on x if p-value < alpha 
        end
    end
    count = pvalmatyrice < alpha;
    count_matrice = count_matrice + count;
end
xvalues = labels;
yvalues = labels;
h = heatmap(xvalues, yvalues, count_matrice);
h.NodeChildren(3).XAxis.TickLabelInterpreter = 'latex';
h.NodeChildren(3).YAxis.TickLabelInterpreter = 'latex';
h.NodeChildren(1).TickLabelInterpreter = 'latex';
h.NodeChildren(2).TickLabelInterpreter = 'latex';
h.FontName = 'CMU Serif';
h.FontSize = 6;
caxis([0 99])
set(fig_count, 'units', 'inches', 'pos', [0 0 Paper_width 0.55 * Paper_width])
pos = get(fig_count,'Position');
set(fig_count, 'PaperPositionMode', 'Auto', 'PaperUnits', 'Inches', 'PaperSize', [pos(3), pos(4)])
if save_figures == 1
    f = gcf;
    exportgraphics(f,'Figures/pinball_GE_DA.pdf','ContentType','vector')
end
fig_count = fig_count + 1;

%% ID
close all

%ID_0__0_g = load('Data_MATLAB_Drive/Data/GARCH_simulation_results_GE_ID_model_7_transformation__dynamic_choice_0_mc_0_dist_g.mat',...
%    'pinball', 'pinball_no_garch', 'mae_hourly', 'rmse_hourly', 'real_value', 'prediction', 'con_var', 'residua', 'samples','samples_no_garch', 'price_mu_vector', 'price_sigma_vector');
%ID_0_asinh_0_g = load('Data_MATLAB_Drive/Data/GARCH_simulation_results_GE_ID_model_7_transformation_asinh_dynamic_choice_0_mc_0_dist_g.mat',...
%    'pinball', 'pinball_no_garch', 'mae_hourly', 'rmse_hourly', 'real_value',  'prediction', 'con_var', 'residua', 'samples','samples_no_garch', 'price_mu_vector', 'price_sigma_vector');
ID_0__1_t = load(['../../../Data_MATLAB_Drive/Data/GARCH_simulation_results_GE_ID_model_7_transformation__dynamic_choice_0_mc_',mc ,'_dist_',dist,'.mat'],...
    'pinball', 'pinball_no_garch', 'mae_hourly', 'rmse_hourly', 'real_value', 'prediction', 'con_var', 'residua', 'samples','samples_no_garch', 'price_mu_vector', 'price_sigma_vector');
ID_0_asinh_1_t = load(['../../../Data_MATLAB_Drive/Data/GARCH_simulation_results_GE_ID_model_7_transformation_asinh_dynamic_choice_0_mc_',mc ,'_dist_',dist,'.mat'],...
    'pinball', 'pinball_no_garch', 'mae_hourly', 'rmse_hourly', 'real_value', 'prediction', 'con_var', 'residua', 'samples','samples_no_garch', 'price_mu_vector', 'price_sigma_vector');
%ID_0__0_g_GJR = load('Data_MATLAB_Drive/Data/GJRGARCH_simulation_results_GE_ID_model_7_transformation__dynamic_choice_0_mc_0_dist_g.mat',...
 %   'pinball', 'pinball_no_garch', 'mae_hourly', 'rmse_hourly', 'real_value', 'prediction', 'con_var', 'residua', 'samples','samples_no_garch', 'price_mu_vector', 'price_sigma_vector');
%ID_0_asinh_0_g_GJR = load('Data_MATLAB_Drive/Data/GJRGARCH_simulation_results_GE_ID_model_7_transformation_asinh_dynamic_choice_0_mc_0_dist_g.mat',...
%    'pinball', 'pinball_no_garch', 'mae_hourly', 'rmse_hourly', 'real_value',  'prediction', 'con_var', 'residua', 'samples','samples_no_garch', 'price_mu_vector', 'price_sigma_vector');
ID_0__1_t_GJR = load(['../../../Data_MATLAB_Drive/Data/GJRGARCH_simulation_results_GE_ID_model_7_transformation__dynamic_choice_0_mc_',mc ,'_dist_',dist,'.mat'],...
    'pinball', 'pinball_no_garch', 'mae_hourly', 'rmse_hourly', 'real_value', 'prediction', 'con_var', 'residua', 'samples','samples_no_garch', 'price_mu_vector', 'price_sigma_vector');
ID_0_asinh_1_t_GJR = load(['../../../Data_MATLAB_Drive/Data/GJRGARCH_simulation_results_GE_ID_model_7_transformation_asinh_dynamic_choice_0_mc_',mc ,'_dist_',dist,'.mat'],...
    'pinball', 'pinball_no_garch', 'mae_hourly', 'rmse_hourly', 'real_value', 'prediction', 'con_var', 'residua', 'samples','samples_no_garch', 'price_mu_vector', 'price_sigma_vector');
models_list = {ID_0__1_t,... % without GARCH hist.
    ID_0__1_t,...
    ID_0__1_t_GJR,...
    ID_0_asinh_1_t,... % without GARCH hist. asinh
    ID_0_asinh_1_t,...
    ID_0_asinh_1_t_GJR};
line_styles = {'-', '--', ':', '-', '--', ':'};
line_widths = {0.7, 1, 1.3, 0.7, 1, 1.3};

for m=1:length(models_list)
    models_list{m}.pinball=models_list{m}.pinball(:,start_eval:end,:);
    models_list{m}.pinball_no_garch=models_list{m}.pinball_no_garch(:,start_eval:end,:);
    models_list{m}.real_value=models_list{m}.real_value(:,start_eval:end);
    models_list{m}.prediction=models_list{m}.prediction(:,start_eval:end);
    models_list{m}.con_var=models_list{m}.con_var(:,start_eval:end);
    models_list{m}.residua=models_list{m}.residua(:,:,start_eval:end);
    models_list{m}.samples=models_list{m}.samples(:,:,start_eval:end);
    models_list{m}.samples_no_garch=models_list{m}.samples_no_garch(:,:,start_eval:end);
    models_list{m}.price_mu_vector=models_list{m}.price_mu_vector(:,start_eval:end);
    models_list{m}.price_sigma_vector=models_list{m}.price_sigma_vector(:,start_eval:end);
end

DM_models_list = {ID_0__1_t.pinball_no_garch,...
    ID_0__1_t.pinball,... 
    ID_0__1_t_GJR.pinball,... 
    ID_0_asinh_1_t.pinball_no_garch,...
    ID_0_asinh_1_t.pinball,...
    ID_0_asinh_1_t_GJR.pinball }; 


for m=1:length(DM_models_list)
    DM_models_list{m}=DM_models_list{m}(:,start_eval:end,:);
end

labels = {'ARX',...
    'GARCH',...
    'GJRGARCH',...
    'ARX asinh',...
    'GARCH asinh',...
    'GJRGARCH asinh'};

t_3 = tiledlayout('flow', 'TileSpacing', 'tight', 'Padding', 'none');
figure(fig_count)

nexttile;

hold on
for m = 1:length(models_list)
    if m <= 8
        plot(mean(mean(models_list{m}.pinball(:, :, :), 2), 3), 'LineStyle', line_styles{m}, 'LineWidth', line_widths{m})
    else
        plot(mean(mean(models_list{m}.pinball_no_garch(:, :, :), 2), 3), 'LineStyle', line_styles{m}, 'LineWidth', line_widths{m})
    end
end

xlabel('hour')
ylabel('pinball score')
hold off
lgd = legend(labels, 'Orientation','horizontal','Box','off');
lgd.Layout.Tile = 'north';

nexttile;
hold on
quantiles = 0.01:0.01:0.99;
for m = 1:length(models_list)
    if m <= 8
        plot(quantiles, squeeze(mean(mean(models_list{m}.pinball(:, :, :), 2))), 'LineStyle', line_styles{m}, 'LineWidth', line_widths{m})
    else
        plot(quantiles, squeeze(mean(mean(models_list{m}.pinball_no_garch(:, :, :), 2))), 'LineStyle', line_styles{m}, 'LineWidth', line_widths{m})
    end
end

xlabel('quantile')
ylabel('pinball score')
hold off

nexttile;
count_matrice = zeros([length(DM_models_list), length(DM_models_list)]);
for i = 1:length(DM_models_list)
    for j = 1:length(DM_models_list)
        if i == j
            count_matrice(i,j) = NaN;
        end
    end
end
for hour = 1:24
    pvalmatyrice = ones([length(DM_models_list), length(DM_models_list)]);
    for m = 1:length(DM_models_list)
        for n = 1:length(DM_models_list)
            [DM, p] = dmtest(mean(DM_models_list{m}(hour, :, :), 3)', mean(DM_models_list{n}(hour, :, :), 3)');
            pvalmatyrice(n, m) = p; % model on y is worse than model on x if p-value < alpha 
        end
    end
    count = pvalmatyrice < alpha;
    count_matrice = count_matrice + count;
end
xvalues = labels;
yvalues = labels;
h = heatmap(xvalues, yvalues, count_matrice);
h.NodeChildren(3).XAxis.TickLabelInterpreter = 'latex';
h.NodeChildren(3).YAxis.TickLabelInterpreter = 'latex';
h.NodeChildren(1).TickLabelInterpreter = 'latex';
h.NodeChildren(2).TickLabelInterpreter = 'latex';
h.FontName = 'CMU Serif';
h.FontSize = 6;
caxis([0 24])

nexttile;
count_matrice = zeros([length(DM_models_list), length(DM_models_list)]);
for i = 1:length(DM_models_list)
    for j = 1:length(DM_models_list)
        if i == j
            count_matrice(i,j) = NaN;
        end
    end
end
for quantile = 1:99
    pvalmatyrice = ones([length(DM_models_list), length(DM_models_list)]);
    for m = 1:length(DM_models_list)
        for n = 1:length(DM_models_list)
            [DM, p] = dmtest(mean(DM_models_list{m}(:, :, quantile), 1)', mean(DM_models_list{n}(:, :, quantile), 1)');
            pvalmatyrice(n, m) = p; % model on y is worse than model on x if p-value < alpha 
        end
    end
    count = pvalmatyrice < alpha;
    count_matrice = count_matrice + count;
end
xvalues = labels;
yvalues = labels;
h = heatmap(xvalues, yvalues, count_matrice);
h.NodeChildren(3).XAxis.TickLabelInterpreter = 'latex';
h.NodeChildren(3).YAxis.TickLabelInterpreter = 'latex';
h.NodeChildren(1).TickLabelInterpreter = 'latex';
h.NodeChildren(2).TickLabelInterpreter = 'latex';
h.FontName = 'CMU Serif';
h.FontSize = 6;
caxis([0 99])
set(fig_count, 'units', 'inches', 'pos', [0 0 Paper_width 0.55 * Paper_width])
pos = get(fig_count,'Position');
set(fig_count, 'PaperPositionMode', 'Auto', 'PaperUnits', 'Inches', 'PaperSize', [pos(3), pos(4)])
if save_figures == 1
    f = gcf;
    exportgraphics(f,'Figures/pinball_GE_ID.pdf','ContentType','vector')    
end
fig_count = fig_count + 1;
