% in this file we plot figures that describe data and results and prepare them
% for usage in publication
clear all
close all
addpath('Functions')

DA_color = [91, 91, 91]/255;
ID_color = [191, 99, 46]/255;

measures = {'std', 'semistd_down',...
            'std_and_profit', 'sharpe', 'mean_and_std', 'mean'};
measures_names = {'std', 'sstd', 'std-profit', 'sharpe', 'mean-std', 'mean'};

if strcmp(getenv('username'), 'andrz') || strcmp(getenv('username'), 'lab')
    addpath('../../../Data_MATLAB_Drive/Data/')
else
    addpath('D:\Nauka\artykuły\GARCH\Matlab\Data_MATLAB_Drive')
end
% setting the figure parameters
Paper_width = 6.47699; % width of plot for TeX document (not universal - dependent on margins used!)
set(groot,'defaultAxesTickLabelInterpreter','latex');  
set(groot,'defaulttextinterpreter','latex');
set(groot,'defaultLegendInterpreter','latex');
fig_count = 1;
save_figures = 1;
d1 = datetime('01/01/2017','InputFormat','dd/MM/uuuu');
d2 = datetime('31/12/2020','InputFormat','dd/MM/uuuu');
days = d1:1:d2; % Creates your x-axis, 90 dates from Jan 1st to March 30
Thresholds = [7, 13, 18, 22];
Thresholds_1 = [22, 7, 13, 18];
Thresholds_2 = [6, 12, 17, 21];

%% prices averaged over intervals
data_german = importdata('../Data/data_Germany_20170101_20201231.txt');
data_polish = importdata('../Data/data_Poland_20170101_20201231.txt');

price_DA_PL = reshape(data_polish(:, 3), [24, size(data_polish(:, 3), 1) / 24])';
price_DA_GE = reshape(data_german(:, 3), [24, size( data_german(:, 3), 1) / 24])';

price_ID_PL = reshape(data_polish(:, 4), [24, size(data_polish(:, 4), 1) / 24])';
price_ID_GE = reshape(data_german(:, 4), [24, size( data_german(:, 4), 1) / 24])';

data_sets = {price_DA_GE, price_ID_GE, price_DA_PL, price_ID_PL};

figure(fig_count)
t = tiledlayout(4, 1,'TileSpacing','compact', 'Padding', 'none');
titles = {'German day-ahead prices', 'German intraday prices', 'Polish day-ahead prices', 'Polish balancing prices'};
ylabels = {'EUR/MWh', 'EUR/MWh', 'PLN/MWh', 'PLN/MWh'};
alphas = [1, 0.8, 0.6, 0.4];
for s = 1:length(data_sets)
    nexttile
    for i = 1:length(Thresholds_1)
        if i == 1
            p = plot(days, mean([data_sets{s}(:, Thresholds_1(i):end), data_sets{s}(:, 1:Thresholds_2(i))], 2), 'DisplayName', [num2str(Thresholds_1(i)), '-', num2str(Thresholds_2(i))], LineWidth=0.6);
            hold on
        else
            p = plot(days, mean(data_sets{s}(:, Thresholds_1(i):Thresholds_2(i)), 2), 'DisplayName', [num2str(Thresholds_1(i)), '-', num2str(Thresholds_2(i))], LineWidth=0.6);
        end
        p.Color(4) = alphas(i);
        datetick('x','mm.yyyy')
    end
    if s < 3
        ylim([-100, 200]);
    end
    title(titles{s})
    ylabel(ylabels{s})
    hold off
end
lgd = legend('Orientation','horizontal','Box','off');
lgd.Layout.Tile = 'south';
% set width and save figure
set(fig_count, 'units', 'inches', 'pos', [0 0 Paper_width 1.2 * Paper_width])
pos = get(fig_count,'Position');
set(fig_count, 'PaperPositionMode', 'Auto', 'PaperUnits', 'Inches', 'PaperSize', [pos(3), pos(4)])
if save_figures == 1
    print(fig_count, 'Figures/Mean_of_intervals_for_every_market.pdf', '-dpdf')
end
fig_count = fig_count + 1;

%% GE prices averaged over intervals + spread
close all
data_german = importdata('../Data/data_Germany_20170101_20201231.txt');

price_DA_GE = reshape(data_german(:, 3), [24, size( data_german(:, 3), 1) / 24])';

price_ID_GE = reshape(data_german(:, 4), [24, size( data_german(:, 4), 1) / 24])';

Thresholds_1 = [22, 7, 13, 18];
Thresholds_2 = [6, 12, 17, 21];

data_sets = {price_DA_GE, price_ID_GE, price_DA_GE - price_ID_GE};

figure(fig_count)
t = tiledlayout(3, 1, 'TileSpacing', 'compact', 'Padding', 'none');
titles = {'German day-ahead prices', 'German intraday prices', 'German spread'};
ylabels = {'EUR/MWh', 'EUR/MWh', 'EUR/MWh'};
alphas = [1, 0.8, 0.6, 0.4];
for s = 1:length(data_sets)
    n_t = nexttile;
    for i = 1:length(Thresholds_1)
        if i == 1
            p = plot(days, mean([data_sets{s}(:, Thresholds_1(i):end), data_sets{s}(:, 1:Thresholds_2(i))], 2), 'DisplayName', [num2str(Thresholds_1(i)), '-', num2str(Thresholds_2(i))], LineWidth=0.6);
            hold on
        else
            p = plot(days, mean(data_sets{s}(:, Thresholds_1(i):Thresholds_2(i)), 2), 'DisplayName', [num2str(Thresholds_1(i)), '-', num2str(Thresholds_2(i))], LineWidth=0.6);
        end
        p.Color(4) = alphas(i);
        datetick('x','mm.yyyy')
    end
    if s == 1
        xline(n_t, days(373), '--','Validation window $\Longrightarrow$', 'LabelOrientation', 'horizontal', 'Interpreter', 'latex', 'Fontsize', 8,'HandleVisibility','off');
        xline(n_t, days(373), '--','$\Longleftarrow$ Calibration window', 'LabelOrientation', 'horizontal', 'Interpreter', 'latex', 'LabelHorizontalAlignment', 'left', 'Fontsize', 8,'HandleVisibility','off');
    else
        xline(n_t, days(373), '--','HandleVisibility','off');
        xline(n_t, days(373), '--','HandleVisibility','off');
    end
    if s < length(data_sets)
        ylim([-100, 160]);
        xticklabels(gca,{})
    end
    title(titles{s})
    ylabel(ylabels{s})
    hold off
end
lgd = legend('Orientation','horizontal','Box','off');
lgd.Layout.Tile = 'south';
% set width and save figure
set(fig_count, 'units', 'inches', 'pos', [0 0 Paper_width 1.2 * Paper_width])
pos = get(fig_count,'Position');
set(fig_count, 'PaperPositionMode', 'Auto', 'PaperUnits', 'Inches', 'PaperSize', [pos(3), pos(4)])
if save_figures == 1
    print(fig_count, 'Figures/Mean_of_intervals_for_GE_market.pdf', '-dpdf')
end
fig_count = fig_count + 1;

%% PL prices averaged over intervals + spread
close all
data_polish = importdata('../Data/data_Poland_20170101_20201231.txt');

price_DA_PL = reshape(data_polish(:, 3), [24, size(data_polish(:, 3), 1) / 24])';

price_ID_PL = reshape(data_polish(:, 4), [24, size(data_polish(:, 4), 1) / 24])';

Thresholds = [7, 13, 18, 22];
Thresholds_1 = [22, 7, 13, 18];
Thresholds_2 = [6, 12, 17, 21];

data_sets = {price_DA_PL, price_ID_PL, price_DA_PL - price_ID_PL};

figure(fig_count)
t = tiledlayout(3, 1,'TileSpacing','compact', 'Padding', 'none');
titles = {'Polish day-ahead prices', 'Polish balancing prices', 'Polish spread'};
ylabels = {'PLN/MWh', 'PLN/MWh', 'PLN/MWh'};
alphas = [1, 0.8, 0.6, 0.4];
for s = 1:length(data_sets)
    n_t = nexttile
    for i = 1:length(Thresholds_1)
        if i == 1
            p = plot(days, mean([data_sets{s}(:, Thresholds_1(i):end), data_sets{s}(:, 1:Thresholds_2(i))], 2), 'DisplayName', [num2str(Thresholds_1(i)), '-', num2str(Thresholds_2(i))], LineWidth=0.6);
            hold on
        else
            p = plot(days, mean(data_sets{s}(:, Thresholds_1(i):Thresholds_2(i)), 2), 'DisplayName', [num2str(Thresholds_1(i)), '-', num2str(Thresholds_2(i))], LineWidth=0.6);
        end
        p.Color(4) = alphas(i);
        datetick('x','mm.yyyy')
    end
    if s == 1
        xline(n_t, days(373), '--','Validation window $\Longrightarrow$', 'LabelOrientation', 'horizontal', 'Interpreter', 'latex', 'Fontsize', 8,'HandleVisibility','off');
        xline(n_t, days(373), '--','$\Longleftarrow$ Calibration window', 'LabelOrientation', 'horizontal', 'Interpreter', 'latex', 'LabelHorizontalAlignment', 'left', 'Fontsize', 8,'HandleVisibility','off');
    else
        xline(n_t, days(373), '--','HandleVisibility','off');
        xline(n_t, days(373), '--','HandleVisibility','off');
    end
    if s < length(data_sets)
        xticklabels(gca,{})
    end
    title(titles{s})
    ylabel(ylabels{s})
    hold off
end
lgd = legend('Orientation','horizontal','Box','off');
lgd.Layout.Tile = 'south';
% set width and save figure
set(fig_count, 'units', 'inches', 'pos', [0 0 Paper_width 1.2 * Paper_width])
pos = get(fig_count,'Position');
set(fig_count, 'PaperPositionMode', 'Auto', 'PaperUnits', 'Inches', 'PaperSize', [pos(3), pos(4)])
if save_figures == 1
    print(fig_count, 'Figures/Mean_of_intervals_for_PL_market.pdf', '-dpdf')
end
fig_count = fig_count + 1;

%% mean vs std
close all
transaction_sides = {'buyer', 'seller'};
countries = {'GE', 'PL'};
countries_names = {'Germany', 'Poland'};
markers = {'p', 'p', '*', '*', '*', '.'};
marker_sizes = {4, 4, 4, 4, 4, 7};
market_names = {'ID', 'BL'};
for side = 1:length(transaction_sides)
    transaction_side = transaction_sides{side};
    for c = 1:length({'GE', 'PL'})
        country = countries{c};
        if strcmp(country, 'GE')
            transformation_DA = 'asinh';
            transformation_ID = 'asinh';
            dist_DA = 'g';
            dist_ID = 'g';
        elseif strcmp(country, 'PL')
            transformation_DA = '';
            transformation_ID = 'asinh';
            dist_DA = 't';
            dist_ID = 't';
        end
        % loading samples and forecasted values
        IS_country_DA = load(['Garch_simulation_results_', country, '_DA_model_7_transformation_', transformation_DA, '_dynamic_choice_0_mc_0_dist_', dist_DA, '.mat'],...
            'samples', 'samples_no_garch', 'real_value', 'prediction');
        IS_country_ID = load(['Garch_simulation_results_', country, '_ID_model_7_transformation_', transformation_ID, '_dynamic_choice_0_mc_0_dist_', dist_ID, '.mat'],...
            'samples', 'samples_no_garch', 'real_value', 'prediction');
        figure((fig_count + c) * side)
        for i = 1:length(measures)
        
            load(['Weights/Weights_', transaction_side, '_', country, '/', measures{i}, '_weights_', 'GARCH_GARCH', ...
                '_', transformation_DA, '_', ...
                transformation_ID, '_', dist_DA, '_', dist_ID, '.mat']);
            [profit_DA, total_profit_DA, hourly_std_profit_DA, wallet,...
                hourly_std_wallet, hourly_mean_wallet, daily_std_wallet,...
                daily_mean_wallet] = strategy_evaluation_metrics( ...
                    IS_country_DA.real_value, IS_country_ID.real_value, Weights, transaction_side, "DA");
            for threshold = 2:length(Thresholds) + 1
                h_0 = Thresholds(threshold - 1);
                subplot(2, 2, threshold - 1)
                if threshold == length(Thresholds) + 1
                    h_1 = Thresholds(1) - 1;
                    p1 = plot(mean([hourly_std_wallet(h_0:24); hourly_std_wallet(1:h_1)]), mean([hourly_mean_wallet(h_0:24); hourly_mean_wallet(1:h_1)]), markers{i}, 'MarkerSize', marker_sizes{i}, 'DisplayName', measures_names{i});
                    hold on
                    x_values(threshold, side, c, i) = mean([hourly_std_wallet(h_0:24); hourly_std_wallet(1:h_1)]);
                    y_values(threshold, side, c, i) = mean([hourly_mean_wallet(h_0:24); hourly_mean_wallet(1:h_1)]);
                else
                    h_1 = Thresholds(threshold) - 1;
                    p2 = plot(mean(hourly_std_wallet(h_0:h_1)), mean(hourly_mean_wallet(h_0:h_1)), markers{i}, 'MarkerSize', marker_sizes{i}, 'DisplayName', measures_names{i});
                    hold on
                    x_values(threshold, side, c, i) = mean(hourly_std_wallet(h_0:h_1));
                    y_values(threshold, side, c, i) = mean(hourly_mean_wallet(h_0:h_1));
                end
                hold on  
            end
        end
    end
end

% adding static (with respect to strategy) elements to the plots
for side = 1:length(transaction_sides)
    transaction_side = transaction_sides{side};
    for c = 1:length({'GE', 'PL'})
        country = countries{c};
        IS_country_DA = load(['Garch_simulation_results_', country, '_DA_model_7_transformation_', transformation_DA, '_dynamic_choice_0_mc_0_dist_', dist_DA, '.mat'],...
            'samples', 'samples_no_garch', 'real_value', 'prediction');
        IS_country_ID = load(['Garch_simulation_results_', country, '_ID_model_7_transformation_', transformation_ID, '_dynamic_choice_0_mc_0_dist_', dist_ID, '.mat'],...
            'samples', 'samples_no_garch', 'real_value', 'prediction');
        figure((fig_count + c) * side)  
        for threshold = 2:length(Thresholds) + 1
            h_0 = Thresholds(threshold - 1);
            ax(threshold - 1) = subplot(2, 2, threshold - 1)
            if threshold == length(Thresholds) + 1
                h_1 = Thresholds(1) - 1;
                p3 = plot(mean([std(IS_country_DA.real_value(h_0:24, :), 0, 2); ...
                    std(IS_country_DA.real_value(1:h_1, :), 0, 2)]), ...
                    mean([mean(IS_country_DA.real_value(h_0:24, :), 2); ...
                    mean(IS_country_DA.real_value(1:h_1, :), 2)]), 'x', 'MarkerSize', 7, ...
                    'DisplayName', 'DA', 'Color', DA_color);
                p4 = plot(mean([std(IS_country_ID.real_value(h_0:24, :), 0, 2); ...
                    std(IS_country_ID.real_value(1:h_1, :), 0, 2)]), ...
                    mean([mean(IS_country_ID.real_value(h_0:24, :), 2); ...
                    mean(IS_country_ID.real_value(1:h_1, :), 2)]), 'x', 'MarkerSize', 7, ...
                    'DisplayName', market_names{c}, 'Color', ID_color);
                quarter_div_x = mean([std(IS_country_DA.real_value(h_0:24, :), 0, 2); ...
                    std(IS_country_DA.real_value(1:h_1, :), 0, 2)]);
                if strcmp(country, 'GE') && strcmp(transaction_side, 'buyer')
                    if h_0 == 7 || h_0 == 18
                        quarter_div_y = mean([mean(IS_country_DA.real_value(h_0:24, :), 2); ...
                            mean(IS_country_DA.real_value(1:h_1, :), 2)]);
                    else
                        quarter_div_y = mean([mean(IS_country_ID.real_value(h_0:24, :), 2); ...
                            mean(IS_country_ID.real_value(1:h_1, :), 2)]);
                    end
                elseif strcmp(country, 'GE') && strcmp(transaction_side, 'seller')
                    if h_0 == 7 || h_0 == 18
                        quarter_div_y = mean([mean(IS_country_ID.real_value(h_0:24, :), 2); ...
                            mean(IS_country_ID.real_value(1:h_1, :), 2)]);
                    else
                        quarter_div_y = mean([mean(IS_country_DA.real_value(h_0:24, :), 2); ...
                            mean(IS_country_DA.real_value(1:h_1, :), 2)]);
                    end                    
                elseif strcmp(country, 'PL') && strcmp(transaction_side, 'buyer')
                    quarter_div_y = mean([mean(IS_country_DA.real_value(h_0:24, :), 2); ...
                        mean(IS_country_DA.real_value(1:h_1, :), 2)]);
                elseif strcmp(country, 'PL') && strcmp(transaction_side, 'seller')
                    quarter_div_y = mean([mean(IS_country_ID.real_value(h_0:24, :), 2); ...
                            mean(IS_country_ID.real_value(1:h_1, :), 2)]);
                end
                xline(quarter_div_x, ':', 'HandleVisibility','off')
                yline(quarter_div_y, ':', 'HandleVisibility','off')
                x_values(threshold, side, c, 7) = mean([std(IS_country_DA.real_value(h_0:24, :), 0, 2); ...
                    std(IS_country_DA.real_value(1:h_1, :), 0, 2)]);
                y_values(threshold, side, c, 7) = mean([mean(IS_country_DA.real_value(h_0:24, :), 2); ...
                    mean(IS_country_DA.real_value(1:h_1, :), 2)]);
                x_values(threshold, side, c, 8) = mean([std(IS_country_ID.real_value(h_0:24, :), 0, 2); ...
                    std(IS_country_ID.real_value(1:h_1, :), 0, 2)]);
                y_values(threshold, side, c, 8) = mean([mean(IS_country_ID.real_value(h_0:24, :), 2); ...
                    mean(IS_country_ID.real_value(1:h_1, :), 2)]);
            else
                h_1 = Thresholds(threshold) - 1;
                p5 = plot(mean(std(IS_country_DA.real_value(h_0:h_1, :), 0, 2)), ...
                    mean(mean(IS_country_DA.real_value(h_0:h_1, :), 2)), 'x', 'MarkerSize', 7, ...
                    'DisplayName', 'DA', 'Color', DA_color);
                p6 = plot(mean(std(IS_country_ID.real_value(h_0:h_1, :), 0, 2)), ...
                    mean(mean(IS_country_ID.real_value(h_0:h_1, :), 2)), 'x', 'MarkerSize', 7, ...
                    'DisplayName', market_names{c}, 'Color', ID_color);
                quarter_div_x = mean(std(IS_country_DA.real_value(h_0:h_1, :), 0, 2));
                if strcmp(country, 'GE') && strcmp(transaction_side, 'buyer')
                    if h_0 == 7 || h_0 == 18
                        quarter_div_y = mean(mean(IS_country_DA.real_value(h_0:h_1, :), 2));
                    else
                        quarter_div_y = mean(mean(IS_country_ID.real_value(h_0:h_1, :), 2));
                    end
                elseif strcmp(country, 'GE') && strcmp(transaction_side, 'seller')
                    if h_0 == 7 || h_0 == 18
                        quarter_div_y = mean(mean(IS_country_ID.real_value(h_0:h_1, :), 2));
                    else
                        quarter_div_y = mean(mean(IS_country_DA.real_value(h_0:h_1, :), 2));
                    end                    
                elseif strcmp(country, 'PL') && strcmp(transaction_side, 'buyer')
                    quarter_div_y = mean(mean(IS_country_DA.real_value(h_0:h_1, :), 2));
                elseif strcmp(country, 'PL') && strcmp(transaction_side, 'seller')
                    quarter_div_y = mean(mean(IS_country_ID.real_value(h_0:h_1, :), 2));
                end
                xline(quarter_div_x, ':', 'HandleVisibility','off')
                yline(quarter_div_y, ':', 'HandleVisibility','off')
                x_values(threshold, side, c, 7) = mean(std(IS_country_DA.real_value(h_0:h_1, :), 0, 2));
                y_values(threshold, side, c, 7) = mean(mean(IS_country_DA.real_value(h_0:h_1, :), 2));
                x_values(threshold, side, c, 8) = mean(std(IS_country_ID.real_value(h_0:h_1, :), 0, 2));
                y_values(threshold, side, c, 8) = mean(mean(IS_country_ID.real_value(h_0:h_1, :), 2));
            end
            limits = get(gca, 'XLim');
            limits_diff = limits(2) - limits(1);
            xlim([limits(1) - 0.05*limits_diff, limits(2) + 0.05*limits_diff]);

            limits = get(gca, 'YLim');
            limits_diff = limits(2) - limits(1);
            ylim([limits(1) - 0.08*limits_diff, limits(2) + 0.08*limits_diff]);
            title(['Mean over hours from ', num2str(h_0), ' to ', num2str(h_1)])
            legend('Orientation','vertical','Box','off', 'Location', 'EastOutside');
            if h_0 == 7 || h_0 == 18
                ylabel("Mean")
            end
            if h_0 == 18 || h_0 == 22
                xlabel("Std")
            end
            grid minor
        end
        sgtitle([countries_names{c}, ', ', transaction_side], 'Fontsize', 10)
        for threshold = 2:length(Thresholds) + 1
            subplot(2, 2, threshold - 1)
            h_0 = Thresholds(threshold - 1);
            if threshold == length(Thresholds) + 1
                h_1 = Thresholds(1) - 1;
                quarter_div_x = mean([std(IS_country_DA.real_value(h_0:24, :), 0, 2); ...
                    std(IS_country_DA.real_value(1:h_1, :), 0, 2)]);
                if strcmp(country, 'GE') && strcmp(transaction_side, 'buyer')
                    if h_0 == 7 || h_0 == 18
                        quarter_div_y = mean([mean(IS_country_DA.real_value(h_0:24, :), 2); ...
                            mean(IS_country_DA.real_value(1:h_1, :), 2)]);
                    else
                        quarter_div_y = mean([mean(IS_country_ID.real_value(h_0:24, :), 2); ...
                            mean(IS_country_ID.real_value(1:h_1, :), 2)]);
                    end
                elseif strcmp(country, 'GE') && strcmp(transaction_side, 'seller')
                    if h_0 == 7 || h_0 == 18
                        quarter_div_y = mean([mean(IS_country_ID.real_value(h_0:24, :), 2); ...
                            mean(IS_country_ID.real_value(1:h_1, :), 2)]);
                    else
                        quarter_div_y = mean([mean(IS_country_DA.real_value(h_0:24, :), 2); ...
                            mean(IS_country_DA.real_value(1:h_1, :), 2)]);
                    end                    
                elseif strcmp(country, 'PL') && strcmp(transaction_side, 'buyer')
                    quarter_div_y = mean([mean(IS_country_DA.real_value(h_0:24, :), 2); ...
                        mean(IS_country_DA.real_value(1:h_1, :), 2)]);
                elseif strcmp(country, 'PL') && strcmp(transaction_side, 'seller')
                    quarter_div_y = mean([mean(IS_country_ID.real_value(h_0:24, :), 2); ...
                            mean(IS_country_ID.real_value(1:h_1, :), 2)]);
                end
                if strcmp(country, 'PL') && strcmp(transaction_side, 'seller') && h_0 == 7
                    [p,z] = zoomPlot(squeeze(x_values(threshold, side, c, :)), squeeze(y_values(threshold, side, c, :)), [58, 60], [0.25 0.60 0.1 0.2], [0], markers, marker_sizes, 0, quarter_div_x, quarter_div_y);
                    legend hide
                elseif strcmp(country, 'PL') && strcmp(transaction_side, 'seller') && h_0 == 13
                    [p,z] = zoomPlot(squeeze(x_values(threshold, side, c, :)), squeeze(y_values(threshold, side, c, :)), [59, 61], [0.7 0.60 0.1 0.2], [0], markers, marker_sizes, 0, quarter_div_x, quarter_div_y);
                    legend hide
                elseif strcmp(country, 'PL') && strcmp(transaction_side, 'seller') && h_0 == 18
                    [p,z] = zoomPlot(squeeze(x_values(threshold, side, c, :)), squeeze(y_values(threshold, side, c, :)), [57, 59], [0.25 0.14 0.1 0.2], [0], markers, marker_sizes, 0, quarter_div_x, quarter_div_y);
                    legend hide
                elseif strcmp(country, 'PL') && strcmp(transaction_side, 'seller') && h_0 == 22
                    [p,z] = zoomPlot(squeeze(x_values(threshold, side, c, :)), squeeze(y_values(threshold, side, c, :)), [36.05, 38.05], [0.7 0.14 0.1 0.2], [0], markers, marker_sizes, 0, quarter_div_x, quarter_div_y);
                    legend hide
                elseif strcmp(country, 'PL') && strcmp(transaction_side, 'buyer') && h_0 == 7
                    [p,z] = zoomPlot(squeeze(x_values(threshold, side, c, :)), squeeze(y_values(threshold, side, c, :)), [57.5, 59.5], [0.27 0.61 0.1 0.2], [0], markers, marker_sizes, 1, quarter_div_x, quarter_div_y);
                    legend hide
                elseif strcmp(country, 'PL') && strcmp(transaction_side, 'buyer') && h_0 == 13
                    [p,z] = zoomPlot(squeeze(x_values(threshold, side, c, :)), squeeze(y_values(threshold, side, c, :)), [59.2, 61.2], [0.72 0.61 0.1 0.2], [0], markers, marker_sizes, 0, quarter_div_x, quarter_div_y);
                    legend hide
                elseif strcmp(country, 'PL') && strcmp(transaction_side, 'buyer') && h_0 == 18
                    [p,z] = zoomPlot(squeeze(x_values(threshold, side, c, :)), squeeze(y_values(threshold, side, c, :)), [57.5, 59.5], [0.27 0.185 0.1 0.2], [0], markers, marker_sizes, 0, quarter_div_x, quarter_div_y);
                    legend hide
                elseif strcmp(country, 'PL') && strcmp(transaction_side, 'buyer') && h_0 == 22
                    [p,z] = zoomPlot(squeeze(x_values(threshold, side, c, :)), squeeze(y_values(threshold, side, c, :)), [36.05, 38.05], [0.72 0.185 0.1 0.2], [0], markers, marker_sizes, 1, quarter_div_x, quarter_div_y);
                    legend hide
                end
            else
                h_1 = Thresholds(threshold) - 1;
                quarter_div_x = mean(std(IS_country_DA.real_value(h_0:h_1, :), 0, 2));
                if strcmp(country, 'GE') && strcmp(transaction_side, 'buyer')
                    if h_0 == 7 || h_0 == 18
                        quarter_div_y = mean(mean(IS_country_DA.real_value(h_0:h_1, :), 2));
                    else
                        quarter_div_y = mean(mean(IS_country_ID.real_value(h_0:h_1, :), 2));
                    end
                elseif strcmp(country, 'GE') && strcmp(transaction_side, 'seller')
                    if h_0 == 7 || h_0 == 18
                        quarter_div_y = mean(mean(IS_country_ID.real_value(h_0:h_1, :), 2));
                    else
                        quarter_div_y = mean(mean(IS_country_DA.real_value(h_0:h_1, :), 2));
                    end                    
                elseif strcmp(country, 'PL') && strcmp(transaction_side, 'buyer')
                    quarter_div_y = mean(mean(IS_country_DA.real_value(h_0:h_1, :), 2));
                elseif strcmp(country, 'PL') && strcmp(transaction_side, 'seller')
                    quarter_div_y = mean(mean(IS_country_ID.real_value(h_0:h_1, :), 2));
                end
                if strcmp(country, 'PL') && strcmp(transaction_side, 'seller') && h_0 == 7
                    [p,z] = zoomPlot(squeeze(x_values(threshold, side, c, :)), squeeze(y_values(threshold, side, c, :)), [58, 60], [0.25 0.60 0.1 0.2], [0], markers, marker_sizes, 0, quarter_div_x, quarter_div_y);
                    legend hide
                elseif strcmp(country, 'PL') && strcmp(transaction_side, 'seller') && h_0 == 13
                    [p,z] = zoomPlot(squeeze(x_values(threshold, side, c, :)), squeeze(y_values(threshold, side, c, :)), [59, 61], [0.7 0.60 0.1 0.2], [0], markers, marker_sizes, 0, quarter_div_x, quarter_div_y);
                    legend hide
                elseif strcmp(country, 'PL') && strcmp(transaction_side, 'seller') && h_0 == 18
                    [p,z] = zoomPlot(squeeze(x_values(threshold, side, c, :)), squeeze(y_values(threshold, side, c, :)), [57, 59], [0.25 0.14 0.1 0.2], [0], markers, marker_sizes, 0, quarter_div_x, quarter_div_y);
                    legend hide
                elseif strcmp(country, 'PL') && strcmp(transaction_side, 'seller') && h_0 == 22
                    [p,z] = zoomPlot(squeeze(x_values(threshold, side, c, :)), squeeze(y_values(threshold, side, c, :)), [36.05, 38.05], [0.7 0.14 0.1 0.2], [0], markers, marker_sizes, 0, quarter_div_x, quarter_div_y);
                    legend hide
                elseif strcmp(country, 'PL') && strcmp(transaction_side, 'buyer') && h_0 == 7
                    [p,z] = zoomPlot(squeeze(x_values(threshold, side, c, :)), squeeze(y_values(threshold, side, c, :)), [57.5, 59.5], [0.27 0.61 0.1 0.2], [0], markers, marker_sizes, 1, quarter_div_x, quarter_div_y);
                    legend hide
                elseif strcmp(country, 'PL') && strcmp(transaction_side, 'buyer') && h_0 == 13
                    [p,z] = zoomPlot(squeeze(x_values(threshold, side, c, :)), squeeze(y_values(threshold, side, c, :)), [59.2, 61.2], [0.72 0.61 0.1 0.2], [0], markers, marker_sizes, 0, quarter_div_x, quarter_div_y);
                    legend hide
                elseif strcmp(country, 'PL') && strcmp(transaction_side, 'buyer') && h_0 == 18
                    [p,z] = zoomPlot(squeeze(x_values(threshold, side, c, :)), squeeze(y_values(threshold, side, c, :)), [57.5, 59.5], [0.27 0.185 0.1 0.2], [0], markers, marker_sizes, 0, quarter_div_x, quarter_div_y);
                    legend hide
                elseif strcmp(country, 'PL') && strcmp(transaction_side, 'buyer') && h_0 == 22
                    [p,z] = zoomPlot(squeeze(x_values(threshold, side, c, :)), squeeze(y_values(threshold, side, c, :)), [36.05, 38.05], [0.72 0.185 0.1 0.2], [0], markers, marker_sizes, 1, quarter_div_x, quarter_div_y);
                    legend hide
                end
            end
        end
        % set width and save figure
        set((fig_count + c) * side, 'units', 'inches', 'pos', [0 0 0.85*Paper_width 0.5 * Paper_width])
        pos = get((fig_count + c) * side,'Position');
        set((fig_count + c) * side, 'PaperPositionMode', 'Auto', 'PaperUnits', 'Inches', 'PaperSize', [pos(3), pos(4)])
        if save_figures == 1
            print((fig_count + c) * side, ['Figures/Mean_vs_std_', transaction_sides{side}, '_', country, '.pdf'], '-dpdf')
        end
    end
end
fig_count = fig_count + 1;

%% mean and std vs hours
close all
market_names = {'ID', 'BL'};
transaction_sides = {'buyer', 'seller'};
countries = {'GE', 'PL'};
color_map = colormap(jet(length(measures)+2));
for c = 1:length({'GE', 'PL'})
    country = countries{c};
    figure(fig_count + c)
    t = tiledlayout('flow','TileSpacing','compact','Padding', 'none');
    for side = 1:length(transaction_sides)
        transaction_side = transaction_sides{side};
        if strcmp(country, 'GE')
            transformation_DA = 'asinh';
            transformation_ID = 'asinh';
            dist_DA = 'g';
            dist_ID = 'g';
        elseif strcmp(country, 'PL')
            transformation_DA = '';
            transformation_ID = 'asinh';
            dist_DA = 't';
            dist_ID = 't';
        end
        % loading samples and forecasted values
        IS_country_DA = load(['Garch_simulation_results_', country, '_DA_model_7_transformation_', transformation_DA, '_dynamic_choice_0_mc_0_dist_', dist_DA, '.mat'],...
            'samples', 'samples_no_garch', 'real_value', 'prediction');
        IS_country_ID = load(['Garch_simulation_results_', country, '_ID_model_7_transformation_', transformation_ID, '_dynamic_choice_0_mc_0_dist_', dist_ID, '.mat'],...
            'samples', 'samples_no_garch', 'real_value', 'prediction');        
        for i = 1:length(measures)
        
            load(['Weights/Weights_', transaction_side, '_', country, '/', measures{i}, '_weights_', 'GARCH_GARCH', ...
                '_', transformation_DA, '_', ...
                transformation_ID, '_', dist_DA, '_', dist_ID, '.mat']);
            [profit_DA, total_profit_DA, hourly_std_profit_DA, wallet,...
                hourly_std_wallet, hourly_mean_wallet, daily_std_wallet,...
                daily_mean_wallet] = strategy_evaluation_metrics( ...
                    IS_country_DA.real_value, IS_country_ID.real_value, Weights, transaction_side, "DA");        
            nexttile(2*side - 1)
            plot(1:24, hourly_mean_wallet, 'DisplayName', measures_names{i})
            hold on 

            nexttile(2*side)
            plot(1:24, hourly_std_wallet, 'DisplayName', measures_names{i})
            hold on             
        end
    end
end

% adding static (with respect to strategy) elements to the plots
for c = 1:length({'GE', 'PL'})
    country = countries{c};    
    figure(fig_count + c)
    for side = 1:length(transaction_sides)  
        transaction_side = transaction_sides{side};
        IS_country_DA = load(['Garch_simulation_results_', country, '_DA_model_7_transformation_', transformation_DA, '_dynamic_choice_0_mc_0_dist_', dist_DA, '.mat'],...
            'samples', 'samples_no_garch', 'real_value', 'prediction');
        IS_country_ID = load(['Garch_simulation_results_', country, '_ID_model_7_transformation_', transformation_ID, '_dynamic_choice_0_mc_0_dist_', dist_ID, '.mat'],...
            'samples', 'samples_no_garch', 'real_value', 'prediction');
        nexttile(2*side - 1)
        p = plot(1:24, mean(IS_country_DA.real_value(:, :), 2), 'DisplayName', 'DA', 'Color', DA_color, 'linestyle', '-.');
%         p.Marker = '.';
        hold on 
        p = plot(1:24, mean(IS_country_ID.real_value(:, :), 2), 'DisplayName', market_names{c}, 'Color', ID_color, 'linestyle', '--');
%         p.Marker = '.';
        xlim([1, 24]);
        ylabel({['\textbf{', transaction_side, '}'], '',  'Mean'});
        if side == 1
            xticklabels(gca,{})
        elseif side == 2
            xlabel('hours', Interpreter='latex')
        end

        nexttile(2*side)
        p = plot(1:24, std(IS_country_DA.real_value(:, :), 0, 2), 'DisplayName', 'DA', 'Color', DA_color, 'linestyle', '--');
%         p.Marker = '.';
        hold on 
        p = plot(1:24, std(IS_country_ID.real_value(:, :), 0, 2), 'DisplayName', market_names{c}, 'Color', ID_color, 'linestyle', '--');
%         p.Marker = '.';
        xlim([1, 24]);
        ylabel('Standard deviation')
        if side == 1
            xticklabels(gca,{})
        elseif side == 2
            xlabel('hours', Interpreter='latex')
        end        
    end
    sgtitle(countries_names{c}, Interpreter="latex")
    lgd = legend('Orientation','vertical','Box','off','NumColumns',1);
    lgd.Layout.Tile = 'east';
    % set width and save figure
    set(fig_count + c, 'units', 'inches', 'pos', [0 0 Paper_width 0.5 * Paper_width])
    pos = get(fig_count + c,'Position');
    set(fig_count + c, 'PaperPositionMode', 'Auto', 'PaperUnits', 'Inches', 'PaperSize', [pos(3), pos(4)])
    if save_figures == 1
        print(fig_count + c, ['Figures/Mean_vs_hours_', country, '.pdf'], '-dpdf')
    end
end
fig_count = fig_count + 1;
%% Transformation hourly analysis GE
close all

h = 18;

data_german = importdata('../Data/data_Germany_20170101_20201231.txt');

price_DA_GE = reshape(data_german(:, 3), [24, size( data_german(:, 3), 1) / 24])';

price_ID_GE = reshape(data_german(:, 4), [24, size( data_german(:, 4), 1) / 24])';

figure(fig_count)
t = tiledlayout(2, 1,'TileSpacing','compact', 'Padding', 'none');
nexttile;
data = price_DA_GE(:, h);
[standardized_price, mu_DA, sigma_DA] =...
    zscore(data);
plot(asinh(standardized_price));
title("DA")
nexttile;
data = price_ID_GE(:, h);
[standardized_price, mu_ID, sigma_ID] =...
    zscore(data);
plot(asinh(standardized_price));
title("ID")

% set width and save figure
set(fig_count, 'units', 'inches', 'pos', [0 0 Paper_width 0.8 * Paper_width])
pos = get(fig_count,'Position');
set(fig_count, 'PaperPositionMode', 'Auto', 'PaperUnits', 'Inches', 'PaperSize', [pos(3), pos(4)])
%% Transformation hourly analysis PL
close all

h = 18;

data_polish = importdata('../Data/data_Poland_20170101_20201231.txt');

price_DA_PL = reshape(data_polish(:, 3), [24, size( data_polish(:, 3), 1) / 24])';

price_ID_PL = reshape(data_polish(:, 4), [24, size( data_polish(:, 4), 1) / 24])';

figure(fig_count)
t = tiledlayout(2, 1,'TileSpacing','compact', 'Padding', 'none');
nexttile;
data = price_DA_PL(:, h);
[standardized_price, mu_DA, sigma_DA] =...
    zscore(data);
plot(asinh(standardized_price));
title("DA")
nexttile;
data = price_ID_PL(:, h);
[standardized_price, mu_ID, sigma_ID] =...
    zscore(data);
plot(asinh(standardized_price));
title("ID")

% set width and save figure
set(fig_count, 'units', 'inches', 'pos', [0 0 Paper_width 0.8 * Paper_width])
pos = get(fig_count,'Position');
set(fig_count, 'PaperPositionMode', 'Auto', 'PaperUnits', 'Inches', 'PaperSize', [pos(3), pos(4)])
%% GE prices averaged over intervals + spread asinh
close all
data_german = importdata('../Data/data_Germany_20170101_20201231.txt');

price_DA_GE = reshape(data_german(:, 3), [24, size( data_german(:, 3), 1) / 24])';

price_ID_GE = reshape(data_german(:, 4), [24, size( data_german(:, 4), 1) / 24])';

Thresholds_1 = [22, 7, 13, 18];
Thresholds_2 = [6, 12, 17, 21];

data_sets = {asinh(zscore(price_DA_GE, [], 1)), asinh(zscore(price_ID_GE, [], 1)), asinh(zscore(price_DA_GE, [], 1)) - asinh(zscore(price_ID_GE, [], 1))};

figure(fig_count)
t = tiledlayout(3, 1, 'TileSpacing', 'compact', 'Padding', 'none');
titles = {'German day-ahead asinh transformed prices', 'German intraday asinh transformed prices', 'German spread of asinh transformed prices'};
ylabels = {'$\widetilde{DA}$', '$\widetilde{ID}$', '$\widetilde{DA} - \widetilde{ID}$'};
alphas = [1, 0.8, 0.6, 0.4];
for s = 1:length(data_sets)
    n_t = nexttile;
    for i = 1:length(Thresholds_1)
        if i == 1
            p = plot(days, mean([data_sets{s}(:, Thresholds_1(i):end), data_sets{s}(:, 1:Thresholds_2(i))], 2), 'DisplayName', [num2str(Thresholds_1(i)), '-', num2str(Thresholds_2(i))], LineWidth=0.6);
            hold on
        else
            p = plot(days, mean(data_sets{s}(:, Thresholds_1(i):Thresholds_2(i)), 2), 'DisplayName', [num2str(Thresholds_1(i)), '-', num2str(Thresholds_2(i))], LineWidth=0.6);
        end
        p.Color(4) = alphas(i);
        datetick('x','mm.yyyy')
    end
    if s == 1
        xline(n_t, days(373), '--','Validation window $\Longrightarrow$', 'LabelOrientation', 'horizontal', 'Interpreter', 'latex', 'Fontsize', 8,'HandleVisibility','off');
        xline(n_t, days(373), '--','$\Longleftarrow$ Calibration window', 'LabelOrientation', 'horizontal', 'Interpreter', 'latex', 'LabelHorizontalAlignment', 'left', 'Fontsize', 8,'HandleVisibility','off');
    else
        xline(n_t, days(373), '--','HandleVisibility','off');
        xline(n_t, days(373), '--','HandleVisibility','off');
    end
    if s < length(data_sets)
        ylim([-3, 3]);
        xticklabels(gca,{})
    end
    title(titles{s})
    ylabel(ylabels{s})
    hold off
end
lgd = legend('Orientation','horizontal','Box','off');
lgd.Layout.Tile = 'south';
% set width and save figure
set(fig_count, 'units', 'inches', 'pos', [0 0 Paper_width 1.2 * Paper_width])
pos = get(fig_count,'Position');
set(fig_count, 'PaperPositionMode', 'Auto', 'PaperUnits', 'Inches', 'PaperSize', [pos(3), pos(4)])
if save_figures == 1
    print(fig_count, 'Figures/Mean_of_intervals_for_asinh_GE_market.pdf', '-dpdf')
end
fig_count = fig_count + 1;

%% PL prices averaged over intervals + spread asinh
close all
data_polish = importdata('../Data/data_Poland_20170101_20201231.txt');

price_DA_PL = reshape(data_polish(:, 3), [24, size(data_polish(:, 3), 1) / 24])';

price_ID_PL = reshape(data_polish(:, 4), [24, size(data_polish(:, 4), 1) / 24])';

Thresholds = [7, 13, 18, 22];
Thresholds_1 = [22, 7, 13, 18];
Thresholds_2 = [6, 12, 17, 21];

data_sets = {asinh(zscore(price_DA_PL, [], 1)), asinh(zscore(price_ID_PL, [], 1)), asinh(zscore(price_DA_PL, [], 1)) - asinh(zscore(price_ID_PL, [], 1))};

figure(fig_count)
t = tiledlayout(3, 1,'TileSpacing','compact', 'Padding', 'none');
titles = {'Polish day-ahead asinh transformed prices', 'Polish balancing asinh transformed prices', 'Polish spread of asinh transformed prices'};
ylabels = {'$\widetilde{DA}$', '$\widetilde{BL}$', '$\widetilde{DA} - \widetilde{BL}$'};
alphas = [1, 0.8, 0.6, 0.4];
for s = 1:length(data_sets)
    n_t = nexttile;
    for i = 1:length(Thresholds_1)
        if i == 1
            p = plot(days, mean([data_sets{s}(:, Thresholds_1(i):end), data_sets{s}(:, 1:Thresholds_2(i))], 2), 'DisplayName', [num2str(Thresholds_1(i)), '-', num2str(Thresholds_2(i))], LineWidth=0.6);
            hold on
        else
            p = plot(days, mean(data_sets{s}(:, Thresholds_1(i):Thresholds_2(i)), 2), 'DisplayName', [num2str(Thresholds_1(i)), '-', num2str(Thresholds_2(i))], LineWidth=0.6);
        end
        p.Color(4) = alphas(i);
        datetick('x','mm.yyyy')
    end
    if s == 1
        xline(n_t, days(373), '--','Validation window $\Longrightarrow$', 'LabelOrientation', 'horizontal', 'Interpreter', 'latex', 'Fontsize', 8,'HandleVisibility','off');
        xline(n_t, days(373), '--','$\Longleftarrow$ Calibration window', 'LabelOrientation', 'horizontal', 'Interpreter', 'latex', 'LabelHorizontalAlignment', 'left', 'Fontsize', 8,'HandleVisibility','off');
    else
        xline(n_t, days(373), '--','HandleVisibility','off');
        xline(n_t, days(373), '--','HandleVisibility','off');
    end
    if s < length(data_sets)
        xticklabels(gca,{})
    end
    title(titles{s})
    ylabel(ylabels{s})
    hold off
end
lgd = legend('Orientation','horizontal','Box','off');
lgd.Layout.Tile = 'south';
% set width and save figure
set(fig_count, 'units', 'inches', 'pos', [0 0 Paper_width 1.2 * Paper_width])
pos = get(fig_count,'Position');
set(fig_count, 'PaperPositionMode', 'Auto', 'PaperUnits', 'Inches', 'PaperSize', [pos(3), pos(4)])
if save_figures == 1
    print(fig_count, 'Figures/Mean_of_intervals_for_asinh_PL_market.pdf', '-dpdf')
end
fig_count = fig_count + 1;
