close all
addpath('Functions')

if strcmp( getenv('username'), 'andrz')
    addpath('../../../Data_MATLAB_Drive/Data/')
else
    addpath('D:\Nauka\artykuy\GARCH\Matlab\Data_MATLAB_Drive')
end

% defining parameters
country = 'PL';

Thresholds = [7, 13, 18, 22];

alpha = 0.05;

measures = {'std', 'sharpe', 'semistd_down', 'semistd_up',...
            'std_and_profit', 'mean_and_std', 'mean'};

transaction_side = 'buyer';

model_DA = 'GARCH';
model_ID = 'GARCH';

transformation_DA = 'asinh';
transformation_ID = 'asinh';

dist_DA = 'g';
dist_ID = 'g';

titles = {'GARCH', 'ARX'};

% empty vectors to collect data
Total_profit_DA = ones(length(measures), 1);
Total_profit_ID = ones(length(measures), 1);
Mean_profit_DA_std = ones(length(measures), 1);
Mean_profit_ID_std = ones(length(measures), 1);
Mean_wallet_std = ones(length(measures), 1);
profit_max_semistd = ones(24, 1);
profit_max_mean = ones(24, 1);
profit_max_semistd_ID = ones(24, 1);
profit_max_semistd_DA = ones(24, 1);
profit_max_mean_ID = ones(24, 1);
profit_max_mean_DA = ones(24, 1);

% loading samples and forecasted values
IS_country_DA = load([ model_DA, '_simulation_results_', country, '_DA_model_7_transformation_', transformation_DA, '_dynamic_choice_0_mc_0_dist_', dist_DA, '.mat'],...
    'samples', 'samples_no_garch', 'real_value', 'prediction');
IS_country_ID = load([ model_ID, '_simulation_results_', country, '_ID_model_7_transformation_', transformation_ID, '_dynamic_choice_0_mc_0_dist_', dist_ID, '.mat'],...
    'samples', 'samples_no_garch', 'real_value', 'prediction');

% calculating the mean of stds of real values in ID and DA markets
Mean_DA_std = mean(std(IS_country_DA.real_value, 0, 2));
Mean_ID_std = mean(std(IS_country_ID.real_value, 0, 2));

%% creating the figures
XDates = datetime(2017,1,1):datetime(2020,12,31);

q = zeros(length(measures), length(Thresholds));

for i = 1:length(measures)

    load(['Weights/Weights_', transaction_side, '_', country, '/', measures{i}, '_weights_', 'GARCH_GARCH', ...
        '_', transformation_DA, '_', ...
        transformation_ID, '_', dist_DA, '_', dist_ID, '.mat']);
    Weights_1 = Weights;

    load(['Weights/Weights_', transaction_side, '_', country, '/', measures{i}, '_weights_', 'ARX', ...
        '_', transformation_DA, '_', ...
        transformation_ID, '_', dist_DA, '_', dist_ID, '.mat']);
    Weights_2 = Weights_ARX;

    Weights_vec = {Weights_1, Weights_2};

    for w = 1:length(Weights_vec)
        Weights = Weights_vec{w};
        [profit_DA, total_profit_DA, hourly_std_profit_DA, wallet,...
            hourly_std_wallet, hourly_mean_wallet, daily_std_wallet,...
            daily_mean_wallet] = strategy_evaluation_metrics( ...
                IS_country_DA.real_value, IS_country_ID.real_value, Weights, transaction_side, "DA");
        [profit_ID, total_profit_ID, hourly_std_profit_ID, ~,...
            ~, ~, ~, ~] = strategy_evaluation_metrics( ...
                IS_country_DA.real_value, IS_country_ID.real_value, Weights, transaction_side, "ID");
        [profit_max, total_profit_max, ~, ~,...
            ~, ~, ~, ~] = strategy_evaluation_metrics( ...
                IS_country_DA.real_value, IS_country_ID.real_value, Weights, transaction_side, "max_ID_DA");

        for h = 1:24
            profit_max_semistd(h) = semistd0(profit_max(h, :), -1);
            profit_max_mean(h) = mean(profit_max(h, :));
        end

        figure(10 + 1)
        subplot(1, length(Weights_vec), w)
        plot(1:24, hourly_std_wallet)
        hold on
    
        figure(10 + 2)
        subplot(1, length(Weights_vec), w)
        plot(1:24, hourly_mean_wallet)
        hold on
      
        figure(10 + 3)
        subplot(1, length(Weights_vec), w)
        plot(1:24, profit_max_semistd)
        hold on
        
        figure(10 + 4)
        subplot(1, length(Weights_vec), w)
        plot(1:24, profit_max_mean)
        hold on

        for threshold = 2:length(Thresholds) + 1
            h_0 = Thresholds(threshold - 1);
            figure(20 + threshold)
            subplot(1, length(Weights_vec), w)
            if threshold == length(Thresholds) + 1
                h_1 = Thresholds(1) - 1;
                plot(mean([hourly_std_wallet(h_0:24); hourly_std_wallet(1:h_1)]), mean([hourly_mean_wallet(h_0:24); hourly_mean_wallet(1:h_1)]), '*', 'MarkerSize', 16)            
            else
                h_1 = Thresholds(threshold) - 1;
                plot(mean(hourly_std_wallet(h_0:h_1)), mean(hourly_mean_wallet(h_0:h_1)), '*', 'MarkerSize', 16)            
            end
            hold on  
        end
        
        for threshold = 2:length(Thresholds) + 1
            h_0 = Thresholds(threshold - 1);
            figure(20 + 2 * length(Thresholds) + threshold)
            subplot(1, length(Weights_vec), w)
            hold on
            if threshold == length(Thresholds) + 1
                h_1 = Thresholds(1) - 1;
                q_max=-mean(quantile(mean([mean(profit_max(h_0:24,:)), mean(profit_max(1:h_1, :))],1), 0.05));
                plot(q_max, mean([profit_max_mean(h_0:24); profit_max_mean(1:h_1)]), '*', 'MarkerSize', 16)
                
            else
                h_1 = Thresholds(threshold) - 1;
                q_max=-mean(quantile(mean(profit_max(h_0:h_1, :),1), 0.05));
                plot(q_max, mean(profit_max_mean(h_0:h_1)), '*', 'MarkerSize', 16)           
            end
            hold on
        end
        
        for threshold = 2:length(Thresholds) + 1
            h_0 = Thresholds(threshold - 1);
            figure(20 + 3 * length(Thresholds) + threshold) 
            subplot(1, length(Weights_vec), w)
            if threshold == length(Thresholds) + 1
                h_1 = Thresholds(1) - 1;
                plot(mean([profit_max_semistd(h_0:24); profit_max_semistd(1:h_1)]), mean([profit_max_mean(h_0:24); profit_max_mean(1:h_1)]), '*', 'MarkerSize', 16)            
            else
                h_1 = Thresholds(threshold) - 1;
                plot(mean(profit_max_semistd(h_0:h_1)), mean(profit_max_mean(h_0:h_1)), '*', 'MarkerSize', 16)            
            end
            hold on
            
        end
    end
end

shifts= [1.8, 1];
% adding static (with respect to strategy) elements to the plots
for w = 1:length(Weights_vec)
    DA_rv = IS_country_DA.real_value;
    ID_rv = IS_country_ID.real_value;
    
    if strcmp(transaction_side, "seller")
        ID_rv(ID_rv < DA_rv) = DA_rv(ID_rv < DA_rv);
    elseif strcmp(transaction_side, "buyer")
        ID_rv(ID_rv > DA_rv) = DA_rv(ID_rv > DA_rv);
    end
    
    diff_DA=IS_country_DA.real_value  - ID_rv;
    diff_ID=IS_country_ID.real_value  - ID_rv;
    
    if strcmp(transaction_side, "buyer")
      diff_DA=-diff_DA;
      diff_ID=-diff_ID;
    end
    
    for h = 1:24
        profit_max_semistd_ID(h) = semistd0(diff_DA(h,:), -1);
        profit_max_semistd_DA(h) = semistd0(diff_ID(h,:), -1);
        profit_max_mean_ID(h) = mean(diff_DA(h,:));
        profit_max_mean_DA(h) = mean(diff_ID(h,:));
    end
    
    figure(10 + 1)
    sph = subplot(1, length(Weights_vec), w);
    sph.Position = sph.Position + 0.1*[shifts(w) 0 -1 0]; % reduce width and shift to the right, leave all else unchanged
    hold on
    plot(1:24, std(IS_country_DA.real_value, 0, 2),'-*')
    hold on
    plot(1:24, std(IS_country_ID.real_value, 0, 2),'-*')
    title(['Std of every strategy hourly ', titles{w}]) % interpreter none to display underscores as underscores
    if w == 2
        Lgnd = legend([measures, 'DA', 'ID'], 'Interpreter', 'none');
        Lgnd.Position(1) = 0.01;
        Lgnd.Position(2) = 0.4;
    elseif w == 1
        fig = gcf;
        fig.Position(3) = fig.Position(3) + 200;
    end
    figure(10 + 2)
    sph = subplot(1, length(Weights_vec), w);
    sph.Position = sph.Position + 0.1*[shifts(w) 0 -1 0]; % reduce width and shift to the right, leave all else unchanged
    hold on
    plot(1:24, mean(IS_country_DA.real_value, 2),'-*')
    hold on
    plot(1:24, mean(IS_country_ID.real_value, 2),'-*')
    title(['Mean of every strategy hourly ', titles{w}]) % interpreter none to display underscores as underscores
    if w == 2
        Lgnd = legend([measures, 'DA', 'ID'], 'Interpreter', 'none');
        Lgnd.Position(1) = 0.01;
        Lgnd.Position(2) = 0.4;
    elseif w == 1
        fig = gcf;
        fig.Position(3) = fig.Position(3) + 200;
    end
    
    figure(10 + 3)
    sph = subplot(1, length(Weights_vec), w);
    sph.Position = sph.Position + 0.1*[shifts(w) 0 -1 0]; % reduce width and shift to the right, leave all else unchanged
    hold on
    plot(1:24, profit_max_semistd_DA,'-*')
    hold on
    plot(1:24, profit_max_semistd_ID,'-*')
    title(['Semistd vs Max of every strategy hourly ', titles{w}]) % interpreter none to display underscores as underscores
    if w == 2
        Lgnd = legend([measures, 'DA', 'ID'], 'Interpreter', 'none');
        Lgnd.Position(1) = 0.01;
        Lgnd.Position(2) = 0.4;
    elseif w == 1
        fig = gcf;
        fig.Position(3) = fig.Position(3) + 200;
    end
    
    figure(10 + 4)
    sph = subplot(1, length(Weights_vec), w);
    sph.Position = sph.Position + 0.1*[shifts(w) 0 -1 0]; % reduce width and shift to the right, leave all else unchanged
    hold on
    plot(1:24, profit_max_mean_DA,'-*')
    hold on
    plot(1:24, profit_max_mean_ID,'-*')
    title(['Mean vs Max of every strategy hourly ', titles{w}]) % interpreter none to display underscores as underscores
    if w == 2
        Lgnd = legend([measures, 'DA', 'ID'], 'Interpreter', 'none');
        Lgnd.Position(1) = 0.01;
        Lgnd.Position(2) = 0.4;
    elseif w == 1
        fig = gcf;
        fig.Position(3) = fig.Position(3) + 200;
    end
    
    for threshold = 2:length(Thresholds) + 1
        h_0 = Thresholds(threshold - 1);
        figure(20 + threshold)
        sph = subplot(1, length(Weights_vec), w);
        sph.Position = sph.Position + 0.1*[shifts(w) 0 -1 0]; % reduce width and shift to the right, leave all else unchanged
        if threshold == length(Thresholds) + 1
            h_1 = Thresholds(1) - 1;
            plot(mean([std(IS_country_DA.real_value(h_0:24, :), 0, 2); std(IS_country_DA.real_value(1:h_1, :), 0, 2)]), mean([mean(IS_country_DA.real_value(h_0:24, :), 2); mean(IS_country_DA.real_value(1:h_1, :), 2)]), '.', 'MarkerSize', 16)
            plot(mean([std(IS_country_ID.real_value(h_0:24, :), 0, 2); std(IS_country_ID.real_value(1:h_1, :), 0, 2)]), mean([mean(IS_country_ID.real_value(h_0:24, :), 2); mean(IS_country_ID.real_value(1:h_1, :), 2)]), '.', 'MarkerSize', 16)
        else
            h_1 = Thresholds(threshold) - 1;
            plot(mean(std(IS_country_DA.real_value(h_0:h_1, :), 0, 2)), mean(mean(IS_country_DA.real_value(h_0:h_1, :), 2)), '.', 'MarkerSize', 16)
            plot(mean(std(IS_country_ID.real_value(h_0:h_1, :), 0, 2)), mean(mean(IS_country_ID.real_value(h_0:h_1, :), 2)), '.', 'MarkerSize', 16)
        end 
        if w == 2
            Lgnd = legend([measures, 'DA', 'ID'], 'Interpreter', 'none');
            Lgnd.Position(1) = 0.01;
            Lgnd.Position(2) = 0.4;
        elseif w == 1
            fig = gcf;
            fig.Position(3) = fig.Position(3) + 200;
        end
        title(['Mean over hours from ', num2str(h_0), ' to ', num2str(h_1), ' ', titles{w}])    
        xlabel("Std")
        ylabel("Mean")
        grid minor
    end
    
    for threshold = 2:length(Thresholds) + 1
        h_0 = Thresholds(threshold - 1);
        figure(20 + 3 * length(Thresholds) + threshold)
        sph = subplot(1, length(Weights_vec), w);
        sph.Position = sph.Position + 0.1*[shifts(w) 0 -1 0]; % reduce width and shift to the right, leave all else unchanged
        if threshold == length(Thresholds) + 1
            h_1 = Thresholds(1) - 1;
            plot(mean([profit_max_semistd_DA(h_0:24); profit_max_semistd_DA(1:h_1)]), mean([profit_max_mean_DA(h_0:24); profit_max_mean_DA(1:h_1)]), '.', 'MarkerSize', 16)
            plot(mean([profit_max_semistd_ID(h_0:24); profit_max_semistd_ID(1:h_1)]), mean([profit_max_mean_ID(h_0:24); profit_max_mean_ID(1:h_1)]), '.', 'MarkerSize', 16)
        else
            h_1 = Thresholds(threshold) - 1;
            plot(mean(profit_max_semistd_DA(h_0:h_1)), mean(profit_max_mean_DA(h_0:h_1)), '.', 'MarkerSize', 16)
            plot(mean(profit_max_semistd_ID(h_0:h_1)), mean(profit_max_mean_ID(h_0:h_1)), '.', 'MarkerSize', 16)
        end 
            title(['Over hours from ', num2str(h_0), ' to ', num2str(h_1), ' ', titles{w}])     
            if w == 2
                Lgnd = legend([measures, 'DA', 'ID'], 'Interpreter', 'none');
                Lgnd.Position(1) = 0.01;
                Lgnd.Position(2) = 0.4;
            elseif w == 1
                fig = gcf;
                fig.Position(3) = fig.Position(3) + 200;
            end
            xlabel("Semistd vs max")
            ylabel("Mean diff from Max")
            grid minor
    end
    
    for threshold = 2:length(Thresholds) + 1
        h_0 = Thresholds(threshold - 1);
        figure(20 + 2 * length(Thresholds) + threshold)
        sph = subplot(1, length(Weights_vec), w);
        sph.Position = sph.Position + 0.1*[shifts(w) 0 -1 0]; % reduce width and shift to the right, leave all else unchanged
        hold on
        if threshold == length(Thresholds) + 1
            h_1 = Thresholds(1) - 1;
                    q_IS_country_DA=-mean(quantile(mean([mean(diff_DA(h_0:24,:)); mean(diff_DA(1:h_1, :))],1), 0.05));
                    q_IS_country_ID=-mean(quantile(mean([mean(diff_ID(h_0:24,:)); mean(diff_ID(1:h_1, :))],1), 0.05));
                    plot(q_IS_country_DA,  mean([profit_max_mean_DA(h_0:24); profit_max_mean_DA(1:h_1)]), '.', 'MarkerSize', 16)
                    plot(q_IS_country_ID, mean([profit_max_mean_ID(h_0:24); profit_max_mean_ID(1:h_1)]), '.', 'MarkerSize', 16)
        else
            h_1 = Thresholds(threshold) - 1;
                    q_IS_country_DA=-(quantile(mean(diff_DA(h_0:h_1, :),1), 0.05));
                    q_IS_country_ID=-(quantile(mean(diff_ID(h_0:h_1, :),1), 0.05));
                    plot(q_IS_country_DA, mean(profit_max_mean_DA(h_0:h_1)), '.', 'MarkerSize', 16)
                    plot(q_IS_country_ID, mean(profit_max_mean_ID(h_0:h_1)), '.', 'MarkerSize', 16)      
        end 
            title(['Mean over hours from ', num2str(h_0), ' to ', num2str(h_1), ' ', titles{w}])     
            if w == 2
                Lgnd = legend([measures, 'DA', 'ID'], 'Interpreter', 'none');
                Lgnd.Position(1) = 0.01;
                Lgnd.Position(2) = 0.4;
            elseif w == 1
                fig = gcf;
                fig.Position(3) = fig.Position(3) + 200;
            end
            xlabel("VaR vs max")
            ylabel("Mean diff from Max")
            grid minor
    end
end




