clear all
addpath('Functions')

redo_simulations = 0; % 0 - only tables are generated once again 
write_in_sheets = 1; % 1 - only part of tables are generated, i.e. those that are not from combination of models
Thresholds = [7, 13, 18, 22];
var_alpha = 0.05;
PS_quantiles = 0.01:0.01:0.99;

% possible parameters
if write_in_sheets
    Models_DA = {'GARCH'};
    Models_ID = {'GARCH'};
else
    Models_DA = {'GJRGARCH', 'GARCH'};
    Models_ID = {'GJRGARCH', 'GARCH'};
end

country = 'PL';

Transaction_sides = {'buyer', 'seller'};
Transformations_DA = {''};
Transformations_ID = {'asinh'};
dist_DA = 't';
dist_ID = 't';
option = 1;
measures = {'std', 'semistd_down', 'semistd_up', 'std_and_profit', 'sharpe', 'mean_and_std', 'mean'};
Measures = measures';
Total_profit_DA = ones(length(measures), 1);
Total_profit_ID = ones(length(measures), 1);
Total_profit_max = ones(length(measures), 1);
Mean_profit_DA_std = ones(length(measures), 1);
Mean_profit_ID_std = ones(length(measures), 1);
Mean_max_profit_semistd = ones(length(measures), 1);
Mean_wallet_std = ones(length(measures), 1);
VaR_profit_DA = ones(length(measures), 1);
VaR_profit_ID = ones(length(measures), 1);
VaR_profit_max = ones(length(measures), 1);
VaR_wallet = ones(length(measures), 1);
max_profit_semistd = ones(24, 1);
Joint_probability_forecasting_PS = ones(4, length(measures), 1088, length(PS_quantiles));
Wallet_values = ones(4, length(measures), 1088);
Results_from_every_combination = ones(length(Transaction_sides), ...
    length(Models_DA), length(Models_ID), length(Transformations_DA), ...
    length(Transformations_ID), length(Thresholds), length(measures), 11);
%% GARCH simulation
for t_s = 1:length(Transaction_sides)
    transaction_side = Transaction_sides{t_s};
    for m_DA = 1:length(Models_DA)
        model_DA = Models_DA{m_DA};
        for m_ID = 1:length(Models_ID)
            model_ID = Models_ID{m_ID};
            for trans_DA = 1:length(Transformations_DA)
                transformation_DA = Transformations_DA{trans_DA};
                for trans_ID = 1:length(Transformations_ID)
                    transformation_ID = Transformations_ID{trans_ID};
                    if write_in_sheets
                        if strcmp(country, 'GE')
                            if ~strcmp(transformation_DA, transformation_ID)
                                continue
                            end
                        elseif strcmp(country, 'PL')
                            if strcmp(transformation_DA, '') && strcmp(transformation_ID, 'asinh')
                                % pass
                            elseif ~strcmp(transformation_DA, transformation_ID)
                                continue
                            end
                        end
                    end
                    IS_country_DA = load(['../../../Data_MATLAB_Drive/Data/', model_DA, '_simulation_results_', country, '_DA_model_7_transformation_', transformation_DA, '_dynamic_choice_0_mc_0_dist_', dist_DA, '.mat'],...
                        'samples', 'samples_no_garch', 'real_value', 'prediction');
                    IS_country_ID = load(['../../../Data_MATLAB_Drive/Data/', model_ID, '_simulation_results_', country, '_ID_model_7_transformation_', transformation_ID, '_dynamic_choice_0_mc_0_dist_', dist_ID, '.mat'],...
                        'samples', 'samples_no_garch', 'real_value', 'prediction');
                    if redo_simulations
                        Weights = ones(size(squeeze(IS_country_DA.samples(:, :, :)), 3), 24);
                        Weights_ARX = ones(size(squeeze(IS_country_DA.samples_no_garch(:, :, :)), 3), 24);
                        % find and save weights for all hours and measures
                        tic;
                        for i = 1:length(measures)
                            parfor hour = 1:24
                                side = transaction_side
                                final_weights = [];
                                scenarios_DA = squeeze(IS_country_DA.samples(hour, :, :)); %#ok<PFBNS> 
                                scenarios_ID = squeeze(IS_country_ID.samples(hour, :, :)); %#ok<PFBNS>

                                if strcmp(country, 'PL')
                                    scenarios_ID(scenarios_ID < 70) = 70; % constraints on PL RB market
                                    scenarios_ID(scenarios_ID > 1500) = 1500;
                                end

                                if strcmp(side, 'seller')
                                    final_weights = optimal_scenarios_measure(scenarios_ID, scenarios_DA, IS_country_DA.prediction(hour, :), option, measures{i}, side); %#ok<PFBNS> 
                                elseif strcmp(side, 'buyer')
                                    final_weights = optimal_scenarios_measure(-scenarios_ID, -scenarios_DA, -IS_country_DA.prediction(hour, :), option, measures{i}, side);
                                end
                                Weights(:, hour) = final_weights;
                            end
                            save(['Weights/Weights_', transaction_side, '_', country, '/', measures{i}, '_weights_', model_DA, '_', model_ID, '_', transformation_DA, '_', transformation_ID, '_', dist_DA, '_', dist_ID, '.mat'],...
                                    'Weights');
                        end
                        disp(['Duration of the whole simulation for ', transaction_side, ', ',...
                            model_DA, ', ', model_ID, ', ', transformation_DA, ', ',...
                            transformation_ID, ': ', num2str(toc)])
                    end
                    % evaluation of the results for GARCH
                    scenarios_DA = IS_country_DA.samples;
                    scenarios_ID = IS_country_ID.samples;

                    if strcmp(country, 'PL')
                        scenarios_ID(scenarios_ID < 70) = 70; % constraints on PL RB market
                        scenarios_ID(scenarios_ID > 1500) = 1500;
                    end
                    for threshold = 2:length(Thresholds) + 1
                        h_0 = Thresholds(threshold - 1);
                
                        if threshold == length(Thresholds) + 1
                            h_1 = Thresholds(1) - 1;
                            Mean_DA_std = mean([std(IS_country_DA.real_value(h_0:24, :), 0, 2); std(IS_country_DA.real_value(1:h_1, :), 0, 2)]);
                            Mean_ID_std = mean([std(IS_country_ID.real_value(h_0:24, :), 0, 2); std(IS_country_ID.real_value(1:h_1, :), 0, 2)]);
                            if strcmp(transaction_side, 'seller')
                                VaR_DA = quantile(mean([IS_country_DA.real_value(h_0:24, :); IS_country_DA.real_value(1:h_1, :)]), var_alpha);
                                VaR_ID = quantile(mean([IS_country_ID.real_value(h_0:24, :); IS_country_ID.real_value(1:h_1, :)]), var_alpha);
                            elseif strcmp(transaction_side, 'buyer')
                                VaR_DA = quantile(mean([IS_country_DA.real_value(h_0:24, :); IS_country_DA.real_value(1:h_1, :)]), 1 - var_alpha);
                                VaR_ID = quantile(mean([IS_country_ID.real_value(h_0:24, :); IS_country_ID.real_value(1:h_1, :)]), 1 - var_alpha);
                            end
                            for i = 1:length(measures)
                                load(['Weights/Weights_', transaction_side, '_', country, '/', measures{i}, '_weights_', model_DA, '_',...
                                    model_ID, '_', transformation_DA, '_',...
                                    transformation_ID, '_', dist_DA, '_', dist_ID, '.mat']);
                                [profit_DA, total_profit_DA, std_profit_DA, wallet, std_wallet] = strategy_evaluation_metrics(IS_country_DA.real_value, IS_country_ID.real_value, Weights, transaction_side, "DA");
                                [profit_ID, total_profit_ID, std_profit_ID, ~, ~] = strategy_evaluation_metrics(IS_country_DA.real_value, IS_country_ID.real_value, Weights, transaction_side, "ID");
                                [max_possible_profit, ~, ~, ~, ~] = strategy_evaluation_metrics(IS_country_DA.real_value, IS_country_ID.real_value, Weights, transaction_side, "max_ID_DA");
                                Total_profit_DA(i) = sum(sum([profit_DA(h_0:24, :); profit_DA(1:h_1, :)]));
                                Total_profit_ID(i) = sum(sum([profit_ID(h_0:24, :); profit_ID(1:h_1, :)]));
                                Total_profit_max(i) = sum(sum([max_possible_profit(h_0:24, :); max_possible_profit(1:h_1, :)]));
                                Mean_profit_DA_std(i) = mean([std_profit_DA(h_0:24); std_profit_DA(1:h_1)]);
                                Mean_profit_ID_std(i) = mean([std_profit_ID(h_0:24); std_profit_ID(1:h_1)]);
                                for h = 1:24
                                    max_profit_semistd(h) = semistd0(max_possible_profit(h, :), -1);
                                end
                                Mean_max_profit_semistd(i) = mean([max_profit_semistd(h_0:24); max_profit_semistd(1:h_1)]);
                                Mean_wallet_std(i) = mean([std_wallet(h_0:24); std_wallet(1:h_1)]);
                                if strcmp(transaction_side, 'seller')
                                    operational_var_alpha = var_alpha;
                                elseif strcmp(transaction_side, 'buyer')
                                    operational_var_alpha = 1 - var_alpha;
                                end
                                VaR_profit_DA(i) = quantile(mean([profit_DA(h_0:24, :); profit_DA(1:h_1, :)]), operational_var_alpha);
                                VaR_profit_ID(i) = quantile(mean([profit_ID(h_0:24, :); profit_ID(1:h_1, :)]), operational_var_alpha);
                                VaR_profit_max(i) = quantile(mean([max_possible_profit(h_0:24, :); max_possible_profit(1:h_1, :)]), operational_var_alpha);
                                VaR_wallet(i) = quantile(mean([wallet(h_0:24, :); wallet(1:h_1, :)]), operational_var_alpha);
                                Joint_probability_forecasting_PS(threshold - 1, i, :, :) = ...
                                    strategy_pinball_score(mean([wallet(h_0:24, :); wallet(1:h_1, :)]), scenarios_DA, scenarios_ID, Weights, [h_0:24, 1:h_1], PS_quantiles);
                                Wallet_values(threshold - 1, i, :) = mean([wallet(h_0:24, :); wallet(1:h_1, :)]);
                            end
                        else
                            h_1 = Thresholds(threshold) - 1;
                            Mean_DA_std = mean(std(IS_country_DA.real_value(h_0:h_1, :), 0, 2));
                            Mean_ID_std = mean(std(IS_country_ID.real_value(h_0:h_1, :), 0, 2));
                            if strcmp(transaction_side, 'seller')
                                VaR_DA = quantile(mean(IS_country_DA.real_value(h_0:h_1, :)), var_alpha);
                                VaR_ID = quantile(mean(IS_country_ID.real_value(h_0:h_1, :)), var_alpha);
                            elseif strcmp(transaction_side, 'buyer')
                                VaR_DA = quantile(mean(IS_country_DA.real_value(h_0:h_1, :)), 1 - var_alpha);
                                VaR_ID = quantile(mean(IS_country_ID.real_value(h_0:h_1, :)), 1 - var_alpha);
                            end
                            for i = 1:length(measures)
                                load(['Weights/Weights_', transaction_side, '_', country, '/', measures{i}, '_weights_', model_DA, '_',...
                                    model_ID, '_', transformation_DA, '_',...
                                    transformation_ID, '_', dist_DA, '_', dist_ID, '.mat']);
                                [profit_DA, total_profit_DA, std_profit_DA, wallet, std_wallet] = strategy_evaluation_metrics(IS_country_DA.real_value, IS_country_ID.real_value, Weights, transaction_side, "DA");
                                [profit_ID, total_profit_ID, std_profit_ID, ~, ~] = strategy_evaluation_metrics(IS_country_DA.real_value, IS_country_ID.real_value, Weights, transaction_side, "ID");
                                [max_possible_profit, ~, ~, ~, ~] = strategy_evaluation_metrics(IS_country_DA.real_value, IS_country_ID.real_value, Weights, transaction_side, "max_ID_DA");
                                Total_profit_DA(i) = sum(sum(profit_DA(h_0:h_1, :)));
                                Total_profit_ID(i) = sum(sum(profit_ID(h_0:h_1, :)));
                                Total_profit_max(i) = sum(sum(max_possible_profit(h_0:h_1, :)));
                                Mean_profit_DA_std(i) = mean(std_profit_DA(h_0:h_1));
                                Mean_profit_ID_std(i) = mean(std_profit_ID(h_0:h_1));
                                for h = 1:24
                                    max_profit_semistd(h) = semistd0(max_possible_profit(h, :), -1);
                                end
                                Mean_max_profit_semistd(i) = mean(max_profit_semistd(h_0:h_1));
                                Mean_wallet_std(i) = mean(std_wallet(h_0:h_1));
                                if strcmp(transaction_side, 'seller')
                                    operational_var_alpha = var_alpha;
                                elseif strcmp(transaction_side, 'buyer')
                                    operational_var_alpha = 1 - var_alpha;
                                end
                                VaR_profit_DA(i) = quantile(mean(profit_DA(h_0:h_1, :)), operational_var_alpha);
                                VaR_profit_ID(i) = quantile(mean(profit_ID(h_0:h_1, :)), operational_var_alpha);
                                VaR_profit_max(i) = quantile(mean(max_possible_profit(h_0:h_1, :)), operational_var_alpha);
                                VaR_wallet(i) = quantile(mean(wallet(h_0:h_1, :)), operational_var_alpha);
                                Joint_probability_forecasting_PS(threshold - 1, i, :, :) = ...
                                    strategy_pinball_score(mean(wallet(h_0:h_1, :)), scenarios_DA, scenarios_ID, Weights, h_0:h_1, PS_quantiles);
                                Wallet_values(threshold - 1, i, :) = mean(wallet(h_0:h_1, :));
                            end
                        end
                        save(['Joint_probability_forecasting_PS/', country, '_PS_', transaction_side, '_', model_DA, '_', model_ID, '_', transformation_DA, '_', transformation_ID], ...
                                'Joint_probability_forecasting_PS')
                        save(['Wallet_values/', country, '_wallet_values_', transaction_side, '_', model_DA, '_', model_ID, '_', transformation_DA, '_', transformation_ID], ...
                                'Wallet_values')
                        % create table for GARCH
                        Results = [round(Total_profit_DA, 3), ...
                            round(Total_profit_ID, 3), ...
                            round(Total_profit_max, 3), ...
                            round(Mean_max_profit_semistd, 3), ...
                            VaR_profit_max, ...
                            round(Mean_wallet_std, 3), ...
                            round(repmat(Mean_DA_std, [length(measures), 1]), 3), ...
                            round(repmat(Mean_ID_std, [length(measures), 1]), 3), ...
                            round(VaR_wallet, 3), ...
                            round(repmat(VaR_DA, [length(measures), 1]), 3), ...
                            round(repmat(VaR_ID, [length(measures), 1]), 3)];
                        Results_from_every_combination(t_s, m_DA, m_ID, trans_DA, trans_ID, threshold - 1, :, :) = Results;
                        Matrice_of_results = [Measures, string(Results)];
%                         [~, ind_min] = min(Results(:, 1:end - 2));
%                         [~, ind_max] = max(Results(:, 1:end - 2));
%                         [~, indices] = sort(Results(:, 1:end - 2), 'descend');
%                         second_max = indices(2, :);
%                         Matrice_of_results(sub2ind(size(Matrice_of_results), ind_min, 2:size(Results, 2) - 1)) = Matrice_of_results(sub2ind(size(Matrice_of_results), ind_min, 2:size(Results, 2) - 1)) + ' (min)';
%                         Matrice_of_results(sub2ind(size(Matrice_of_results), ind_max, 2:size(Results, 2) - 1)) = Matrice_of_results(sub2ind(size(Matrice_of_results), ind_max, 2:size(Results, 2) - 1)) + ' (max)';
%                         Matrice_of_results(sub2ind(size(Matrice_of_results), second_max, 2:size(Results, 2) - 1)) = Matrice_of_results(sub2ind(size(Matrice_of_results), second_max, 2:size(Results, 2) - 1)) + ' (2nd max)';
                        table_GARCH = array2table(Matrice_of_results(:,[1,2,3,7]), ...
                            'VariableNames', {'measures', ...
                            'total_profit_DA', ...
                            'total_profit_ID', ...
                            'mean_of_stds_of_wallet'});
                        if write_in_sheets
                            if strcmp(country, 'PL')
                                writetable(table_GARCH, [ ...
                                    'Tables_short/Tables_', transaction_side, '_', country, '/', transaction_side, ...
                                    num2str(h_0), '_to_', num2str(h_1), '.xlsx'], 'Sheet', [model_DA, transformation_DA, transformation_ID]);
                            elseif strcmp(country, 'GE')
                                writetable(table_GARCH, [ ...
                                    'Tables_short/Tables_', transaction_side, '_', country, '/', transaction_side, ...
                                    num2str(h_0), '_to_', num2str(h_1), '.xlsx'], 'Sheet', [model_DA, transformation_DA]);
                            end
                        else
                            writetable(table_GARCH, [ ...
                                'Tables/Tables_', transaction_side, '_', country, '/', transaction_side, '_strategy_results_', ...
                                model_DA, '_', model_ID, '_', transformation_DA, '_', ...
                                transformation_ID, '_', dist_DA, '_', dist_ID, '_', ...
                                num2str(h_0), '_', num2str(h_1), '.xlsx']);
                        end
                    end
                end
            end
        end
    end
end

save(['Results_from_every_combination_GARCH_', country, '.mat'], 'Results_from_every_combination')

%% ARX simulation
Results_from_every_combination = ones(length(Transaction_sides), ...
    length(Transformations_DA), length(Transformations_ID), ...
    length(Thresholds), length(measures), 11);

for t_s = 1:length(Transaction_sides)
    transaction_side = Transaction_sides{t_s};
    for trans_DA = 1:length(Transformations_DA)
        transformation_DA = Transformations_DA{trans_DA};
        for trans_ID = 1:length(Transformations_ID)
            transformation_ID = Transformations_ID{trans_ID};
            if write_in_sheets
                if strcmp(country, 'GE')
                    if ~strcmp(transformation_DA, transformation_ID)
                        continue
                    end
                elseif strcmp(country, 'PL')
                    if strcmp(transformation_DA, '') && strcmp(transformation_ID, 'asinh')
                        % pass
                    elseif ~strcmp(transformation_DA, transformation_ID)
                        continue
                    end
                end
            end
            IS_country_DA = load([ Models_DA{1}, '_simulation_results_', country, '_DA_model_7_transformation_', transformation_DA, '_dynamic_choice_0_mc_0_dist_', dist_DA, '.mat'],...
                'samples', 'samples_no_garch', 'real_value', 'prediction');
            IS_country_ID = load([ Models_ID{1}, '_simulation_results_', country, '_ID_model_7_transformation_', transformation_ID, '_dynamic_choice_0_mc_0_dist_', dist_ID, '.mat'],...
                'samples', 'samples_no_garch', 'real_value', 'prediction');
            if redo_simulations
                % find and save weights for all hours and measures
                tic;
                for i = 1:length(measures)
                    parfor hour = 1:24
                        final_weights_ARX = [];
                        side = transaction_side
                        scenarios_RDN_ARX = squeeze(IS_country_DA.samples_no_garch(hour, :, :)); %#ok<PFBNS> 
                        scenarios_RB_ARX = squeeze(IS_country_ID.samples_no_garch(hour, :, :)); %#ok<PFBNS>

                        if strcmp(country, 'PL')
                            scenarios_RB_ARX(scenarios_RB_ARX < 70) = 70; % constraints on PL RB market
                            scenarios_RB_ARX(scenarios_RB_ARX > 1500) = 1500;
                        end
                                
                        if strcmp(side, 'seller')
                            final_weights_ARX = optimal_scenarios_measure(scenarios_RB_ARX, scenarios_RDN_ARX, IS_country_DA.prediction(hour, :), option, measures{i}, side); %#ok<PFBNS>
                        elseif strcmp(side, 'buyer')
                            final_weights_ARX = optimal_scenarios_measure(-scenarios_RB_ARX, -scenarios_RDN_ARX, -IS_country_DA.prediction(hour, :), option, measures{i}, side); 
                        end
                        Weights_ARX(:, hour) = final_weights_ARX;
                    end
                    save(['Weights/Weights_', transaction_side, '_', country, '/', measures{i}, '_weights_ARX_', transformation_DA, '_', transformation_ID, '_', dist_DA, '_', dist_ID, '.mat'],...
                            'Weights_ARX');
                end
                disp(['Duration of the whole simulation for ', transaction_side, ', ',...
                            transformation_DA, ', ',...
                            transformation_ID, ': ', num2str(toc)])
            end
            % evaluation of the results for ARX
            for threshold = 2:length(Thresholds) + 1
                h_0 = Thresholds(threshold - 1);
        
                if threshold == length(Thresholds) + 1
                    h_1 = Thresholds(1) - 1;
                    Mean_DA_std = mean([std(IS_country_DA.real_value(h_0:24, :), 0, 2); std(IS_country_DA.real_value(1:h_1, :), 0, 2)]);
                    Mean_ID_std = mean([std(IS_country_ID.real_value(h_0:24, :), 0, 2); std(IS_country_ID.real_value(1:h_1, :), 0, 2)]);
                    if strcmp(transaction_side, 'seller')
                        VaR_DA = quantile(mean([IS_country_DA.real_value(h_0:24, :); IS_country_DA.real_value(1:h_1, :)]), var_alpha);
                        VaR_ID = quantile(mean([IS_country_ID.real_value(h_0:24, :); IS_country_ID.real_value(1:h_1, :)]), var_alpha);
                    elseif strcmp(transaction_side, 'buyer')
                        VaR_DA = quantile(mean([IS_country_DA.real_value(h_0:24, :); IS_country_DA.real_value(1:h_1, :)]), 1 - var_alpha);
                        VaR_ID = quantile(mean([IS_country_ID.real_value(h_0:24, :); IS_country_ID.real_value(1:h_1, :)]), 1 - var_alpha);
                    end
                    for i = 1:length(measures)
                        load(['Weights/Weights_', transaction_side, '_', country, '/', measures{i}, '_weights_ARX_', transformation_DA, '_',...
                            transformation_ID, '_', dist_DA, '_', dist_ID, '.mat']);
                        [profit_DA, total_profit_DA, std_profit_DA, wallet, std_wallet] = strategy_evaluation_metrics(IS_country_DA.real_value, IS_country_ID.real_value, Weights_ARX, transaction_side, "DA");
                        [profit_ID, total_profit_ID, std_profit_ID, ~, ~] = strategy_evaluation_metrics(IS_country_DA.real_value, IS_country_ID.real_value, Weights_ARX, transaction_side, "ID");
                        [max_possible_profit, ~, ~, ~, ~] = strategy_evaluation_metrics(IS_country_DA.real_value, IS_country_ID.real_value, Weights_ARX, transaction_side, "max_ID_DA");
                        Total_profit_DA(i) = sum(sum([profit_DA(h_0:24, :); profit_DA(1:h_1, :)]));
                        Total_profit_ID(i) = sum(sum([profit_ID(h_0:24, :); profit_ID(1:h_1, :)]));
                        Total_profit_max(i) = sum(sum([max_possible_profit(h_0:24, :); max_possible_profit(1:h_1, :)]));
                        Mean_profit_DA_std(i) = mean([std_profit_DA(h_0:24); std_profit_DA(1:h_1)]);
                        Mean_profit_ID_std(i) = mean([std_profit_ID(h_0:24); std_profit_ID(1:h_1)]);
                        for h = 1:24
                            max_profit_semistd(h) = semistd0(max_possible_profit(h, :), -1);
                        end
                        Mean_max_profit_semistd(i) = mean([max_profit_semistd(h_0:24); max_profit_semistd(1:h_1)]);
                        Mean_wallet_std(i) = mean([std_wallet(h_0:24); std_wallet(1:h_1)]);
                        if strcmp(transaction_side, 'seller')
                            operational_var_alpha = var_alpha;
                        elseif strcmp(transaction_side, 'buyer')
                            operational_var_alpha = 1 - var_alpha;
                        end
                        VaR_profit_DA(i) = mean(quantile([profit_DA(h_0:24, :); profit_DA(1:h_1, :)], operational_var_alpha, 2));
                        VaR_profit_ID(i) = mean(quantile([profit_ID(h_0:24, :); profit_ID(1:h_1, :)], operational_var_alpha, 2));
                        VaR_profit_max(i) = quantile(mean([max_possible_profit(h_0:24, :); max_possible_profit(1:h_1, :)]), operational_var_alpha);
                        VaR_wallet(i) = quantile(mean([wallet(h_0:24, :); wallet(1:h_1, :)]), operational_var_alpha);
                        Joint_probability_forecasting_PS(threshold - 1, i, :, :) = ...
                                    strategy_pinball_score(mean([wallet(h_0:24, :); wallet(1:h_1, :)]), scenarios_DA, scenarios_ID, Weights, [h_0:24, 1:h_1], PS_quantiles);
                        Wallet_values(threshold - 1, i, :) = mean([wallet(h_0:24, :); wallet(1:h_1, :)]);
                    end
                else
                    h_1 = Thresholds(threshold) - 1;
                    Mean_DA_std = mean(std(IS_country_DA.real_value(h_0:h_1, :), 0, 2));
                    Mean_ID_std = mean(std(IS_country_ID.real_value(h_0:h_1, :), 0, 2));
                    if strcmp(transaction_side, 'seller')
                        VaR_DA = quantile(mean(IS_country_DA.real_value(h_0:h_1, :)), var_alpha);
                        VaR_ID = quantile(mean(IS_country_ID.real_value(h_0:h_1, :)), var_alpha);
                    elseif strcmp(transaction_side, 'buyer')
                        VaR_DA = quantile(mean(IS_country_DA.real_value(h_0:h_1, :)), 1 - var_alpha);
                        VaR_ID = quantile(mean(IS_country_ID.real_value(h_0:h_1, :)), 1 - var_alpha);
                    end
                    for i = 1:length(measures)
                        load(['Weights/Weights_', transaction_side, '_', country, '/', measures{i}, '_weights_ARX_', transformation_DA, '_',...
                            transformation_ID, '_', dist_DA, '_', dist_ID, '.mat']);
                        [profit_DA, total_profit_DA, std_profit_DA, wallet, std_wallet] = strategy_evaluation_metrics(IS_country_DA.real_value, IS_country_ID.real_value, Weights_ARX, transaction_side, "DA");
                        [profit_ID, total_profit_ID, std_profit_ID, ~, ~] = strategy_evaluation_metrics(IS_country_DA.real_value, IS_country_ID.real_value, Weights_ARX, transaction_side, "ID");
                        [max_possible_profit, ~, ~, ~, ~] = strategy_evaluation_metrics(IS_country_DA.real_value, IS_country_ID.real_value, Weights_ARX, transaction_side, "max_ID_DA");
                        Total_profit_DA(i) = sum(sum(profit_DA(h_0:h_1, :)));
                        Total_profit_ID(i) = sum(sum(profit_ID(h_0:h_1, :)));
                        Total_profit_max(i) = sum(sum(max_possible_profit(h_0:h_1, :)));
                        Mean_profit_DA_std(i) = mean(std_profit_DA(h_0:h_1));
                        Mean_profit_ID_std(i) = mean(std_profit_ID(h_0:h_1));
                        for h = 1:24
                            max_profit_semistd(h) = semistd0(max_possible_profit(h, :), -1);
                        end
                        Mean_max_profit_semistd(i) = mean(max_profit_semistd(h_0:h_1));
                        Mean_wallet_std(i) = mean(std_wallet(h_0:h_1));
                        if strcmp(transaction_side, 'seller')
                            operational_var_alpha = var_alpha;
                        elseif strcmp(transaction_side, 'buyer')
                            operational_var_alpha = 1 - var_alpha;
                        end
                        VaR_profit_DA(i) = mean(quantile(profit_DA(h_0:h_1, :), operational_var_alpha, 2));
                        VaR_profit_ID(i) = mean(quantile(profit_ID(h_0:h_1, :), operational_var_alpha, 2));
                        VaR_profit_max(i) = quantile(mean(max_possible_profit(h_0:h_1, :)), operational_var_alpha);
                        VaR_wallet(i) = quantile(mean(wallet(h_0:h_1, :)), operational_var_alpha);
                        Joint_probability_forecasting_PS(threshold - 1, i, :, :) = ...
                                    strategy_pinball_score(mean(wallet(h_0:h_1, :)), scenarios_DA, scenarios_ID, Weights, h_0:h_1, PS_quantiles);
                        Wallet_values(threshold - 1, i, :) = mean(wallet(h_0:h_1, :));
                    end
                end
                save(['Joint_probability_forecasting_PS/', country, '_PS_', transaction_side, '_ARX_', transformation_DA, '_', transformation_ID], ...
                        'Joint_probability_forecasting_PS')
                save(['Wallet_values/', country, '_wallet_values_', transaction_side, '_ARX_', transformation_DA, '_', transformation_ID], ...
                        'Wallet_values')
                % create table for ARX
                Results = [round(Total_profit_DA, 3), ...
                            round(Total_profit_ID, 3), ...
                            round(Total_profit_max, 3), ...
                            round(Mean_max_profit_semistd, 3), ...
                            VaR_profit_max, ...
                            round(Mean_wallet_std, 3), ...
                            round(repmat(Mean_DA_std, [length(measures), 1]), 3), ...
                            round(repmat(Mean_ID_std, [length(measures), 1]), 3), ...
                            round(VaR_wallet, 3), ...
                            round(repmat(VaR_DA, [length(measures), 1]), 3), ...
                            round(repmat(VaR_ID, [length(measures), 1]), 3)];
                Results_from_every_combination(t_s, trans_DA, trans_ID, threshold - 1, :, :) = Results;
                Matrice_of_results = [Measures, string(Results)];
%                 [~, ind_min] = min(Results(:, 1:end - 2));
%                 [~, ind_max] = max(Results(:, 1:end - 2));
%                 [~, indices] = sort(Results(:, 1:end - 2), 'descend');
%                 second_max = indices(2, :);
%                 Matrice_of_results(sub2ind(size(Matrice_of_results), ind_min, 2:size(Results, 2) - 1)) = Matrice_of_results(sub2ind(size(Matrice_of_results), ind_min, 2:size(Results, 2) - 1)) + ' (min)';
%                 Matrice_of_results(sub2ind(size(Matrice_of_results), ind_max, 2:size(Results, 2) - 1)) = Matrice_of_results(sub2ind(size(Matrice_of_results), ind_max, 2:size(Results, 2) - 1)) + ' (max)';
%                 Matrice_of_results(sub2ind(size(Matrice_of_results), second_max, 2:size(Results, 2) - 1)) = Matrice_of_results(sub2ind(size(Matrice_of_results), second_max, 2:size(Results, 2) - 1)) + ' (2nd max)';
                table_ARX = array2table(Matrice_of_results(:,[1,2,3,7]), ...
                            'VariableNames', {'measures', ...
                            'total_profit_DA', ...
                            'total_profit_ID', ...
                            'mean_of_stds_of_wallet'});
                if write_in_sheets
                    if strcmp(country, 'PL')
                        writetable(table_ARX, [ ...
                            'Tables_short/Tables_', transaction_side, '_', country, '/', transaction_side, ...
                            num2str(h_0), '_to_', num2str(h_1), '.xlsx'], 'Sheet', ['ARX', transformation_DA, transformation_ID]);
                    elseif strcmp(country, 'GE')
                        writetable(table_ARX, [ ...
                            'Tables_short/Tables_', transaction_side, '_', country, '/', transaction_side, ...
                            num2str(h_0), '_to_', num2str(h_1), '.xlsx'], 'Sheet', ['ARX', transformation_DA]);
                    end
                else
                    writetable(table_ARX, [ ...
                        'Tables/Tables_', transaction_side, '_', country, '/', transaction_side, ...
                        '_strategy_results_ARX_', transformation_DA, '_', transformation_ID, '_', dist_DA, '_', dist_ID, '_', ...
                        num2str(h_0), '_', num2str(h_1), '.xlsx']);
                end
            end
        end
    end
end

save(['Results_from_every_combination_ARX_', country ,'.mat'], 'Results_from_every_combination')
